(function() {
	'use strict';
	var TILE_WIDTH = 2, HALF_TILE = 5, FULL_TILE = 10, CRITICAL = 4, WARNING = 3, GOOD = 2, INFO = 1;

//	, CATEGORIES = [
//			1, // "Configuration",
//			4, // "Client",
//			3, // "Capacity Planning",
//			6, // "DeDuplication",
//			7, // "Indexing and Search",
//			8, // "Usage Profile",
//			5, // "Job",
//			15 ]; // "Scale Statistics" ];

	var TILE_ID_IDX = 0, TILE_NAME_IDX = 1, TILE_GUID_IDX = 2, TILE_PROP_IDX = 3, TILE_STATUS_IDX = 6;

	// poly fill for the phanton js issue reported here
	// https://github.com/angular/angular.js/issues/13794
	if (typeof Function.prototype.bind != 'function') {
		Function.prototype.bind = function bind(obj) {
			var args = Array.prototype.slice.call(arguments, 1), self = this, nop = function() {
			}, bound = function() {
				return self.apply(this instanceof nop ? this : (obj || {}),
						args.concat(Array.prototype.slice.call(arguments)));
			};
			nop.prototype = this.prototype || {};
			bound.prototype = new nop();
			return bound;
		};
	}

	angular.module("healthDashboardApp").controller("healthDashboardCtrl",[
							"$scope",
							"$location",
							"$window",
							"$timeout",
							"$uibModal",
							"healthService",
							"healthUtilService",
							"reportService",
							function($scope, $location, $window, $timeout,$uibModal,
									healthService, healthUtilService, reportService) {
								cvUtil.mask("#content");
								$scope.isAlarm = cvUtil.getParameter("isAlarm");
								$scope.hasHealthData = false;
								$scope.gridsterOpts = {};
								$scope.hiddenTiles = [];
								$scope.hiddenTilesMap = [];
								$scope.gridsterOpts.rowHeight = 50;
								$scope.showHiddenTileMenu = false;
								$scope.tileCounter = 0;
								$scope.tileInRow = 4;
								$scope.tilePositionIndex = 0;
								$scope.cacheId = healthService.generateUUID();
								$scope.gridsterOpts.columns = 8;

								$scope.allTiles = [];
								$scope.categories = [];

								var browserExp = new RegExp(
										'Android|webOS|iPhone|iPad|'
												+ 'BlackBerry|Windows Phone|'
												+ 'Opera Mini|IEMobile|Mobile',
										'i');
								if (browserExp.test(navigator.userAgent)) {
									$scope.gridsterOpts.draggable = {
										enabled : false
									};
								} else {
									$scope.gridsterOpts.draggable = {
										enabled : true
									};
								}
								$scope.gridsterOpts.resizable = {
									enabled : false
								}

								$scope.gridsterOpts.draggable = false;
								$scope.gridsterOpts.mobileBreakPoint = 300;
								if ((window.navigator.appName === 'Microsoft Internet Explorer' && window.navigator.userAgent
										.indexOf('MSIE') !== -1)
										|| (window.navigator.appName === 'Netscape' && window.navigator.userAgent
												.indexOf('Trident') !== -1)) {
									$scope.gridsterOpts.draggable.enabled = false;
								}

								$scope.sorting =  "category";
								$scope.filterStatus =  -1; //all
								$scope.aggregate = {
									criticalCount : 0,
									warningCount : 0,
									goodCount : 0,
									infoCount : 0
								}

								$scope.tileGroups = [];


								if (cvUtil.getParameter("exportType")) {
									 var date = new Date();
									 angular.element("#generatedOnSpan").html(moment.utc(date.valueOf() - (date.getTimezoneOffset() * 60 * 1000)).format("MMM DD, YYYY hh:mm A"));
									 angular.element("#generatedOnText").show();
								}


								$scope.initialize = function() {

									$(document).attr("title", cvUtil.cvLocalize("Health_Report"));
									$scope.attachEventHandlers();

									healthService.getCategories().then(function(response) {
										if (response.data && response.data.errorMsg) {
											cvUtil.errorToast(response.data.errorMsg);
										} else {
											if(response.data.records) {
												angular.forEach(response.data.records, function(row) {
													$scope.categories.push(
															{categoryId: row[2],
															 categoryName: row[3],
															 priority: row[1]});
												})

												//add the dummy category unknown
												$scope.categories.push(
														{categoryId: -1,
														 categoryName: "Unknown",
														 priority: 999});
											}
											$scope.loadTiles();
										}
									},function() {
										cvUtil.errorToast("Failed to load tiles categories.");
									});
								}


								$scope.loadTiles = function () {
									healthService.getHealth().then(function(response) {
										var errorMsg = response.data.data.errorMsg || response.data.data.error;
										if (response.data.data.summaryData) {
											setSummaryData(response.data.data.summaryData);
										}
										if (errorMsg) {
											angular.element("#infoErrorMessage").html(errorMsg);
											$scope.hasHealthData = false;
											$scope.loaded = true;
											angular.element("#reportButton").addClass("disabled");
											cvUtil.unmask("#content");
										} else {
											var resp = JSON.parse( response.data.data.tileData);
											var paramGrouping = cvUtil.getParameter("grouping","");
											var paramFilter = cvUtil.getParameter("severity", "");

											if (paramGrouping) {
												$scope.sorting = paramGrouping;
											}else if(response.data.data.grouping) {
												$scope.sorting = response.data.data.grouping;
											}

											if (paramFilter) {
												$scope.filterStatus = getStatusAsInt(paramFilter);
											} else if (response.data.data.statusFilter) {
												$scope.filterStatus = getStatusAsInt(response.data.data.statusFilter);
											}

											if (!resp.records || resp.records.length ==0) {
												$scope.hasHealthData = false;
												$scope.loaded = true;
												return;
											}

											$scope.allTiles = resp.records;

											if (cvUtil.getParameter("exportType")) {
												$scope.gridsterOpts.columns = 4;
												$scope.tileInRow = 2;
												$("body").addClass('exportfriendly');
											} else if ($window.innerWidth > 1760) {
												$scope.gridsterOpts.columns = 8;
												$scope.tileInRow = 4;
											} else if ($window.innerWidth <= 950) {
												$scope.gridsterOpts.columns = 2;
												$scope.tileInRow = 1;
											} else if ($window.innerWidth <= 1400) {
												$scope.gridsterOpts.columns = 4;
												$scope.tileInRow = 2;
											} else if ($window.innerWidth <= 1760) {
												$scope.gridsterOpts.columns = 6;
												$scope.tileInRow = 3;
											}

											//check the urlParams and getAlarmDetails.
								            if(!$scope.requestParms){
								                $scope.requestParms = cvUtil.getUrlParams();
								            }

								            if($scope.requestParms.alarmId && $scope.requestParms.alarmId > 0){
								                var param = {};
								                param.alarmId = $scope.requestParms.alarmId;
								                var alarmDetails ;
								                reportService.getAlarmDetails(param).success(function (data) {
								                        if (data && data.length) {
								                            alarmDetails = data[0];
								                            if(alarmDetails.error){
								                                var errorMessage = "Unable to find alert details for alarmId ["+alarmDetails.alarmId+"], Error Message :" + alarmDetails.error;
								                                console.error(errorMessage);
								                                $scope.alarmDetails = {};
								                                $scope.alarmDetails.error = errorMessage;
								                                cvUtil.errorToast(alarmDetails.error);
								                            }else{
								                                var alarmURL = decodeURIComponent(alarmDetails.alarmURL);
								                                var tileId = cvUtil.getParameterByName("tileId", alarmURL);
								                                var componentName = cvUtil.getParameterByName("componentName", alarmURL);
								                                var tileStatus =  cvUtil.getParameterByName("tileStatus", alarmURL);
								                                alarmDetails.tileId = tileId;
								                                alarmDetails.componentName = componentName;
								                                alarmDetails.editCriteriaSet = true;
								                                alarmDetails.tileStatus = tileStatus;
								                                alarmDetails.tileIdFound = false;
								                                $scope.alarmDetails = alarmDetails;

								                                if(angular.isDefined($scope.alarmDetails.tileId) && $scope.alarmDetails.tileId == 0){
								                                	$scope.alarmDetails.tileIdFound = true;
																	$scope.alarmDetails.tileName = "Any Tile";
								                                }else{
								                                	$.each($scope.allTiles,function(index, tile) {
																		var status = 0;
																		tile.tileId = tile[TILE_ID_IDX];
																		tile.tileName = tile[TILE_NAME_IDX];

																		//check if tileId still exist in the loaded Tiles, for edit criteria option.
																		if($scope.alarmDetails && $scope.alarmDetails.editCriteriaSet && !$scope.alarmDetails.tileIdFound && !!$scope.alarmDetails.tileId){
																			if(tile.tileId == $scope.alarmDetails.tileId){
																				$scope.alarmDetails.tileIdFound = true;
																				$scope.alarmDetails.tileName = tile.tileName;
																			}
																		}
																	});
								                                }


																if($scope.alarmDetails.tileIdFound){
																		$scope.openAlarmsDialog($scope.alarmDetails.tileId, $scope.alarmDetails.tileName, true);
																	}else{
																		  var errorMessage = "Unable to find tile details for alarmId ["+param.alarmId+"]";
													                      console.error(errorMessage);
													                      $scope.alarmDetails = {};
													                      $scope.alarmDetails.error = errorMessage;
													                      cvUtil.errorToast(errorMessage);
																	}
																}
								                        }

								                    }).error(function(e){
								                        var errorMessage = "Unable to find alert details for alarmId ["+param.alarmId+"], Error Message :" + e;
								                        console.error(errorMessage);
								                        $scope.alarmDetails = {};
								                        $scope.alarmDetails.error = errorMessage;
								                        cvUtil.errorToast(e);
								                    });
								            }


											$.each($scope.allTiles,function(index, tile) {
												var props = {};
												var status = 0;
												if (tile[TILE_PROP_IDX]) {
													props = JSON.parse(tile[TILE_PROP_IDX]);
												}
												tile.tileProperty = props;
												tile.tileId = tile[TILE_ID_IDX];
												tile.tileName = tile[TILE_NAME_IDX];
												tile.tileStatus =  tile[TILE_STATUS_IDX];

												status = tile[TILE_STATUS_IDX] || 1;
												if (status ==4) {
													$scope.aggregate.criticalCount++;
												} else if (status == 3) {
													$scope.aggregate.warningCount++;
												} else if (status ==2) {
													$scope.aggregate.goodCount++;
												} else if (status ==1) {
													$scope.aggregate.infoCount++;
												}
											});

											$scope.populateHiddenTileList(response.data.data.hiddenTiles);
											$scope.drawTiles();
											$scope.hasHealthData = true;
											$timeout(function() {
												cvUtil.unmask("#content");
											}, 1000);

										}
										},function() {
											console.log("Failed to load health report.");
										});
								}
								$scope.populateHiddenTileList = function(hTiles) {
									$scope.hiddenTiles = hTiles;
									$scope.hiddenTilesMap = [];
									$.each($scope.allTiles, function(index,
											tile) {
										if ($scope.hiddenTiles
												.indexOf(tile.tileId) != -1) {
											$scope.hiddenTilesMap.push({
												tileId : tile.tileId,
												tileName : tile.tileName
											});
										}
									});
								}

								$scope.addToHiddenTileList = function(tileId,
										tileName) {
									$scope.hiddenTiles.push(tileId);
									$scope.hiddenTilesMap.push({
										tileId : tileId,
										tileName : tileName
									});
								}

								$scope.isCritical = function(tile) {
									return tile[TILE_STATUS_IDX] && tile[TILE_STATUS_IDX] == 4;
								}

								$scope.isWarning = function(tile) {
									return tile[TILE_STATUS_IDX] && tile[TILE_STATUS_IDX] == 3;
								}
								$scope.isGood = function(tile) {
									return tile[TILE_STATUS_IDX] && tile[TILE_STATUS_IDX] == 2;
								}

								$scope.isInfo = function(tile) {
									return tile[TILE_STATUS_IDX] && tile[TILE_STATUS_IDX] == 1;
								}

								$scope.getOutcome = function(outcome) {
									return outcome.value;
								}

								$scope.hasNoRemarks = function(tile) {
									if (tile.recommendations == undefined
											|| tile.recommendations.length === 0) {
										return true;
									}

									return false;
								}

								$scope.viewChanged = function(selection) {
									$scope.sorting = selection;
									updateURL(null, selection);
									healthService.setHealthGrouping(selection).then(function(resp) {
													},
													function() {
														console.log("Unable to update user preference with health grouping.");
														cvUtil.errorToast("Unable to update user preference with health grouping.");
													});
									$scope.drawTiles();
								}

								$scope.showAllTiles = function() {
									healthService.showAllTiles().then(function(resp) {
										$scope.filterStatus = -1;
										$scope.hiddenTiles = [];
										$scope.hiddenTilesMap = []


										healthService.setTileStatusFilter($scope.filterStatus).then(function(resp) {},
												function() {
													console.log("Unable to update user preference with health status filter.");
													cvUtil.errorToast("Unable to update user preference with health status filter.");
												});

										$scope.drawTiles();
									},function(error) {
										console.log("Failed to clear user preference for hidden tiles.");
										cvUtil.errorToast("Failed to clear user preference for hidden tiles.");
									});
								}

								$scope.showTile = function(tile) {
									healthService.showTile(tile.tileId).then(function(resp) {
										$scope.populateHiddenTileList(resp.data.data.hiddenTiles);
										$scope.drawTiles();
										console.log("Successfully removed tile from hidden list.");
									},function(error) {
										console.log("Failed to remove tile from hidden list: "+ tile.tileName);
										cvUtil.errorToast("Failed to remove tile from hidden list: "+ tile.tileName);
									});
								}

								$scope.updateTiles = function(filterStatus) {

									if (filterStatus == $scope.filterStatus && filterStatus != -1) {
										filterStatus = -1;
									}
									updateURL(filterStatus, null);
									$scope.filterStatus = filterStatus;
									if ($scope.sorting != "severity") {
										$scope.sorting = "severity";
										healthService.setHealthGrouping($scope.sorting).then(function(resp) {},
											function() {
												console.log("Unable to update user preference with health grouping.");
												cvUtil.errorToast("Unable to update user preference with health grouping.");
										});
									}
									healthService.setTileStatusFilter(filterStatus).then(function(resp) {},
											function() {
												console.log("Unable to update user preference with health status filter.");
												cvUtil.errorToast("Unable to update user preference with health status filter.");
										});

									$scope.drawTiles();
								}

								$scope.setTileIndex = function(tile) {
									tile.tileIndex = $scope.tileCounter++;
								}

								$scope.setTileDimension = function(tiles) {
									$.each($scope.allTiles, function(index,tile) {
										tile.tileProperty = {};
									});
								};

								function clone(obj) {
									if (null == obj || "object" != typeof obj) {
										return obj;
									}
									var copy = obj.constructor();
									for ( var attr in obj) {
										if (obj.hasOwnProperty(attr)) {
											copy[attr] = obj[attr];
										}
									}
									return copy;
								}

								$scope.attachEventHandlers = function() {
									// angular.element("body").on("click",
									// function() {
									// angular.element(".fixActionMenuDiv").hide();
									// });

									angular.element("#exportPpt").on("mouseover",function() {
												angular.element("#_exportppt").show();
											});

									angular.element("#exportPpt").on("mouseout",
											function() {
												angular.element("#_exportppt").hide();
											});

									angular.element(".exportatserver").on("click",function() {
											var type = this.id.substring("export".length);
											if (type) {
												type = type.toLowerCase();
											}
											$scope.saveReportAs(type, $scope.customerName);
									});

//									angular.element("#tilesPanel").on("click", ".fixItButton", function(event) {
//										$scope.runFixIt(event, angular.element(".fixActionMenu", angular.element(this).parent()).val());
//									});

									angular.element("#tilesPanel").on("click", ".fixItItem", function(event) {
										$scope.runFixIt(event, angular.element(this).text());
									});

									angular.element("#tilesPanel").on("change", ".fixActionMenu", function(event) {
										$scope.runFixIt(event,$(this).find("option:selected").text());
									});


									angular.element("#exportEmail").on("click",function() {
												$scope.exportEmail();
											});

									angular.element("#reportAlert").on("click", function() {
										$scope.openAlarmsDialog(0, "Any Tile");
									});

									reportsUtil.scheduleReport({exportFileNamePrefix : healthUtilService.getFilePrefix($scope.customerName),format : [ 'PDF' ]});

									if ($($(".vw-js-nav-trigger .vw-close")[0]).css("display") !== "none") {
										angular.element(".vw-js-nav-trigger").click();
										angular.element("#reportSideBar .sec-title").addClass("hidden");
									}

									angular.element($window).bind('resize',function() {
										var repaint = false;
										if ($window.innerWidth > 1760) {
											$scope.gridsterOpts.columns = 8;
											$scope.tileInRow = 4;
											repaint = true;
										} else if ($window.innerWidth <= 950) {
											$scope.gridsterOpts.columns = 2;
											$scope.tileInRow = 1;
											repaint = true;
										} else if ($window.innerWidth <= 1400) {
											$scope.gridsterOpts.columns = 4;
											$scope.tileInRow = 2;
											repaint = true;
										} else if ($window.innerWidth <= 1760) {
											$scope.gridsterOpts.columns = 6;
											$scope.tileInRow = 3;
											repaint = true;
										}
										if (repaint) {
											$scope.tilePositionIndex = 0;
											$.each($scope.tileGroups,function(key,value) {
												$scope.setGroupPosition(value);
											});
										}
									});
								}


								$scope.openAlarmsDialog = function(tileId, tileName, editAlert) {

									if(!editAlert){
							        	//if we open up alarm dialog after modifying the existing alarm opened from alert manager page, resetting the alarmDetails, to avoid any other issues.
							        	$scope.alarmDetails = undefined;
									}
									var title = "";
									if ($scope.customerName) {
										title = $scope.customerName + " - ";
									}

									if (tileId == 0) {
										title += localMsg.healthReport;
									} else {
										title += localMsg.healthParameter + " [" + tileName + "]";
									}

									$scope.customReport = {
										report : {
											customReportName : title,
											customReportId: tileId
										}
									};
									$scope.isTileAlert = true;
									$scope.conditions = [];
									$scope.component = {};
									$scope.component.id = "healthAlertTable";
									$scope.component.columns = [];
									$scope.component.columns.push($scope.getColumn("Tile Status",
											"Status",
											"Tile Status",
											"Integer",
											true,
											false));
									$scope.component.columns.push($scope.getColumn("Tile Name",
											"Name",
											"TileName",
											"String",
											true,
											false));
									$scope.component.columns.push($scope.getColumn("Tile Outcome",
											"Outcome",
											"TileOutcome",
											"String",
											true,
											false));
									$scope.component.columns.push($scope.getColumn("Tile Remarks",
											"Remarks",
											"TileRemarks",
											"String",
											true,
											false));

									var condition = {
										columnName : "TileStatus",
										operator : '',
										value : "4",
										valueOptions : [ {
											"id" : '=',
											"label" : "Equal To"
										} ],
										type : ''
									};

									$scope.conditions.push(condition);
									$scope.tileData = {};
									$scope.tileData.tileId = tileId;
									$scope.tileData.tileName =tileName;

									$uibModal.open({
										templateUrl : customReports.contextPath + "/reportsplus/alarms/alarmsModal.jsp",
										controller : 'alarmsCtrl',
										windowClass : 'reportsPlusAlarm',
										scope : $scope
									});
								};

								$scope.getColumn = function(dataField, displayName, id, type, visible, hidden) {
									var column = {
										dataField : dataField,
										displayName : displayName,
										id : id,
										type : type,
										visible : visible,
										hidden : hidden

									}

									return column;
								}

								$scope.translate = function(expression) {
									var retunVal = expression;
									if (typeof expression === "string" && expression.substring(0, 1) === "=") {
										try {
											retunVal = eval(expression.substring(1));
										} catch (exception) {
											console.error(exception.message)
										}
									} else if (typeof expression === "string" && expression.substring(0, 2) === ":=") {
										try {
											var expr = "(function() {" + expression.substring(2) + " }) ();";
											retunVal = eval(expr);
										} catch (exception) {
											console.error(exception.message)
										}
									}
									return retunVal;
								}

								$scope.runFixIt = function($event, fixName, fixId) {
									if (fixName == undefined) {
										return
									}

									if (fixName) {
										return openForm({
											'workflowName' : fixName
//											'workflowId' : fixId,
//											'inputs' : {
//												'Client' : 'expressserver',
//												'Agent' : 'SQL Server',
//												'Subclient' : 'default',
//												'subclientID' : '7'
//											}
										});
									}
								}

								$scope.exportEmail = function() {
									var filePrefix = healthUtilService
											.getFilePrefix($scope.customerName);
									var target = cvUtil.getContextPath()
											+ "/reports/";

									target += "newReportEmail.do?"
											+ cvUtil.createParamString(
													'filePrefix', filePrefix);
									target += "&source=health&format=PDF&";

									if (window.reportEnum) {
										target += cvUtil
												.createParamString(
														'reportEnum',
														window.reportEnum)
												+ "&";
									}
									if (window.reportUniqId) {
										target += cvUtil.createParamString(
												'reportUniqId',
												window.reportUniqId);
									}
									cvUtil.showModal({
										width : 380,
										height : 130,
										title : cvUtil
												.cvLocalize('Email_Report'),
										href : target,
										iframe : true
									});
									$(".modal").css('height', 140)
								}

						$scope.saveReportAs = function(exportType, customerName) {
							var subscriptionID = window.sessionStorage.getItem('export_subscriptionID');
							var filePrefix = healthUtilService.getFilePrefix(customerName);
							var url = document.URL;
							var char = "?";
							if (url.indexOf("?") >0) {
								char = "&";
							}
							url = url + char + "exportType=" + exportType;

									angular
											.element("#content")
											.mask(
													localMsg['WaitMsg']
															+ "&nbsp&nbsp",
													0,
													function() {
														return (exportajaxRequest && exportajaxRequest
																.abort());
													});

									if (exportType === "healthppt"
											|| exportType == "word"
											|| exportType == "valassessppt") {
										angular.element("#content").unmask();
										healthUtilService.exportX(exportType,
												customerName);
										return;
									}

									var saveAsUrl = cvUtil.getContextPath()
											+ '/reports/exp.do?orientation=portrait&exportType='
											+ exportType
											+ '&viewportWidth=1600&commUniId='
											+ cvUtil.getParameter("commUniId");
									var params = {
										url : url,
										mhtmlURL : "/survey/health",
										filePrefix : filePrefix,
										directDownload : false,
										type : 'report'
									};
							if (exportType == "pdf") {
								 angular.element("#content").unmask();
								params.subscriptionID = subscriptionID;
							}
							var exportajaxRequest = healthService.saveAs(saveAsUrl, params);

							exportajaxRequest.then(function(data) {
								if (data && exportType != "pdf") {
									var data1 = data.data;
									angular.element("#content").unmask();
									downloadReportFile(data1);
								}
								angular.element("#content").unmask();

													},
													function(error) {
														angular.element(
																"#content")
																.unmask();
														if (error != 'userAborted') {
															cvUtil
																	.errorToast(error);
														}
													});
								}

								$scope.drawTiles = function() {
									cvUtil.mask("content");
									if ($scope.sorting == "category") { // By category
										$scope.filterStatus = -1;
									}

									$location.search('severity',$scope.filterStatus);
									$location.search('grouping', $scope.sorting);
									$scope.loaded = false;
									$scope.tileGroups = [];


									// group the tiles
									if ($scope.sorting == "category") { // By Category
										var groups = {};
										$.each($scope.allTiles,function(index, tile) {
											if ($scope.hiddenTiles.indexOf(tile[TILE_ID_IDX]) != -1) {
												return true;
											}

											var props = tile.tileProperty;
											if (props && !props.category) {
												props.category = {id: -1, name: "Unknown"};

											}
											if (props) {
												var groupId = props.category.id;
												if ($scope.filterStatus === -1 || tile[TILE_STATUS_IDX] ===$scope.filterStatus) {
													if (groups[groupId] == undefined) {
														groups[groupId] = {
																groupName : props.category.name,
																groupId : props.category.id,
																tiles : []
														};
													}
													tile.groupId = groupId;
													groups[groupId].tiles.push(clone(tile));
												}
											}
										});

										// when viewing by category, display
										// tiles according to CATEGORIES order
										$.each($scope.categories,function(key,category) {
											var tempGroup = $scope.getGroup(category.categoryId,groups);
											if (tempGroup) {
												var tileGroup = {};
													tileGroup.name = tempGroup.groupName;
													tileGroup.id = tempGroup.groupId;
													tileGroup.tiles = tempGroup.tiles;
													$scope.tileGroups.push(tileGroup);
											}
										});
									} else { // by status
										var grpCritical = [], grpWarning = [], grpGood = [], grpInfo = [];

										$.each($scope.allTiles,function(index, tile) {
											if ($scope.hiddenTiles.indexOf(tile.tileId) != -1) {
												return true;
											}
											if ($scope.filterStatus === -1 || tile[TILE_STATUS_IDX] === $scope.filterStatus) {
												if (tile[TILE_STATUS_IDX] == CRITICAL) {
													grpCritical.push(clone(tile));
												} else if (tile[TILE_STATUS_IDX]  == WARNING) {
													grpWarning.push(clone(tile));
												} else if (tile[TILE_STATUS_IDX]  == GOOD) {
													grpGood.push(clone(tile));
												} else if (tile[TILE_STATUS_IDX]  == INFO) {
													grpInfo.push(clone(tile));
												}

												tile.groupName = getStatusDisplayName(tile[TILE_STATUS_IDX]);
											}

										});

										if (grpCritical.length > 0) {
											$scope.tileGroups.push({
														id : CRITICAL,
														name : getStatusDisplayName(CRITICAL),
														tiles : grpCritical
													});
											updateTileGroupId(grpCritical, CRITICAL);
										}

										if (grpWarning.length > 0) {
											$scope.tileGroups
													.push({
														id : WARNING,
														name : getStatusDisplayName(WARNING),
														tiles : grpWarning
													});
											updateTileGroupId(grpWarning,
													WARNING);
										}

										if (grpGood.length > 0) {
											$scope.tileGroups
													.push({
														id : GOOD,
														name : getStatusDisplayName(GOOD),
														tiles : grpGood
													});
											updateTileGroupId(grpGood, GOOD);
										}

										if (grpInfo.length > 0) {
											$scope.tileGroups
													.push({
														id : INFO,
														name : getStatusDisplayName(INFO),
														tiles : grpInfo
													});
											updateTileGroupId(grpInfo, INFO);
										}

									}

									$scope.tilePositionIndex = 0;
									$.each($scope.tileGroups, function(key,value) {
										$scope.setGroupPosition(value);
									});

									$scope.loaded = true;
									cvUtil.unmask("#content");
								}

								$scope.getGroup = function(groupId, groups) {
									var group = undefined;
									$.each(groups, function(key, value) {
										if (value.groupId === groupId) {
											group = value;
											return false;
										}
									});

									return group;
								}

								$scope.setGroupPosition = function(group) {
									var halfTileLoc = {}, curRow = 0, curCol = 0;
									var halfTileExist = false;
									var tileCtr = 0;
									var halfTiles = [];
									var fullTiles = [];
									var tileInRow = $scope.tileInRow;
									var singleRow = false;

									$.each(group.tiles,function(index, curTile) {
										curTile.sizeX = TILE_WIDTH;
										var tHeight = FULL_TILE;
										if (curTile.tileProperty&& curTile.tileProperty.height) {
											tHeight = curTile.tileProperty.height;
										}
										if (tHeight === 'HALF') {
											curTile.sizeY = HALF_TILE;
											halfTiles.push(curTile);
										} else {curTile.sizeY = FULL_TILE;
											fullTiles.push(curTile);
										}
									});

									// plot full tiles first
									$.each(fullTiles, function(index, curTile) {
										curTile.row = curRow;
										curTile.col = curCol;
										tileCtr++;
										curCol += TILE_WIDTH;
										if (tileCtr == tileInRow) {
											curRow += FULL_TILE;
											curCol = 0;
											tileCtr = 0;
										}
										curTile.tilePositionIndex = $scope.tilePositionIndex++;
									});

									// plot all halftiles if less than four and
									// no full
									if ((fullTiles.length == 0 || tileCtr == 0)
											&& halfTiles.length <= tileInRow) {
										$.each(halfTiles, function(index, curTile) {
											curTile.row = curRow;
											curTile.col = curCol;
											curCol += TILE_WIDTH;
											curTile.tilePositionIndex = $scope.tilePositionIndex++;
										});
									} else {
										$.each(halfTiles, function(index, curTile) {
											if (halfTileExist) {
												curTile.row = halfTileLoc.row + HALF_TILE;
												curTile.col = halfTileLoc.col;
												halfTileExist = false;
												tileCtr++;
											} else {
												curTile.row = curRow;
												curTile.col = curCol;
												halfTileLoc.row = curRow;
												halfTileLoc.col = curCol;
												halfTileExist = true;
												curCol += TILE_WIDTH;
											}

											curTile.tilePositionIndex = $scope.tilePositionIndex++;
											if (tileCtr == tileInRow) {
												curRow += FULL_TILE;
												curCol = 0;
												tileCtr = 0;
											}
										});
									}
								}

								$scope.updateTilePositions = function(groupId, tileId) {
									var group = [];

									if ($scope.sorting == "severity") {
										group = $scope.tileGroups[0];
									}

									$.each($scope.tileGroups, function(index,
											val) {
										if (val.id === groupId) {
											group = val;
											return false;
										}
									})

									$.each(group.tiles, function(index, tile) {
										if (tile.tileId == tileId) {
											group.tiles.splice(index, 1);
											return false;
										}
									});

									$scope.setGroupPosition(group);

								}

								function updateTileGroupId(tiles, groupId) {
									$.each(tiles, function(index, tile) {
										tile.groupId = groupId;
									});
								}

								function getStatusDisplayName(status) {
									if (status == CRITICAL) {
										return localMsg.criticalStatus;
									} else if (status == WARNING) {
										return localMsg.warningStatus;
									} else if (status == GOOD) {
										return localMsg.goodStatus;
									} else {
										return localMsg.infoStatus;
									}
								}

								function updateURL(severity, grouping) {
									var commUniId = cvUtil.getParameter("commUniId", 0);
									var ccGroupId = cvUtil.getParameter("ccGroupId", 0);
									var companyId = cvUtil.getParameter("companyId", 0);
									var currentSeverity = cvUtil.getParameter("severity", "");
									var currentGrouping = cvUtil.getParameter("grouping", "");
									var params = [];
									var urlParams = "";

									if (commUniId) {
										params.push("commUniId=" + commUniId);
									}
									if (ccGroupId) {
										params.push("ccGroupId=" + ccGroupId);
									}
									if (companyId) {
										params.push("companyId=" + companyId);
									}

									if (severity) {
										params.push("severity=" + severity);
									}else if (currentSeverity) {
										params.push("severity=" + currentSeverity);
									}

									if (grouping) {
										params.push("grouping=" + grouping);
									}else if (currentGrouping) {
										params.push("grouping=" + currentGrouping);
									}

									History.replaceState("severity", document.title, "?" + params.join("&"));
								}

								function getStatusAsInt(status) {
									if (isNaN(parseInt(status))) {
										if (status== 'critical') {
											return CRITICAL;
										} else if (status == 'warning') {
											return WARNING;
										} else  {
											return GOOD;
										}

									} else {
										return parseInt(status);
									}
								}

								// header is not angular based
								function setSummaryData(summaryData) {
									var customerName = summaryData.Customer ? summaryData.Customer.value : "";
									var commcellId = summaryData.CommCellID ? summaryData.CommCellID.value : "";
									var time = summaryData.Time ? summaryData.Time.value : "";
									var guid = summaryData.CommServeGUID ? summaryData.CommServeGUID.value : "";
									var monitoredGroup = summaryData.MonitoredGroup ? summaryData.MonitoredGroup.value : "";
									var groupName = summaryData.GroupName ? summaryData.GroupName.value : undefined;
									$scope.customerName = customerName;

									var reportName = cvUtil.cvLocalize("Health_Report");
									if (customerName) {
										$(document).attr("title",cvUtil.cvLocalize("Health_Report_title", customerName));
										angular.element('#welcomeMsg').html(cvUtil.cvLocalize("Health_Report_title", customerName));
										angular.element('.reportsLink').text(customerName + ' ('+ cvUtil.cvLocalize("Reports")+ ')');
									} else {
										$(document).attr("title", reportName);
										angular.element('#welcomeMsg').html(reportName);
										angular.element('.reportsLink').text(cvUtil.cvLocalize("Reports"));
									}
									cvUtil.logToReportsUsage(reportName);

									if (summaryData.enableDoc) {
										angular.element('#exportWord').show();
										angular.element('#exportPpt').show();
									} else {
										angular.element('#exportWord').hide();
										angular.element('#exportPpt').hide();
									}

									cvUtil.GUIAuditReportLog("Health Report",customerName);

									if (commcellId != undefined && $.trim(commcellId).length != 0) {
										angular.element("#metaCCId").html(cvUtil.cvLocalize("commcell.info.label.CommCell_ID")+ ': ' +
													'<span class="mItem-val">' +decimalToHex(commcellId,2).toUpperCase()+ '</span>');
									}

									if (time != undefined && $.trim(time).length != 0) {
										angular.element('#metaAsOf').html(cvUtil.cvLocalize("commcell.info.label.As_of")+ ': ' +
													'<span class="mItem-val">' + time+ '</span>');
									}

									if (guid != undefined && $.trim(guid).length != 0) {
										guid = registrationCode(guid);
										angular.element('#GUID').html(cvUtil.cvLocalize("commcell.info.label.Registration_Code")+ ': ' +
													'<span class="mItem-val">' + guid + '</span>');
									}

									if (monitoredGroup != undefined && $.trim(monitoredGroup).length != 0) {
										angular.element('#metaClientGrp').html(cvUtil.cvLocalize("audit.label.Client_Group") + ': ' +
													'<span class="mItem-val">' + monitoredGroup + '</span>');
									}

									if (groupName) {
										angular.element('.commcellsLink').text(groupName+ ' ('+ cvUtil.cvLocalize("CommCells")+ ')');
									}
								}

							} ]);
}());