(function() {
	'use strict';
	var TILE_WIDTH = 2, HALF_TILE = 4, FULL_TILE = 8, CRITICAL = 4, WARNING = 3, GOOD = 2, INFO = 1, CATEGORIES = [ 1, //"Configuration",
	4, //"Client",
	3, //"Capacity Planning",
	6, //"DeDuplication",
	7, //"Indexing and Search",
	8, //"Usage Profile",
	5, //"Job",
	15 ]; //"Scale Statistics" ];

	//poly fill for the phanton js issue reported here  https://github.com/angular/angular.js/issues/13794
	if (typeof Function.prototype.bind != 'function') {
		Function.prototype.bind = function bind(obj) {
			var args = Array.prototype.slice.call(arguments, 1), self = this, nop = function() {
			}, bound = function() {
				return self.apply(this instanceof nop ? this : (obj || {}), args.concat(Array.prototype.slice
						.call(arguments)));
			};
			nop.prototype = this.prototype || {};
			bound.prototype = new nop();
			return bound;
		};
	}

	angular
			.module("healthDashboardApp")
			.controller("healthDashboardCtrl",
					[
							"$scope",
							"$location",
							"$window",
							"$timeout",
							"healthService",
							"healthUtilService",
							function($scope, $location, $window, $timeout, healthService, healthUtilService) {
								$scope.hasHealthData = false;
								$scope.gridsterOpts = {};
								$scope.hiddenTiles = [];
								$scope.hiddenTilesMap = [];
								$scope.gridsterOpts.rowHeight = 50;
								$scope.showHiddenTileMenu = false;
								$scope.tileCounter = 0;
								$scope.tileInRow = 4;
								$scope.tilePositionIndex = 0;

								$scope.gridsterOpts.columns = 8;
								$scope.gridsterOpts.draggable = {
									enabled : false
								};
								$scope.gridsterOpts.resizable = {
									enabled : false
								}

								//								var browserExp = new RegExp('Android|webOS|iPhone|iPad|' + 'BlackBerry|Windows Phone|'
								//										+ 'Opera Mini|IEMobile|Mobile', 'i');
								//								if (browserExp.test(navigator.userAgent)) {
								//									$scope.gridsterOpts.draggable = {
								//										enabled : false
								//									};
								//								} else {
								//									$scope.gridsterOpts.draggable = {
								//										enabled : true
								//									};
								//								}

								$scope.gridsterOpts.mobileBreakPoint = 300;
								if ((window.navigator.appName === 'Microsoft Internet Explorer' && window.navigator.userAgent
										.indexOf('MSIE') !== -1) ||
										(window.navigator.appName === 'Netscape' && window.navigator.userAgent
												.indexOf('Trident') !== -1)) {
									$scope.gridsterOpts.draggable.enabled = false;
								}

								$scope.sorting = "category";
								$scope.filterStatus = -1;
								$scope.aggregate = {
									criticalCount : 0,
									warningCount : 0,
									goodCount : 0,
									infoCount : 0
								}

								$scope.tileGroups = [];
								$scope.allTiles = [];

								$scope.initialize = function() {
									$scope.attachEventHandlers();
									var exportType = cvUtil.getParameter('exportType', '');
									if (exportType.toLowerCase() == 'csv') {
										$scope.exportCSV();
									}

									$scope.loadTiles();
								}

								$scope.exportCSV = function() {
									var params = {};
									params.isCSVExport = true;
									healthService.getHealth(params).then(function() {
										console.log('CSV File downloaded successfully');
										window.csvFileDownloaded = true;
									}, function() {
										console.log('CSV File downloaded failed');
										window.csvFileDownloaded = true;
									});
								}

								$scope.loadTiles = function() {
									cvUtil.mask("#healthContent", cvUtil.cvLocalize("Loading"));
									healthService.getHealth().then(function(response) {
										var errorMsg = response.data.data.errorMsg || response.data.data.error;
										if (response.data.data.summaryData) {
											setSummaryData(response.data.data.summaryData);
										}

										if (response.data.data.paramData) { //old style alert
											angular.element("body").addClass("exportfriendly");
											angular.element("#container").addClass("healthAlert");
										}

										if (errorMsg) {
											console.log("ERROR received: " + errorMsg);
											angular.element("#infoErrorMessage").html(errorMsg);
											$scope.hasHealthData = false;
											cvUtil.unmask("#healthContent");
										} else if (response.data.data.paramData) { //old style alert
											console.log("Creating table for alert");
											$scope.drawTable(response.data.data.paramData);
											cvUtil.unmask("#healthContent");
										} else {
											var respData = response.data.data.healthData;
											$scope.aggregate.criticalCount = respData.critical;
											$scope.aggregate.warningCount = respData.warning;
											$scope.aggregate.goodCount = respData.good;
											$scope.aggregate.infoCount = respData.info;
											$scope.allTiles = respData.tiles;

											if (response.data.data.threshold) {
												//												$scope.paramsTable = response.data.data.threshold;
												healthUtilService.setParamsTable(response.data.data.threshold);
											}
											//											$scope.setTileDimension($scope.allTiles);

											var urlParams = $location.search();
											var localCommUniId = undefined;
											if (urlParams) {
												if (!urlParams.commUniId) {
													localCommUniId = response.data.data.commUniId;
												}
												if (urlParams.grouping) {
													$scope.sorting = urlParams.grouping;
												}

												if (urlParams.severity) {
													$scope.filterStatus = getStatusAsInt(urlParams.severity);
												}

											}

											updateURL($scope.filterStatus, $scope.sorting, localCommUniId);

											if (cvUtil.getParameter("exportType")) {
												$scope.gridsterOpts.columns = 6;
												$scope.tileInRow = 3;
												angular.element("body").addClass('exportfriendly');
											} else if ($window.innerWidth > 1760) {
												$scope.gridsterOpts.columns = 8;
												$scope.tileInRow = 4;
											} else if ($window.innerWidth <= 950) {
												$scope.gridsterOpts.columns = 2;
												$scope.tileInRow = 1;
											} else if ($window.innerWidth <= 1400) {
												$scope.gridsterOpts.columns = 4;
												$scope.tileInRow = 2;
											} else if ($window.innerWidth <= 1760) {
												$scope.gridsterOpts.columns = 6;
												$scope.tileInRow = 3;
											}

											$scope.populateHiddenTileList(response.data.data.hiddenTiles);
											$scope.drawTiles();

											$timeout(function() {
												$scope.hasHealthData = true;
												cvUtil.unmask("#healthContent");
											}, 500);

										}

									}, function() {
										console.log("Failed to load health report.");
									});
								}

								$scope.drawTable = function(paramData) {
									var view = {};
									view.title = '';
									view.filters = false;
									view.collapsible = false;
									view.alarm = false;
									view.exportExcel = false;
									view.colChooser = false;
									angular.element("#tableWrapper").append(SurveyTemplates.makeReportTable("table_0",
											view));
									var gridid = "table_0";
									var dataTabl = angular.element("#" + gridid + CVTable.table)
											.hasClass("initialized");
									if (dataTabl) {
										angular.element("#" + gridid + CVTable.table).removeClass("initialized");
										angular.element("#" + gridid + CVTable.table).dataTable().fnClearTable();
										angular.element("#" + gridid + CVTable.table).dataTable().fnDestroy();
										angular.element("#" + gridid + CVTable.table).empty();
										angular.element("#" + gridid + "_FilterButton").removeClass("filterOn");
									}
									//
									var data = paramData.table_0.value;

									var originalColumns = data.aoColumns;
									angular.element("#" + gridid + CVTable.table).addClass("initialized").dataTable({
										"aaData" : data.aaData,
										"bFilter" : false,
										"aoColumns" : data.aoColumns,
										"aoColumnDefs" : data.aoColumnDefs,
										"bRetrieve" : false,
										"bInfo" : false,
										"bAutoWidth" : false,
										"sDom" : "Rlrti",
										"bLengthChange" : false,
										"aaSorting" : {},
										"iDisplayLength" : data.aaData.length,
										"bUseRendered" : false,
										"oLanguage" : {
											"sEmptyTable" : "No data available"
										},
										"fnDrawCallback" : function(oSettings) {
											$('#' + gridid + CVTable.table).addClass('drawn');
										}
									});
									var dataTable = angular.element("#" + gridid + CVTable.table).dataTable();
									angular.element("#" + gridid + CVTable.table + " thead th").each(function(index,
											value) {
										$(value).attr("title", $(value).text());
									});

									angular.element("#" + gridid + CVTable.table + ">tbody>tr>td")
											.each(function(index, value) {
												angular.element(value)
														.attr("data-label",
																angular.element("#" + gridid + CVTable.table +
																		" thead th")[index %
																		(angular.element("#" + gridid + CVTable.table +
																				" thead th").length)].innerText);
											});

									angular.element(".dataTable tbody td").each(function(index) {
										var td = $(this);
										var titleVal = td.text();
										if (titleVal != '') {
											td.attr("title", titleVal);
										}
									});

									angular.element("#table_0 th").removeClass("sorting_asc");

								}

								$scope.populateHiddenTileList = function(hTiles) {
									$scope.hiddenTiles = hTiles;
									$scope.hiddenTilesMap = [];
									$.each($scope.allTiles, function(index, tile) {
										if ($scope.hiddenTiles.indexOf(tile.tileId) != -1) {
											$scope.hiddenTilesMap.push({
												tileId : tile.tileId,
												tileName : tile.tileName
											});
										}
									});
								}

								$scope.addToHiddenTileList = function(tileId, tileName) {
									$scope.hiddenTiles.push(tileId);
									$scope.hiddenTilesMap.push({
										tileId : tileId,
										tileName : tileName
									});
								}

								$scope.isCritical = function(tile) {
									return tile.status.status === "Critical";
								}

								$scope.isWarning = function(tile) {
									return tile.status.status === "Warning";
								}
								$scope.isGood = function(tile) {
									return tile.status.status === "Good";
								}

								$scope.isInfo = function(tile) {
									return tile.status.status === "Info";
								}

								$scope.getOutcome = function(outcome) {
									return outcome.value;
								}

								$scope.hasNoRemarks = function(tile) {
									if (tile.recommendations == undefined || tile.recommendations.length === 0) {
										return true;
									}

									return false;
								}

								$scope.viewChanged = function(selection) {
									cvUtil.mask("#healthContent", cvUtil.cvLocalize("Loading"));
									$scope.sorting = selection;
									updateURL(null, selection);
									$scope.drawTiles();
									$timeout(function() {
										cvUtil.unmask("#healthContent");
									}, 500);
								}

								$scope.showAllTiles = function() {
									cvUtil.mask("#healthContent", cvUtil.cvLocalize("Loading"));
									healthService.showAllTiles().then(function(resp) {
										$scope.hiddenTiles = [];
										$scope.hiddenTilesMap = [];
										$scope.drawTiles();
										cvUtil.unmask("#healthContent");
									}, function(error) {
										console.log("Failed to clear user preference for hidden tiles.");
										cvUtil.errorToast("Failed to clear user preference for hidden tiles.");
										cvUtil.unmask("#healthContent");
									});
								}

								$scope.showTile = function(tile) {
									cvUtil.mask("#healthContent", cvUtil.cvLocalize("Loading"));
									healthService.showTile(tile.tileId).then(function(resp) {
										$scope.populateHiddenTileList(resp.data.data.hiddenTiles);
										$scope.drawTiles();
										console.log("Successfully removed tile from hidden list.");
										cvUtil.unmask("#healthContent");
									}, function(error) {
										console.log("Failed to remove tile from hidden list: " + tile.tileName);
										cvUtil.errorToast("Failed to remove tile from hidden list: " + tile.tileName);
										cvUtil.unmask("#healthContent");
									});
								}

								$scope.updateTiles = function(filterStatus) {
									if (filterStatus == $scope.filterStatus && filterStatus != -1) { //toggling status
										filterStatus = -1;
									}
									updateURL(filterStatus, null);

									$scope.filterStatus = filterStatus;

									cvUtil.mask("#tilesPanel", cvUtil.cvLocalize("Loading"));
									$scope.drawTiles();
									cvUtil.unmask("#tilesPanel");
								}

								$scope.setTileIndex = function(tile) {
									tile.tileIndex = $scope.tileCounter++;
								}

								$scope.setTileDimension = function(tiles) {
									$.each($scope.allTiles, function(index, tile) {
										tile.tileProperty = {};
									});
								};

								function updateURL(severity, grouping, localCommUniId) {
									var commUniId = cvUtil.getParameter("commUniId", 0);
									var ccGroupId = cvUtil.getParameter("ccGroupId", 0);
									var companyId = cvUtil.getParameter("companyId", 0);
									var currentSeverity = cvUtil.getParameter("severity", "");
									var currentGrouping = cvUtil.getParameter("grouping", "");
									var exportType = cvUtil.getParameter("exportType", "");

									if (localCommUniId) {
										commUniId = localCommUniId;
									}

									var params = [];
									var urlParams = "";

									if (commUniId) {
										params.push("commUniId=" + commUniId);
									}
									if (ccGroupId) {
										params.push("ccGroupId=" + ccGroupId);
									}
									if (companyId) {
										params.push("companyId=" + companyId);
									}

									if (severity) {
										params.push("severity=" + severity);
									} else if (currentSeverity) {
										params.push("severity=" + currentSeverity);
									}

									if (grouping) {
										params.push("grouping=" + grouping);
									} else if (currentGrouping) {
										params.push("grouping=" + currentGrouping);
									}

									if (exportType) {
										params.push("exportType=" + exportType);
									}

									History.replaceState("healthState", document.title, "?" + params.join("&"));
								}

								function getTileHeight(tile) {
									var height = 'FULL';
									switch (tile.tileId) {
									case 1: //CommCell Version
									case 2: //DDB Partition counts per Media Agent
									case 7: //Clients at 2 versions behind CommServer
									case 9: //Subclients without Storage Policy Association
									case 10: // Largest 25 Clients
									case 11: //Long Running Backup Jobs for last 1 day
									case 13: //Incremental backup change rate for Last 7 days
									case 15: //Unused Schedule Policies
									case 16: //DeDuplication Ratio by Storage Policy Copy in last 30 days
									case 18: //Top 10 Errors in last 24 hours
									case 19: //DR Backup
									case 20: // Clients completing with errors in last 30 days
									case 24: //Mount Path
									case 25: //Capacity License Usage
									case 27: //Snap Engine Usage
									case 37: //Scale Statistics
									case 42: //Prune all database agent logs only by days retention rule
									case 43: //Database Index Fragmentation
									case 47: //Value Assessment
									case 52: //Proactive Support
										height = 'HALF';
										break;
									}

									if (tile.plotType == 'PIE_CHART' || tile.plotType == 'BAR_CHART' ||
											tile.plotType == 'DONUT_CHART') {
										var chartDataFound = false;
										for (var i = 0; i < tile.outcomes.length; i++) {
											var outcome = tile.outcomes[i];
											if (outcome.outcomeType == 'Slice') {
												chartDataFound = true;
											}
										}
										if (!chartDataFound) {
											height = 'HALF';
										}

									}

									return height;
								}

								function clone(obj) {
									if (null == obj || "object" != typeof obj) {
										return obj;
									}
									var copy = obj.constructor();
									for ( var attr in obj) {
										if (obj.hasOwnProperty(attr)) {
											copy[attr] = obj[attr];
										}
									}
									return copy;
								}

								$scope.attachEventHandlers = function() {
									//							angular.element("body").on("click", function() {
									//								angular.element(".fixActionMenuDiv").hide();
									//							});

									angular.element("#exportPpt").on("mouseover", function() {
										angular.element("#_exportppt").show();
									});

									angular.element("#exportPpt").on("mouseout", function() {
										angular.element("#_exportppt").hide();
									});

									angular.element(".exportatserver").on("click", function() {
										var type = this.id.substring("export".length);
										if (type) {
											type = type.toLowerCase();
										}
										$scope.saveReportAs(type, $scope.customerName);
									});

									angular.element("#exportEmail").on("click", function() {
										$scope.exportEmail();
									});

									reportsUtil.scheduleReport({
										exportFileNamePrefix : healthUtilService.getFilePrefix($scope.customerName),
										format : [ 'PDF', 'CSV', 'EMAIL_BODY' ]
									});

									//									if ($($(".vw-js-nav-trigger .vw-close")[0]).css("display") !== "none") {
									//										angular.element(".vw-js-nav-trigger").click();
									//										angular.element("#reportSideBar .sec-title").addClass("hidden");
									//									}

									angular.element($window).bind('resize', function() {
										var repaint = false;
										if ($window.innerWidth > 1760) {
											$scope.gridsterOpts.columns = 8;
											$scope.tileInRow = 4;
											repaint = true;
										} else if ($window.innerWidth <= 950) {
											$scope.gridsterOpts.columns = 2;
											$scope.tileInRow = 1;
											repaint = true;
										} else if ($window.innerWidth <= 1400) {
											$scope.gridsterOpts.columns = 4;
											$scope.tileInRow = 2;
											repaint = true;
										} else if ($window.innerWidth <= 1760) {
											$scope.gridsterOpts.columns = 6;
											$scope.tileInRow = 3;
											repaint = true;
										}
										if (repaint) {
											$scope.tilePositionIndex = 0;
											$.each($scope.tileGroups, function(key, value) {
												$scope.setGroupPosition(value);
											});
										}
									});

								}

								$scope.reportAlert = function() {
									Reports.configureAlarm({
										"paramId" : -1,
										"paramName" : "Health Report",
										"type" : Alarms.TYPE_HEALTH,
										"paramsTable" : null,
										"manageLink" : true,
										"isTiledHealth" : true,
										"reportName" : "Health Report",
										"alertCriteria" : {
											"Parameter" : "Health Report",
											"criteria" : undefined,
											"Editable" : 'Yes'
										}
									});
								}

								$scope.exportEmail = function() {
									var filePrefix = healthUtilService.getFilePrefix($scope.customerName);
									var target = cvUtil.getContextPath() + "/reports/";

									target += "newReportEmail.do?" + cvUtil.createParamString('filePrefix', filePrefix);
									target += "&source=health&format=PDF,CSV,EMAIL_BODY&";

									if (window.reportEnum) {
										target += cvUtil.createParamString('reportEnum', window.reportEnum) + "&";
									}
									if (window.reportUniqId) {
										target += cvUtil.createParamString('reportUniqId', window.reportUniqId);
									}
									cvUtil.showModal({
										width : 550,
										height : 190,
										title : cvUtil.cvLocalize('Email_Report'),
										href : target,
										iframe : true
									});
									$(".modal").css('height', 200)
								}

								$scope.saveReportAs = function(exportType, customerName) {
									var filePrefix = healthUtilService.getFilePrefix(customerName);
									var url = document.URL;
									url = url + "&exportType=" + exportType;

									angular.element("#content").mask(localMsg['WaitMsg'] + "&nbsp&nbsp", 0, function() {
										return (exportajaxRequest && exportajaxRequest.abort());
									});

									if (exportType === "healthppt" || exportType == "word" ||
											exportType == "valassessppt") {
										angular.element("#content").unmask();
										healthUtilService.exportX(exportType, customerName);
										return;
									}

									var subscriptionID = window.sessionStorage.getItem('export_subscriptionID');

									var saveAsUrl = cvUtil.getContextPath() +
											'/reports/exp.do?orientation=portrait&exportType=' + exportType +
											'&viewportWidth=1600&commUniId=' + cvUtil.getParameter("commUniId");
									var params = {
										url : url,
										mhtmlURL : "/survey/health",
										filePrefix : filePrefix,
										directDownload : false,
										type : 'report',
									};

									var isPushDownload = false;
									if (exportType == "pdf" || exportType == "csv") {
										isPushDownload = true;
									}

									if (isPushDownload) {
										angular.element("#content").unmask();
										params.subscriptionID = subscriptionID;
									}
									var exportajaxRequest = healthService.saveAs(saveAsUrl, params);

									exportajaxRequest.then(function(data) {
										if (data && !isPushDownload) {
											var data1 = data.data;
											angular.element("#content").unmask();
											downloadReportFile(data1);
										}
										angular.element("#content").unmask();

									}, function(error) {
										angular.element("#content").unmask();
										if (error != 'userAborted') {
											cvUtil.errorToast(error);
										}
									});
								}

								$scope.drawTiles = function() {
									$scope.loaded = false;
									$scope.tileGroups = [];

									//group the tiles
									if ($scope.sorting == "category") { //By Category
										var groups = {};
										$.each($scope.allTiles, function(index, tile) {
											if ($scope.hiddenTiles.indexOf(tile.tileId) != -1) {
												return true;
											}

											var groupId = tile.category.categoryId;
											if ($scope.filterStatus === -1 ||
													getStatusAsInt(tile.status.status) === $scope.filterStatus) {
												if (groups[groupId] == undefined) {
													groups[groupId] = {
														groupName : tile.category.categoryName,
														groupId : tile.category.categoryId,
														tiles : []
													};

												}
												tile.groupId = groupId;
												groups[groupId].tiles.push(clone(tile));
											}
										});

										//when viewing by category, display tiles according to CATEGORIES order
										$.each(CATEGORIES, function(key, categoryId) {
											var tempGroup = $scope.getGroup(categoryId, groups);
											if (tempGroup) {
												var tileGroup = {};
												tileGroup.name = tempGroup.groupName;
												tileGroup.id = tempGroup.groupId;
												tileGroup.tiles = tempGroup.tiles;
												$scope.tileGroups.push(tileGroup);
											}
										});

									} else { //by status
										var grpCritical = [], grpWarning = [], grpGood = [], grpInfo = [];

										$.each($scope.allTiles, function(index, tile) {
											if ($scope.hiddenTiles.indexOf(tile.tileId) != -1) {
												return true;
											}
											var statusAsInt = getStatusAsInt(tile.status.status);
											if ($scope.filterStatus === -1 || statusAsInt === $scope.filterStatus) {
												if (statusAsInt === CRITICAL) {
													grpCritical.push(clone(tile));
												} else if (statusAsInt === WARNING) {
													grpWarning.push(clone(tile));
												} else if (statusAsInt === GOOD) {
													grpGood.push(clone(tile));
												} else if (statusAsInt === INFO) {
													grpInfo.push(clone(tile));
												}

												tile.groupName = tile.status.status;
											}
										});

										if (grpCritical.length > 0) {
											$scope.tileGroups.push({
												id : CRITICAL,
												name : getStatusDisplayName('Critical'),
												tiles : grpCritical
											});
											updateTileGroupId(grpCritical, CRITICAL);
										}

										if (grpWarning.length > 0) {
											$scope.tileGroups.push({
												id : WARNING,
												name : getStatusDisplayName('Warning'),
												tiles : grpWarning
											});
											updateTileGroupId(grpWarning, WARNING);
										}

										if (grpGood.length > 0) {
											$scope.tileGroups.push({
												id : GOOD,
												name : getStatusDisplayName('Good'),
												tiles : grpGood
											});
											updateTileGroupId(grpGood, GOOD);
										}

										if (grpInfo.length > 0) {
											$scope.tileGroups.push({
												id : INFO,
												name : getStatusDisplayName('Info'),
												tiles : grpInfo
											});
											updateTileGroupId(grpInfo, INFO);
										}

									}

									$scope.tilePositionIndex = 0;
									$.each($scope.tileGroups, function(key, value) {
										$scope.setGroupPosition(value);
									});

									$scope.loaded = true;
								}

								function getStatusAsInt(status) {
									if (isNaN(parseInt(status))) {
										if (status == 'Critical') {
											return CRITICAL;
										} else if (status == 'Warning') {
											return WARNING;
										} else if (status == 'Good') {
											return GOOD;
										} else if (status == 'Info') {
											return INFO;
										} else {
											return -1;
										}
									} else {
										return parseInt(status);
									}
								}

								$scope.getGroup = function(groupId, groups) {
									var group = undefined;
									$.each(groups, function(key, value) {
										if (value.groupId === groupId) {
											group = value;
											return false;
										}
									});

									return group;
								}

								$scope.setGroupPosition = function(group) {
									var halfTileLoc = {}, curRow = 0, curCol = 0;
									var halfTileExist = false;
									var tileCtr = 0;
									var halfTiles = [];
									var fullTiles = [];
									var tileInRow = $scope.tileInRow;
									var singleRow = false;
									var secAId = 63;
									var secATile = null;
									var secATileIndex = 0;

									if (group.id == 1) { //CATEGORY
										if (group.tiles.length > 1) {
											$.each(group.tiles, function(index, curTile) {
												if (curTile.tileId == secAId) {
													secATileIndex = index;
													secATile = curTile;
													return false;
												}
											});

											if (secATile) {
												group.tiles.splice(secATileIndex, 1);
											}
										}

									}

									$.each(group.tiles, function(index, curTile) {
										curTile.sizeX = TILE_WIDTH;
										var tHeight = FULL_TILE;
										if (curTile.tileProperty && curTile.tileProperty.height) {
											tHeight = curTile.tileProperty.height;
										}
										if (tHeight === 'HALF') {
											curTile.sizeY = HALF_TILE;
											halfTiles.push(curTile);
										} else {
											curTile.sizeY = FULL_TILE;
											fullTiles.push(curTile);
										}
									});

									//plot full tiles first
									$.each(fullTiles, function(index, curTile) {
										curTile.row = curRow;
										curTile.col = curCol;
										tileCtr++;
										curCol += TILE_WIDTH;
										if (tileCtr == tileInRow) {
											curRow += FULL_TILE;
											curCol = 0;
											tileCtr = 0;
										}
										curTile.tilePositionIndex = $scope.tilePositionIndex++;
									});

									//plot all halftiles if less than four and no full
									if ((fullTiles.length == 0 || tileCtr == 0) && halfTiles.length <= tileInRow &&
											group.id != 1) {
										$.each(halfTiles, function(index, curTile) {
											curTile.row = curRow;
											curTile.col = curCol;
											curCol += TILE_WIDTH;
											curTile.tilePositionIndex = $scope.tilePositionIndex++;
										});
									} else {
										$.each(halfTiles, function(index, curTile) {
											if (halfTileExist) {
												curTile.row = halfTileLoc.row + HALF_TILE;
												curTile.col = halfTileLoc.col;
												halfTileExist = false;
												tileCtr++;
											} else {
												curTile.row = curRow;
												curTile.col = curCol;
												halfTileLoc.row = curRow;
												halfTileLoc.col = curCol;
												halfTileExist = true;
												curCol += TILE_WIDTH;
											}

											curTile.tilePositionIndex = $scope.tilePositionIndex++;
											if (tileCtr == tileInRow) {
												curRow += FULL_TILE;
												curCol = 0;
												tileCtr = 0;
											}

										});
									}

									if (secATile) {
										secATile.row = curRow;
										secATile.col = curCol;
										secATile.sizeY = FULL_TILE;
										group.tiles.push(secATile);
									}
								}

								$scope.updateTilePositions = function(groupId, tileId) {
									var group = [];

									if ($scope.sorting == "severity") {
										group = $scope.tileGroups[0];
									}

									$.each($scope.tileGroups, function(index, val) {
										if (val.id === groupId) {
											group = val;
											return false;
										}
									})

									$.each(group.tiles, function(index, tile) {
										if (tile.tileId == tileId) {
											group.tiles.splice(index, 1);
											return false;
										}
									});

									$scope.setGroupPosition(group);

								}

								function updateTileGroupId(tiles, groupId) {
									$.each(tiles, function(index, tile) {
										tile.groupId = groupId;
									});
								}

								function getStatusDisplayName(status) {
									if (status === 'Critical') {
										return localMsg.criticalStatus;
									} else if (status == 'Warning') {
										return localMsg.warningStatus;
									} else if (status == 'Good') {
										return localMsg.goodStatus;
									} else {
										return localMsg.infoStatus;
									}
								}

								//header is not angular based
								function setSummaryData(summaryData) {
									var customerName = summaryData.Customer ? summaryData.Customer.value : "";
									var commcellId = summaryData.CommCellID ? summaryData.CommCellID.value : "";
									var time = summaryData.Time ? summaryData.Time.value : "";
									var guid = summaryData.CommServeGUID ? summaryData.CommServeGUID.value : "";
									var monitoredGroup = summaryData.MonitoredGroup ? summaryData.MonitoredGroup.value
											: "";
									var groupName = summaryData.GroupName ? summaryData.GroupName.value : undefined;
									$scope.customerName = customerName;

									var reportName = cvUtil.cvLocalize("Health_Report");
									if (customerName) {
										$(document).attr("title",
												cvUtil.cvLocalize("Health_Report_title", customerName));
										angular.element('#welcomeMsg').html(cvUtil.cvLocalize("Health_Report_title",
												customerName));
										angular.element('.reportsLink').text(customerName + ' (' +
												cvUtil.cvLocalize("Reports") + ')');
									} else {
										$(document).attr("title", reportName);
										angular.element('#welcomeMsg').html(reportName);
										angular.element('.reportsLink').text(cvUtil.cvLocalize("Reports"));
									}
									cvUtil.logToReportsUsage(reportName);

									if (summaryData.enableDoc) {
										angular.element('#exportWord').show();
										angular.element('#exportPpt').show();
									} else {
										angular.element('#exportWord').hide();
										angular.element('#exportPpt').hide();
									}

									if ($('#exportWord').hasClass('wellnessTag')) { //Custom capability to be set by Sanjay MR:266343
										angular.element('#exportWord').show();
									}

									cvUtil.GUIAuditReportLog("Health Report", customerName);

									if (commcellId != undefined && $.trim(commcellId).length != 0) {
										angular.element("#metaCCId").html('<span>' +
												cvUtil.cvLocalize("commcell.info.label.CommCell_ID") + ': ' +
												decimalToHex(commcellId, 2).toUpperCase() + '</span>');
									}

									if (time != undefined && $.trim(time).length != 0) {
										angular.element('#metaAsOf').html('<span>' +
												cvUtil.cvLocalize("commcell.info.label.As_of") + ': ' + time +
												'</span>');
									}

									if (guid != undefined && $.trim(guid).length != 0) {
										guid = registrationCode(guid);
										angular.element('#GUID').html('<span>' +
												cvUtil.cvLocalize("commcell.info.label.Registration_Code") + ': ' +
												guid + '</span>');
									}

									if (monitoredGroup != undefined && $.trim(monitoredGroup).length != 0) {
										angular.element('#metaClientGrp').html('<span>' +
												cvUtil.cvLocalize("audit.label.Client_Group") + ': ' + monitoredGroup +
												'</span>');
									}

									if (groupName) {
										angular.element('.commcellsLink').text(groupName + ' (' +
												cvUtil.cvLocalize("CommCells") + ')');
									}
								}

							} ]);
}());