(function() {
	'use strict';

	var healthApp = angular.module("healthDashboardApp");

	healthApp.controller("donutController", [ "$scope", "healthUtilService", function($scope, healthUtilService) {

		var drilldownHandler = function(event) {
			if (event.target.drilldown) {
				window.open(event.target.drilldown, "_blank");
			}
			event.preventDefault();
			return false;
		}

		$scope.chartConfig = getChartConfig();

		function getChartConfig() {
			var interactive = isDrillDownEnabled($scope.tile.tileId);
			var chartConfig = {
				size : {
					width : 350,
					height : 175
				},
				options : {
					chart : {
						type : "pie",
						zoomType : 'x',
						animation : false,
						interactiveEnabled : false,
						marginRight : 20,
						marginTop : 0,
						spacingTop : 0,
						spacingBottom : 0,
						spacingLeft : 0,
						spacingRight : 0,
						style : {
							"color" : "#777",
							"font-family" : "Helvetica,Arial,sans-serif",
							"pointer-events" : "none",
							"float" : "left"
						},
						events : interactive ? drilldownHandler : null
					},
					exporting : {
						enabled : false
					},
					legend : getLegendStyle(interactive),
					credits : {
						enabled : false
					},
					tooltip : {
						enabled : false,
						animation : false
					},
					plotOptions : {
						series : {
							animation : false,
							interactiveEnabled : false,
							allowPointSelect : false,
							point : {
								events : {
									select : interactive ? drilldownHandler : null
								}
							}
						},
						pie : {
							size : "100%",
							borderWidth : 0,
							dataLabels : {
								enabled : false,
								format : undefined
							},
							showInLegend : true,
							point : {
								events : {
									legendItemClick : interactive ? drilldownHandler : null
								}
							},
							states : {
								hover : {
									enabled : false
								}
							}
						}
					}
				},
				title : {
					text : getTitle(),
					align : "center",
					verticalAlign : "middle",
					y : 5,
					style : {
						"color" : "#777",
						"font-family" : "Helvetica,Arial,sans-serif",
						"font-size" : "22px",
						"font-weight" : "bold"
					}

				},
				series : [ {
					data : getData(),
					size : "100%",
					innerSize : 115,
					startAngle : 0,
					endAngle : 360,
					pointPadding : 0,
					stacking : "normal",
					name : 100
				} ],
				loading : false,
				noData : true
			}

			if ($scope.tile.tileId == 65) {
				chartConfig.options.plotOptions.pie.center = [ '30%', '50%' ];
				chartConfig.size.width = 400;
				chartConfig.title.align = "left";
				chartConfig.title.x = 110;
			}

			return chartConfig;
		}

		function getLegendStyle(interactive) {
			var style = {
				align : 'right',
				verticalAlign : 'top',
				layout : 'vertical',
				enabled : true,
				y : 20,
				itemMarginBottom : 8,
				itemStyle : {
					"color" : "#777",
					"font-size" : "11px",
					"pointer-events" : interactive ? "auto" : "none",
					"font-weight" : "normal"
				}
			}

			return style;
		}

		function isDrillDownEnabled(tileId) {
			switch (tileId) {
			case 65:
				return true;//VMs Need Attention
			default:
				return false;
			}
		}

		function getTitle() {
			for (var i = 0; i < $scope.tile.outcomes.length; i++) {
				if ($scope.tile.outcomes[i].outcomeType === 'Title') {
					return $scope.tile.outcomes[i].value;
				}
			}
		}
		function getData() {
			var data = [];

			var COLORS = healthUtilService.getColors();
			$.each($scope.tile.outcomes, function(index, outcome) {
				if (outcome.outcomeType !== 'Title') {
					var sliceData = {};
					if (outcome.property.style === 'Success') {
						sliceData.color = COLORS.SUCCESS;
					} else if (outcome.property.style === 'Failed') {
						if ($scope.tile.tileId == 65) {
							sliceData.color = COLORS.GOLD;
						}else{
							sliceData.color = COLORS.KILLED;
						}
					} else if (outcome.property.style === "Unsupported") {
						if ($scope.tile.tileId == 65) {
							sliceData.color = COLORS.PURPLE;
						}else{
							sliceData.color = COLORS.GOLD;
						}
					} else if (outcome.property.style === "Warning") {
							sliceData.color = COLORS.WARNING;
					} else if (outcome.property.style === "FileDetailsCollectionFailure") {
						if ($scope.tile.tileId == 65) {
							sliceData.color = COLORS.LIGHT_BLUE;
						}else{
							sliceData.color = COLORS.WARNING;
						}
					}
					sliceData.y = parseInt(outcome.value);
					sliceData.name = outcome.key;
					if (outcome.drillDownLink) {
						sliceData.drilldown = outcome.drillDownLink;
					}

					data.push(sliceData);
				}
			});

			return data;
		}

	} ]);
}());