(function() {
	"user strict";

	angular.module("healthDashboardApp").service('healthService', [ '$http', '$q', function($http, $q) {

		this.getHealth = function(parameters) {
			var commUniId = getParameter("commUniId", null, 0, true);
			var paramId = getParameter("paramId", null, 0, true);
			var paramStatus = getParameter("paramStatus", null, "", true);
			var paramValue = getParameter("paramValue", null, -1, true);
			var op = getParameter("op", null, -1, true);
			var params = {};
			params.commUniId = commUniId;
			params.paramId = paramId;
			params.paramStatus = paramStatus;
			params.paramValue = paramValue;
			params.op = op;

			if (parameters && parameters.isCSVExport === true) {
				params.exportedFileName = getParameter("exportedFileName");
				return $http.post("exportHealthCSV.do", params, {
					cache : true,
					csrf: cvUtil.readCookie('csrf')
				});
			} else {
				return $http.post("getHealth.do", params, {
					cache : true,
					csrf: cvUtil.readCookie('csrf')
				});
			}

		};
		
		this.getHealthAlertTable = function(tileId, tileStatus) {
			var params = {};
			params.tileId = tileId;
			params.tileStatus = tileStatus;
			return $http.post("getHealthAlertTable.do", params, {
				cache : true,
				csrf: cvUtil.readCookie('csrf')                        
			});
		}
		
		this.getCategories = function() {
			var values = ['Priority'];
			var params = {name:'orderby', values:values};
			return $http.get("getCategories.do", {parameters: JSON.stringify(params)}, {
				cache : false,
				csrf: cvUtil.readCookie('csrf')     
			});
		}

		this.hideTile = function(tileId) {
			return $http.post("hideTile.do", {
				'tileId' : tileId
			}, {
				cache : false
			});
		};

		this.showTile = function(tileId) {
			return $http.post("showTile.do", {
				'tileId' : tileId
			}, {
				cache : false
			});
		};

		this.showAllTiles = function() {
			return $http.post("showAllTile.do", {}, {
				cache : false
			});
		};
		
		this.setHealthGrouping = function(grouping) {
			return $http.post("setHealthGrouping.do", {'grouping':grouping}, {
				cache : false
			});
		};
		
		this.setTileStatusFilter = function(filter) {
			return $http.post("setTileStatusFilter.do", {'filter': filter}, {
				cache : false
			});
		};
		
		 this.saveAs = function(url,params){
			 var deferredAbort = $q.defer();
			 var promise=$http.post(url,params,{timeout: deferredAbort.promise}).then(
	                 function( response ) {
	                     return( response);
	                 },
	                 function(error) {
	                     return( deferredAbort.reject(error));
	                 }
	             );

			 promise.abort = function() {
	             deferredAbort.resolve();
	         };
	         promise.finally(
	                 function() {
	                     promise.abort = angular.noop;
	                     deferredAbort = request = promise = null;
	                 }
	             );

			 return promise;
		 }
		 
		 this.generateUUID = function() {
				var currentTime = new Date().getTime();
				var uuid = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function(chr) {
					var randomNum = (currentTime + Math.random() * 16) % 16 | 0;
					currentTime = Math.floor(currentTime / 16);
					return (chr == 'x' ? randomNum : (randomNum & 0x7 | 0x8)).toString(16);
				});
				return uuid;
		};

		function getParameter(name, url, defaultValue, getUrlViaHistoryJs) {
			name = name.replace(/[\[]/, "\\\[").replace(/[\]]/, "\\\]");
			var regexS = "[\\?&]" + name + "=([^&#]*)";
			var regex = new RegExp(regexS);
			if (typeof url === "undefined" || url === null) {
				url = window.location.href;
			}
			var results = regex.exec(url);
			if (results === null) {
				if (typeof defaultValue !== 'undefined') {
					return defaultValue;
				} else {
					return "";
				}
			} else {
				return decodeURIComponent(results[1].replace(/\+/g, " "));
			}
		}
		;
	} ]);
}());