(function() {
	'use strict';

	var DEFAULT_COLORS = [ "#02557d", "#22b7e5", "#e8dc6d", "#86b300", "#5900b3", "#29a3a3" ];

	var healthApp = angular.module("healthDashboardApp");

	healthApp.controller("pieBarChartController", [ "$scope", "healthUtilService", function($scope, healthUtilService) {

		$scope.chartConfig = getChartConfig();
		setChartData();

		$scope.hasChartData = function() {
			var total = 0;
			for (var i = 0; i < $scope.tile.outcomes.length; i++) {
				var outcome = $scope.tile.outcomes[i];
				if (outcome.outcomeType == 'Slice') {
					total += parseInt(outcome.value);
				}
			}
			return total > 0;
		}

		function getChartConfig() {
			var chartConfig = {
				size : getChartSize(),
				options : {
					chart : {
						type : getChartType(),
						zoomType : 'x',
						animation : false,
						interactiveEnabled : false,
						//						marginRight : $scope.tile.plotType === 'PIE_CHART' ? 150 : 0,
						marginLeft : 10,
						marginBottom : 70,
						//						marginTop : $scope.tile.plotType === 'PIE_CHART' ? 0 : 20,
						spacingTop : 0,
						spacingBottom : 0,
						spacingLeft : 0,
						spacingRight : 0,

						style : {
							"color" : "#777",
							"font-family" : "Helvetica,Arial,sans-serif",
							"pointer-events" : "none"
						}
					},
					colors : DEFAULT_COLORS,
					exporting : {
						enabled : false
					},
					legend : getLegendStyle(),
					credits : {
						enabled : false
					},
					tooltip : {
						enabled : false,
						animation : false
					},
					plotOptions : {
						series : {
							animation : false,
						},
						pie : {
							size : "100%",
							borderWidth : 0,
							dataLabels : {
								enabled : false,
								format : undefined
							},
							showInLegend : true
						},
						bar : {
							stacking : 'normal',
							pointWidth : 20
						}
					}
				},
				title : {
					text : undefined
				},
				series : [],
				xAxis : {
					lineWidth : 1,
					gridLineWidth : 0,
					tickWidth : 0
				},
				yAxis : {
					lineWidth : 1,
					gridLineWidth : 0,
					minorGridLineWidth : 0,
					title : {
						enabled : false
					}
				},
				loading : false,
				noData : true
			}

			return chartConfig;
		}

		function getChartSize() {
			if ($(window).width() <= 767) {
				return {
					width : 225,
					height : 250
				}
			}
			if ($scope.tile.plotType === 'PIE_CHART') {
				return {
					width : 400,//275,
					height : 250
				}
			} else {
				return {
					width : 400,
					height : 120
				}
			}
		}

		function getChartType() {
			return $scope.tile.plotType === 'PIE_CHART' ? "pie" : "bar";
		}

		function getLegendStyle() {
			var style = {
				align : 'right',
				verticalAlign : 'top',
				layout : 'vertical',
				enabled : true,
				y : 20,
				itemMarginBottom : 8,
				itemStyle : {
					"color" : "#777",
					"font-size" : "11px",
					"pointer-events" : "none",
					"font-weight" : "normal"
				}
			}

			if ($scope.tile.plotType === "BAR_CHART") {
				style.align = "left";
				style.layout = "horizontal";
				style.verticalAlign = "bottom";
			}
			return style;
		}

		function setChartData() {
			if ($scope.tile.tileId === 3) {
				var f = 0;
			}
			var series = $scope.chartConfig.series;
			var categories = [];

			if ($scope.tile.plotType === 'PIE_CHART') {
				var defaultSeries = {};
				var style = "";
				var COLORS = healthUtilService.getColors();
				defaultSeries.data = [];
				series.push(defaultSeries);

				$.each($scope.tile.outcomes, function(index, outcome) {
					if (outcome.outcomeType === 'Slice') {
						var slice = {};
						style = outcome.property.style;
						slice.name = outcome.key;
						slice.y = parseInt(outcome.value);
						if (style === 'Failed') {
							slice.color = COLORS.FAILED;
						} else if (style === 'Killed') {
							slice.color = COLORS.KILLED;
						} else if (style === 'CWR') {
							slice.color = COLORS.CWE;
						} else if (style === 'CWW') {
							slice.color = COLORS.CWW;
						} else if (style === 'Success') {
							slice.color = COLORS.SUCCESS;
						}

						defaultSeries.data.push(slice);
						categories.push(outcome.key);

					}
				});

				$scope.chartConfig.xAxis = {};
				$scope.chartConfig.xAxis.categories = categories;

			} else {
				var defaultSeries = {};
				defaultSeries.data = [];
				var len = DEFAULT_COLORS.length;

				$.each($scope.tile.outcomes, function(index, outcome) {
					if (outcome.outcomeType === 'Slice') {
						var bar = {};
						bar.data = [ parseInt(outcome.value) ];
						bar.name = outcome.key.trim();
						bar.color = DEFAULT_COLORS[(index - 1) % len];
						series.push(bar);
						defaultSeries.data.push(bar);
					}
				});

				var ctr = defaultSeries.data.length - 1;
				var legendIndex = 0;
				$.each(defaultSeries.data, function(index, bar) {
					bar.index = ctr--;
					bar.legendIndex = legendIndex++;
				});

				$scope.chartConfig.xAxis.categories = categories;
			}

		}

	} ]);
}());