(function() {
	'use strict';

	var healthApp = angular.module("healthDashboardApp");

	healthApp.controller("tileController", [
			"$scope",
			"healthService",
			"healthUtilService",
			function($scope, healthService, healthUtilService) {

				$scope.customGridsterItem = {};

				var browserExp = new RegExp('Android|webOS|iPhone|iPad|' + 'BlackBerry|Windows Phone|'
						+ 'Opera Mini|IEMobile|Mobile', 'i');

				if (browserExp.test(navigator.userAgent)) {
					$scope.isShowOptions = true;
				} else {
					$scope.isShowOptions = false;
				}
				$scope.detailTooltip = localMsg.Health_View_Details_Tooltip;

				$scope.detailHref = "";
				$scope.tileToolTip = "";
				$scope.opts = {};
				$scope.opts.subAppName = "health";
				$scope.opts.maskEle = "tile_" + $scope.tile.tileId;
				$scope.opts.healtTileViewer = true;
				$scope.opts.noTitleUpdate = true;
				$scope.opts.cacheKey = $scope.cacheId + 'health-report-cache-key';

				if ($scope.tile.tileStatus == 4) {
					$scope.tileToolTip = $scope.tile.tileProperty.thCritical || "";
				} else if ($scope.tile.tileStatus == 3) {
					$scope.tileToolTip = $scope.tile.tileProperty.thWarning || "";
				} else if ($scope.tile.tileStatus == 2) {
					$scope.tileToolTip = $scope.tile.tileProperty.thGood || "";
				}

				$scope.showOptions = function() {
					$scope.isShowOptions = true;
				}

				$scope.hideOptions = function() {
					$scope.isShowOptions = false;
				}

				$scope.isHalfTile = function() {
					return ($scope.tile.tileProperty && $scope.tile.tileProperty.height === 'HALF');
				}

				$scope.isFullTile = function() {
					return ($scope.tile.tileProperty && $scope.tile.tileProperty.height === 'FULL');
				}

				$scope.hideTile = function() {
					healthService.hideTile($scope.tile.tileId).then(function(resp) {
						$scope.addToHiddenTileList($scope.tile.tileId, $scope.tile.tileName);
						$scope.tilePositionIndex = 0;
						$scope.updateTilePositions($scope.tile.groupId, $scope.tile.tileId);
						console.log("Tile " + $scope.tile.tileName + " is now hidden.");
					}, function(error) {
						console.log("Failed to hide tile.");
						cvUtil.errorToast("Failed to hide tile.");
					});
				}

				$scope.getDetailURL = function() {
					var params = "", url = "";
					if ($scope.tile.tileProperty.detailReportName) {
						if (!$scope.tile.tileProperty.detailType || $scope.tile.tileProperty.detailType == "1") {
							var aliasName = cvUtil.getParameter("aliasName", "");
							var CCId = cvUtil.getParameter("CCID", "");
							var CCGUID = cvUtil.getParameter("CCGUID", "");
							url = cvUtil.getContextPath() + "/reportsplus/reportViewer.jsp?reportId=" +
									$scope.tile.tileProperty.detailReportName;
							url = url.replace("@sys_aliasName", aliasName);
							url = url.replace("@sys_CCID", CCId);
							url = url.replace("@sys_CCGUID", CCGUID);
						} else if ($scope.tile.tileProperty.detailType == "2") {
							var commUId = cvUtil.getParameter("commUniId", "");
							var ccGroupId = cvUtil.getParameter("ccGroupId", "");
							url = cvUtil.getContextPath() + $scope.tile.tileProperty.detailReportName;
							url = url.replace("@sys_commUId", commUId);
							url = url.replace("@sys_CCGroupId", ccGroupId);
						}
					}

					return url;
				}

				$scope.viewDetails = function() {
					var url = $scope.getDetailURL();

					if (url.length > 0) {
						window.open(url, "_blank");
					}
				}

				$scope.refreshTile = function() {

					var data = {
						tileId : $scope.tile.tileId
					};
					$scope.eventProcessed = false;
					$scope.$broadcast("refreshTile", data);

				}

				$scope.isDetailApplicable = function() {
					if ($scope.tile.tileProperty.detailReportName) {
						return true;
					}
					return false;
				}

				$scope.showDropdownMenu = function(event) {
					var targetElement, menuSelector;
					if ($(event.srcElement).hasClass('icon-arrow')) {
						targetElement = event.target.parentElement.parentElement;
					} else {
						targetElement = event.target.parentElement;
					}
					menuSelector = $(targetElement).find('.dropdown-selector-menu');
					if ($(menuSelector).hasClass('show')) {
						$(menuSelector).removeClass('show').addClass('hide');
					} else {
						$(menuSelector).removeClass('hide').addClass('show');
						// Remove the menu-item when focus is lost
						$(targetElement).find('[data-toggle=\"dropdown-selector\"]').focusout(function() {
							setTimeout(function() {
								$(menuSelector).removeClass('show').addClass('hide');
							}, 500);
						});
					}

					event.stopPropagation();
					event.preventDefault();
				}

				$scope.detailHref = $scope.getDetailURL();

			} ]);
}());