(function() {
	'use strict';

	var healthApp = angular.module("healthDashboardApp");

	healthApp.controller("tileController", [
			"$scope",
			"healthService",
			"healthUtilService",
			function($scope, healthService, healthUtilService) {
				$scope.customGridsterItem = {};
				$scope.tileOutcomeLabels = getOutcomeLabels();

				var browserExp = new RegExp('Android|webOS|iPhone|iPad|' + 'BlackBerry|Windows Phone|'
						+ 'Opera Mini|IEMobile|Mobile', 'i');

				if (browserExp.test(navigator.userAgent)) {
					$scope.isShowOptions = true;
				} else {
					$scope.isShowOptions = false;
				}
				$scope.fixItItems = [];
				$scope.defaultFixIt = {};
				$scope.detailTooltip = "";
				$scope.detailHref = "";
				$scope.isCR = false;

				$scope.init = function() {
					if ($scope.tile.plotType === 'CUSTOM_REPORT') {
						$scope.isCR = true;
						return;
					}

					for (var i = 0; i < $scope.tile.outcomes.length; i++) {
						var o = $scope.tile.outcomes[i];
						if (o.fixId != undefined && !isFixItFound(o.fixId)) {
							$scope.fixItItems.push({
								fixId : o.fixId,
								fixName : o.fixName,
								fixDisplayName : o.fixDisplayName,
								fixWfGuid : o.fixWfGuid
							});
						}
					}

					if ($scope.fixItItems.length > 0) {
						$scope.defaultFixIt.fixDisplayName = $scope.fixItItems[0].fixDisplayName;
						$scope.defaultFixIt.fixName = $scope.fixItItems[0].fixName;
						$scope.defaultFixIt.fixId = $scope.fixItItems[0].fixId;
					}

					if ($scope.tile.alertCriteria) {
						$.each($scope.tile.alertCriteria, function(index, value) {
							value.DescriptionText = value.criteriaValue;
						});
					}

					$scope.thresholds = "";
					if ($scope.tile.status.thresholds) {
						$.each($scope.tile.status.thresholds, function(index, val) {
							if ($scope.thresholds) {
								$scope.thresholds += "<br>Or " + val
							} else {
								$scope.thresholds += val;
							}
						});

						if ($scope.tile.status.thresholdTitle) {
							$scope.thresholds = "<b>" + $scope.tile.status.thresholdTitle + "</b><br>" +
									$scope.thresholds;
						}
					}

					//set remarks
					$scope.tile.remarks = [];
					$.each($scope.tile.recommendations, function(index, r) {
						var remark = {
							text : r
						};
						var tmp = document.createElement("DIV");
						tmp.innerHTML = r;
						remark.tooltip = tmp.textContent || tmp.innerText || "";
						$scope.tile.remarks.push(remark);
					});

					$scope.detailHref = $scope.getDetailURL();
				}

				$scope.isMultipleFixIt = function() {
					return $scope.fixItItems.length > 1;
				}

				$scope.isChart = function() {
					if ($scope.tile.plotType === 'SOLID_GAUGE_CHART' || $scope.tile.plotType == 'PIE_CHART' ||
							$scope.tile.plotType == 'BAR_CHART' || $scope.tile.plotType == 'DONUT_CHART') {
						return true;
					}

					return false;
				}

				$scope.isText = function() {
					if ($scope.tile.plotType == undefined || $scope.tile.plotType === 'TEXT') {
						return true;
					}

					return false;
				}

				$scope.isIconText = function() {
					if ($scope.tile.plotType === 'ICON_TEXT') {
						return true;
					}

					return false;
				}

				$scope.isSolidDonutChart = function() {
					return $scope.tile.plotType === 'SOLID_GAUGE_CHART';
				}

				$scope.isDonutChart = function() {
					return $scope.tile.plotType === 'DONUT_CHART';
				}

				$scope.isPieChart = function() {
					return $scope.tile.plotType === 'PIE_CHART'
				}

				$scope.isBarChart = function() {
					return $scope.tile.plotType === 'BAR_CHART'
				}

				$scope.isCustomReport = function() {
					return $scope.isCR;
				}

				$scope.isTitleApplicable = function() {
					if ($scope.tile.tileId == 17 || $scope.tile.tileId == 12 || $scope.tile.tileId == 58 ||
							$scope.tile.tileId == 59 || $scope.tile.tileId == 65) { //SLA, Strike count, VMs Need attention
						for (var i = 0; i < $scope.tile.outcomes.length; i++) {
							if ($scope.tile.outcomes[i].outcomeType !== 'Title') {
								return false;
							}
						}
					}

					return true;
				}

				$scope.isDetailAvailable = function() {
					if ($scope.tile.detailReportName || ($scope.tile.surveyId && $scope.tile.surveyId != -1)) {
						return true;
					} else {
						return false;
					}
				}

				$scope.showOptions = function() {
					$scope.isShowOptions = true;
				}

				$scope.hideOptions = function() {
					$scope.isShowOptions = false;
				}

				$scope.isHalfTile = function() {
					return ($scope.tile.tileProperty && $scope.tile.tileProperty.height === 'HALF');
				}

				$scope.isFullTile = function() {
					return ($scope.tile.tileProperty && $scope.tile.tileProperty.height === 'FULL');
				}

				$scope.addAlert = function() {
					var paramsTable = healthUtilService.getParamsTable();
					Reports.configureAlarm({
						"paramId" : $scope.tile.tileId,
						"paramName" : $scope.tile.tileName,
						"type" : Alarms.TYPE_HEALTH,
						"paramsTable" : paramsTable,
						"manageLink" : true,
						"isTiledHealth" : $scope.tile.surveyId == undefined,
						"alertCriteria" : {
							"Parameter" : $scope.tile.tileName,
							"criteria" : $scope.tile.alertCriteria,
							"Editable" : 'Yes'
						}
					});
				}

				$scope.hideTile = function() {
					healthService.hideTile($scope.tile.tileId).then(function(resp) {
						$scope.addToHiddenTileList($scope.tile.tileId, $scope.tile.tileName);
						$scope.tilePositionIndex = 0;
						$scope.updateTilePositions($scope.tile.groupId, $scope.tile.tileId);
					}, function(error) {
						console.log("Failed to hide tile.");
						cvUtil.errorToast("Failed to hide tile.");
					});
				}

				$scope.getDetailURL = function() {
					var params = "", url = "";

					if (!$scope.isDetailAvailable()) {
						return url;
					}

					if ($scope.tile.detailReportName) { //open Custom Report
						url = cvUtil.getContextPath() + "/reportsplus/reportViewer.jsp?reportId=" +

						$scope.tile.detailReportName;
					} else if ($scope.tile.surveyId && $scope.tile.surveyId != -1) {
						if (cvUtil.getParameter("commUniId")) {
							params = "&commUniId=" + cvUtil.getParameter("commUniId");
						}

						if (cvUtil.getParameter("ccGroupId")) {
							params += "&ccGroupId=" + cvUtil.getParameter("ccGroupId");
						}

						var path = cvUtil.getContextPath() + "/survey/reports/indcommcellsurvey.jsp";
						var activityReports = [ 23, 25, 30 ];
						if (activityReports.indexOf($scope.tile.surveyId) !== -1) {
							path = cvUtil.getContextPath() + "/survey/reports/operations.jsp";
						}

						url = path + "?surveyId=" + $scope.tile.surveyId + params + "&trigger=health";
					}

					return url;
				}

				$scope.viewDetails = function() {
					var url = $scope.getDetailURL();

					if (url.length > 0) {
						window.open(url, "_blank");
					}
				}

				$scope.isDetailApplicable = function() {
					if ($scope.tile.detailReportName || ($scope.tile.surveyId && $scope.tile.surveyId != -1)) {
						$scope.detailTooltip = localMsg.Health_View_Details_Tooltip;
						return true;
					}
					return false;
				}

				$scope.showFixItMenu = function($event) {
					angular.element("#tileFixMenu_" + $scope.tile.tileId).show();
					$event.stopPropagation();
				}

				$scope.runFixIt = function($event, fixName, fixId) {
					angular.element(".fixActionMenuDiv").hide();
					$event.stopPropagation();
					if (fixName == undefined) {
						if ($scope.fixItItems.length > 0) {
							fixName = $scope.fixItItems[0].fixName;
						}
					}

					if (fixName) {
						return openForm({
							'workflowName' : fixName,
							'workflowId' : fixId,
							'inputs' : {
								'Client' : 'expressserver',
								'Agent' : 'SQL Server',
								'Subclient' : 'default',
								'subclientID' : '7'
							}
						});
					}
				}

				$scope.hasNoFixit = function() {
					return $scope.fixItItems.length == 0;
				}

				function isFixItFound(fixId) {
					for (var i = 0; i < $scope.fixItItems.length; i++) {
						if ($scope.fixItItems[i].fixId == fixId) {
							return true;
						}
					}

					return false;
				}

				function getOutcomeLabels() {
					var labels = [];
					if ($scope.tile.outcomes) {
						$.each($scope.tile.outcomes, function(index, outcome) {
							if (outcome.outcomeType === "Title") {
								var label = "";
								if (outcome.key != undefined) {
									label = outcome.key;
								}

								if (outcome.value != undefined) {
									if (label !== "") {
										label += ": ";
									}
									label += outcome.value;
								}
								labels.push(label);
							}
						});
					}
					return labels;
				}

				$scope.setRemarks = function(remark) {
					var label = $("#remarks_" + $scope.tile.tileIndex + " label");
					label.html(remark);

				}

				$scope.init();

			} ]);
}());