TABLESIZE_THRESHOLD_FOR_PDFQUALITY = 1000;
MAX_ACTIVE_PAGES = 10;
DEFAULT_MAX_TIMEOUT_MILLI_SEC = 60000;
DEFAULT_MAIN_PAGE_TIMEOUT = 15 * 60 * 1000;
DEFAULT_SUB_PAGE_TIMEOUT = 5 * 60 * 1000
phantom.cookiesEnabled = true;

phantom.onError = function(msg, trace) {
	var msgStack = [ 'PHANTOM ERROR: ' + msg ];
	if (trace && trace.length) {
		msgStack.push('TRACE:');
		trace.forEach(function(t) {
			msgStack.push(' -> ' + (t.file || t.sourceURL) + ': ' + t.line +
					(t["function"] ? ' (in function ' + t["function"] + ')' : ''));
		});
	}
	console.error(msgStack.join('\n'));
};

//global variables
var fs = require('fs');
var successStatus = 0;
var outputFileName="";
var outputType = "";
var screenshot = {};
var pageLoadedFirstTime = true;	//this is used for count
var pageAlreadyLoadedArray = [];
var linksLoaded = 0;
var totalRowsInPage = 0;
var heights = [];

if (console) {
	console.error = function(msg) {
		console.log('Error: ' + msg);
	};
}


var phantomExportUtility = (function(){
	var phantomExportUtility = {};

	var pageArray = [];
	var countActivePages = 0;
	var currentPage = 0;
	var system = require('system');

	var start, condition, interval;

	phantomExportUtility.init =function(){
		var JSONInput = fs.read(system.args[1]);
		fs.remove(system.args[1]);
		var fileOptionsJSON = system.args[2];
		fileOptionsJSON = fileOptionsJSON.replace(/\\"/g, '"');
		console.log('Json is: ' + JSONInput);
		console.log('fileOptionsJSON : ' + fileOptionsJSON);

		screenshot = JSON.parse(JSONInput);
		var fileOptions = JSON.parse(fileOptionsJSON);
		screenshot.outputFileName=fileOptions.outputFileName;
		outputFileName = screenshot.outputFileName;
		console.log('outputFileName = '+outputFileName);
		outputType= outputFileName.substr(outputFileName.lastIndexOf(".") + 1);
		return;

	};

	phantomExportUtility.loadPage = function(url, onPageLoadCallBack) {
		pageArray.push({
			'url' : url,
			'onPageLoadCallBack' : onPageLoadCallBack
		});
		if (countActivePages < MAX_ACTIVE_PAGES) {
			actualLoadPage();
		}
	};

	phantomExportUtility.closePage = function(page) {
		page.close();
		countActivePages--;
		console.log("Active pages :" +countActivePages);
		actualLoadPage();
		if(countActivePages==0)
			phantomExportUtility.exitPhantom(successStatus);
	}


	phantomExportUtility.exitPhantom = function(exitCode) {
		setTimeout(function() {
			phantom.exit(exitCode);
		}, 0);
	};

	phantomExportUtility.waitFor = function(testFx, onReady, timeOutMillis) {

		var maxtimeOutMillis = timeOutMillis ? timeOutMillis : DEFAULT_MAX_TIMEOUT_MILLI_SEC;

		start = new Date().getTime();
		var condition = false;
		var interval = setInterval(function() {
			if ((new Date().getTime() - start < maxtimeOutMillis) && !condition) {
				// If not time-out yet and condition not yet fulfilled
				condition = (typeof (testFx) === "string" ? eval(testFx) : testFx()); //< defensive code
			} else {
				if (!condition) {
					// If condition still not fulfilled (timeout but condition is 'false')
					console.error("'waitFor()' timeout");
					phantomExportUtility.exitPhantom(1);
					return;
				} else {
					// Condition fulfilled (timeout and/or condition is 'true')
					console.log("'waitFor()' finished in " + (new Date().getTime() - start) + "ms.");
					typeof (onReady) === "string" ? eval(onReady) : onReady(); //< Do what it's supposed to do once the condition is fulfilled
					clearInterval(interval); //< Stop this interval
				}
			}
		}, 250); //< repeat check every 250ms
	}

	phantomExportUtility.checkforErrorOnPage = function(status, page){
		console.error("Page url is:" + page.url);
		if (status !== 'success') {
			console.error("Pageload status is " + status + ".\n Exiting with return code 1");
			successStatus=1;
		}else if(page.url.indexOf('noPermission.jsp') != -1 || page.url.indexOf('/applications/') != -1) {
			console.error("loaded page is nopermission.jsp or /aapplications/ ...exiting");
			successStatus=403;
		}else {
			var pageAttrs = page.evaluate(function() {
				pageAttrs = {};
				if (window.Reports && Reports.hasFatalError && Reports.hasFatalError()) {
					pageAttrs.error = true;
					pageAttrs.errorMessage = Reports.getFatalError();
				} else {
					pageAttrs.error = false;
				}
				return pageAttrs;
			});
			if (pageAttrs.error) {
				console.error("Error on report. " + pageAttrs.errorMessage + ". Exiting with error code 2");
				successStatus=2;
			}
		}
		return successStatus;
	}

	phantomExportUtility.createLocalLinks = function(server){
		console.log("Replacing localhost in href");
		$.each($("a:not(.crawledLink)").filter("[href*='//localhost']"), function(index, value) {
			var el = $(value);
			var h = el.attr("href").replace(/https?:\/\/localhost[:]?[\d]*/g, server);
			console.log("Replacing " + el.attr("href") + " with " + h);
			el.attr("href", h);
		});
		$('a:not(.crawledLink)').not('[href^="http"],[href^="https"],[href^="mailto:"],[href^="#"]')
		.each(function(index, value) {
			$(value).attr("href", server + value.pathname + value.search + value.hash);
		});
	}

	phantomExportUtility.writeFile =function(fs, page, outputFileName){
		console.log('writing html file at ' + "WEB-INF\\export\\temp\\" + outputFileName);
		var content = page.content;
		var jsonContent = page.evaluate(function() {
			var metaData = window.reportMetaData;
			return metaData ? JSON.stringify(metaData) : null;
		});
		var rscript = /<script\b[^<]*(?:(?!<\/script>)<[^<]*)*<\/script>/gi; //remove script tags
		var comments = /<!--[\s\S]*?-->/g; //removes html comments
		content = content.replace(rscript, '');
		content = content.replace(comments, '');
		fs.write("../../WEB-INF/export/temp/" + outputFileName + ".html", content, "w");
		if(jsonContent != null && jsonContent != undefined)
			fs.write("../../WEB-INF/export/temp/" + outputFileName + ".JSON", jsonContent, "w");
		console.log('Exiting with return code : '+successStatus);
	}

	function actualLoadPage() {
		if (pageArray.length <= currentPage) {
			return;
		}
		var url = pageArray[currentPage].url, onPageLoadCallBack = pageArray[currentPage].onPageLoadCallBack;
		countActivePages++;
		currentPage++;

		var page = new WebPage();
		var results = (new RegExp("[\\?&]viewportWidth=([^&#]*)")).exec(url);
		var viewportWidth = "1720";
		if (results && results.length > 1) {
			viewportWidth = results[1];
		}
		if (screenshot.viewportWidth || viewportWidth) {
			page.viewportSize = {
					width : parseInt(screenshot.viewportWidth || viewportWidth),
					height : 800
			};
		}
		if (screenshot.operatorCompanyId) {
			page.customHeaders = {
				"operatorCompanyId" : screenshot.operatorCompanyId
			};
		}
		page.settings.userAgent = 'Simpana PhantomJS';
		page.onConsoleMessage = function(msg, line, source) {
			console.log(source + " " + line + ":" + msg);
		};
		page.onError = function(msg, trace) {
			var msgStack = [ 'ERROR ON PAGE: ' + msg ];
			if (trace) {
				msgStack.push('TRACE:');
				trace.forEach(function(t) {
					msgStack.push(' -> ' + t.file + ': ' + t.line +
							(t["function"] ? ' (in function "' + t["function"] + '")' : ''));
				});
			}
			console.error(msgStack.join('\n'));
			//should phantom.exit(1) be called here also ??
			//phantom.exit() is not called so that pages with JS error could be processed
		};
		for (var i = 3; i < system.args.length; i++) {
			console.log(system.args[i]);
			var cookie = system.args[i].split("=");
			console.log("COOKIE NAME: " + cookie[0]);
			console.log("COOKIE VALUE: " + cookie[1]);
			phantom.addCookie({
				'name' : cookie[0],
				'value' : cookie[1],
				'domain' : screenshot.domain
			});
		}
		page.onLoadFinished = function(status) {
			setTimeout(onPageLoadCallBack(status, page), 500); //give the page 500 ms to load any js if it is
		};
		console.log("opening: ");
		console.log(url);
		page.open(url);
	}
	return phantomExportUtility;
}());

try {
	console.log("started parsing params");
	phantomExportUtility.init();
	console.log("url : " + screenshot.url);
	phantomExportUtility.loadPage(screenshot.url, mainPageLoadCallBack);
} catch (e) {
	console.error("Error while initializing page in screenshot.js" + e.name + " " + e.message + "\n" + e.stack);
	console.error("Exiting with return code 1");
	exitPhantom(1);
}

function mainPageLoadCallBack(status, page) {
	//this is called for each frame/iframe, so execute only once
	if (!pageLoadedFirstTime) {
		pageLoadedFirstTime=false;
		return;
	}

	try {
		console.log('customReportExport:::mainPageLoadCallBack(): main page loaded with status: '+status);
		successStatus = phantomExportUtility.checkforErrorOnPage(status, page);
		if( successStatus!=0){
			phantomExportUtility.exitPhantom(successStatus);
		}

		phantomExportUtility.waitFor(function(){
			return page.evaluate(function() {
				var retVal = true;
				console.log('customReportExport::mainPageLoadCallBack():waitFor ::window.jQuery = '+window.jQuery);
				console.log('customReportExport::mainPageLoadCallBack():waitFor ::window.allOnReadiesExecuted = ' + window.allOnReadiesExecuted);
				console.log('customReportExport::mainPageLoadCallBack():waitFor ::$.active = '+ $.active);
				if (!window.jQuery || !window.allOnReadiesExecuted || $.active > 0) {
					console.log('customReportExport::mainPageLoadCallBack(): ::waitFor ::condition is...false');
					return false;
				}
				var loadmask = $(".loadmask:visible");
				console.log('customReportExport::mainPageLoadCallBack():waitFor ::loadmask.length = ' + loadmask.length);
				retVal = retVal && loadmask.length == 0;
				var datatables = $(".dataTable:visible:not(.drawn)").length == 0;
				console.log('customReportExport::mainPageLoadCallBack():waitFor ::datatables.length = '+ $(".dataTable:visible:not(.drawn)").length);
				retVal = retVal && datatables; //this assumes that when all the tables are drawn completely, we consider page complete. reportsbase.js adds drawn class ontablerendercallback
				console.log('customReportExport::mainPageLoadCallBack():waitFor ::condition is...' + retVal);
				return retVal;
			});
		}, function() {
			//this will be used to determine type of PDf converter that will be used for this request
			totalRowsInPage += page.evaluate(function() {
				return $(".dataTable tbody tr").filter(':visible').length;
			});
			page.evaluate(function(server , createLocalLinks) {
				$("body").addClass("exportfriendly");
				createLocalLinks(server);
				$("#reportArea .gridster-loaded li.gridster-item").css("vertical-align","top");
				localStorage.clear();
			}, screenshot.server, phantomExportUtility.createLocalLinks);
			var pageLinks=[];
			if (outputType == "pdf") {
				pageLinks = getPageLinks(page,outputType, ".healthDetailExport", "data-position");
				pageLinks.forEach(
						function(value, index, a) { //create html file for each followforlink url
							console.log(index+"  URL:"+value);
							phantomExportUtility.loadPage(value, processSubPage);
						});
			}
			if (totalRowsInPage > TABLESIZE_THRESHOLD_FOR_PDFQUALITY) {
				successStatus = 11; //there is no confirmation in documentation but in all likelihood and assumption, this is not being used internally phantomjs
			}
			var waitForTime = 2 * 1000;
			var loadMaskTimerStarted = false;
			var start = 0;
			phantomExportUtility.waitFor(function() {
				var retVal = pageLinks.length == linksLoaded;
				console.log('waitfor condition is...' + retVal + ". Number of links loaded:" + linksLoaded);
				 if (screenshot.isCustomAlarmv2 == 'true') {
					 retVal = page.evaluate(function(outputType) {
						 if(outputType === 'csv'){
							 console.log("Pending csv file download from cre. " + window.csvFileDownloaded);
							 return window.csvFileDownloaded;
						 }else{
							 console.log("Pending requests " + window.pendingRequests);
							 return window.pendingRequests <= 0;
						 }
					 },outputType);
				 }
				 console.log('phantomExportUtility.waitFor::retVal = '+retVal);
				 if (retVal) {
					 if (!loadMaskTimerStarted) {
						 // start waitTimer for 2 seconds.
						 start = new Date().getTime();
						 loadMaskTimerStarted = true;
						 console.log('waitfor condition is...' + retVal + ". Number of links loaded:" + linksLoaded + ", waiting for 2000ms for any UI rendering to finish");
					 }
					 var now = new Date().getTime();
					 if (now - start < waitForTime) {
						 console.log('Waiting for 2000ms, covered so far:' + (now - start) + ", returning false");
						 return false;
					 } else {
						 console.log('Finished waiting for 2000ms...');
					 }
				 }
				return retVal;
			}, function() {

				page.evaluate(function(outputFileName, heights,codeHackwkhtmltopdf) {
					$("svg a").contents().unwrap();
					/*****************************************************************************************
					 * Start code hack to get around this bug of wkhtmltopdf
					 * https://github.com/wkhtmltopdf/wkhtmltopdf/issues/1524
					 ****************************************************************************************/
					 codeHackwkhtmltopdf();
					/*****************************************************************************************
					 * End of code hack to get around this bug of wkhtmltopdf
					 * https://github.com/wkhtmltopdf/wkhtmltopdf/issues/1524
					 ****************************************************************************************/
					for (var i = 0; i < heights.length; i++) {
						var height = heights[i];
						console.log('creating iframe');
						var crawled = $('<iframe style="height:' + height +
								'px;" scrolling="no" frameborder="0" class="crawled" id="crawled' + i + '"></iframe>');
						crawled.attr("src", "mhtml.do?file=" + outputFileName + "test" + i + ".html");
						var d = $('<div class="crawledWrapper"></div>').append(crawled);
						$("#container").append(d);
					}
				}, outputFileName, heights,codeHackwkhtmltopdf );

				try {
					console.log('Main page and all links loaded, writing main page');
					if (outputType == 'csv' &&  screenshot.isCustomAlarmv2 == 'false') {
						isCSVMetaDataHidden = (screenshot.HideCSVAdditionalInfo && screenshot.HideCSVAdditionalInfo == "true" ) ? true : false ; //HideCSVAdditionalInfo key had to be converted to string when inserting to Hashmap in Java ; Reversing to boolean
						screenshot.isAlarm = screenshot.isAlarm == true || screenshot.isAlarm == "true" ? true : false;
						if (screenshot.isAlarm) {
							var ndx = screenshot.url.indexOf("filteredTable-0=") > -1 ? (screenshot.url
									.indexOf("filteredTable-0=") + "filteredTable-0=".length) : -1;
							var nxtAnd = screenshot.url.indexOf("&", ndx);
							var tableSelected = "";
							if (ndx > -1) {
								if (nxtAnd > -1) {
									tableSelected = screenshot.url.substring(ndx, nxtAnd);
								} else {
									tableSelected = screenshot.url.substring(ndx);
								}
							} else {
								tableSelected = "table_0";
							}
							console.log("TableSelected : " + tableSelected);
							var tableId = tableSelected + "_table";
							csv(page,
									screenshot.outputDir + screenshot.outputFileName,
									screenshot.isAlarm,
									tableId,
									screenshot.isReportMetadataCall,isCSVMetaDataHidden);
						} else {
							csv(page,
									screenshot.outputDir + screenshot.outputFileName,
									false,
									0,
									screenshot.isReportMetadataCall,isCSVMetaDataHidden);
						}
					} else if (outputType == 'pdf') {
						if(screenshot.isCustomAlarmv2 == 'true' || screenshot.isCustomAlarmv2 == true){
							/*****************************************************************************************
							 * Start code hack to get around with page size restriction in phantomJS
							 ****************************************************************************************/
							page.evaluate(function() {
								var reportTables = $('.reportTable:visible');
									if (reportTables && reportTables.length > 0) {

										// Demarcate the table contents for processing at the backend.
										$('tbody[role=rowgroup]', reportTables).each(function() {
											var tableId = $(this).parents('.reportTable').attr('id');
											$(this).before('<input type="hidden" name="table_' + tableId + '-TBODY_BEGIN" />')
											.after('<input type="hidden" name="table_' + tableId + '-TBODY_END" />');
										});

										$("#content #reportArea #pageDiv div[data-gridster]").css({
											height : 'auto'
										});
										$.each($("#content #reportArea #pageDiv div[data-gridster] ul li[comp]"), function(
												index, value) {
											$(value).css({
												position : 'static',
												height : 'auto',
												display:'inline-block',
												marginLeft : '10px'
											});
										});
									}
								$(".dataTables_paginate").hide();

							});
							/**************************************************************************************
							 *  End of code hack for phantomJS *
							 **************************************************************************************/
						}
						phantomExportUtility.writeFile(fs, page, outputFileName);
					}
					phantomExportUtility.closePage(page);
				} catch (e) {
					console.error("Error while exporting after page load" + e);
					console.error("Exiting with error code 1");
					exitPhantom(1);
				}
			},DEFAULT_MAIN_PAGE_TIMEOUT);
		}, DEFAULT_SUB_PAGE_TIMEOUT);//5 min. timeout - end waitfor
	} catch (e) {
		console.error("Error on pageload " + e);
		console.error("Exiting with error code 1");
		phantomExportUtility.exitPhantom(1);
	}
};

function getPageLinks(page,outputType, selectorClass, attributes){
	var pageLinks = page.evaluate(function(outputType,selectorClass, attributes) {
		var links = [];
		var allLinks = $(".followForExport:visible");
		var tilePositions = [];
		var domElements = $(selectorClass);

		if (domElements && domElements.length > 0) {
			//sort the links because in health the tiles are re-ordered through grid after DOM are created.
			$.each(domElements, function(i, tile) {
				tilePositions.push(parseInt($(tile).attr(attributes)));
			});
			tilePositions = tilePositions.sort(function(a, b) {
				return a - b;
			});
			$.each(tilePositions, function(i, pos) {
				for (var idx = 0; idx < domElements.length; idx++) {
					var tile = domElements[idx];
					if (pos === parseInt($(tile).attr(attributes))) {
						allLinks.push(tile);
						console.log("pushing tile, name: " + $(tile).attr("data-name"));
						break;
					}
				}
			});
		}
		$.each(allLinks, function(index, value) {
			var v = value.href;
			if (window.convertToExportUrl) {
				v = convertToExportUrl(v, outputType, 100);
			}
			console.log("href:" + v);
			links.push(v);
			$(value).attr("href", "#crawled" + index);
			$(value).addClass("crawledLink");
		});
		return links;
	}, outputType,selectorClass, attributes);
	console.log('links loaded: '+pageLinks.length);
	return pageLinks;
}

function processSubPage(status, otherPage){
	console.log("url :"+otherPage.url+ "  loaded with " + status);
	successStatus = phantomExportUtility.checkforErrorOnPage(status, otherPage);
	if( successStatus!=0){
		phantomExportUtility.exitPhantom(successStatus);
	}
	phantomExportUtility.waitFor(
			function() {
				return otherPage.evaluate(function() {
					var retVal = true;
					if (!window.jQuery) {//Wait for jquery to load - Requires that target page has jquery
						console.debug('jQuery not loaded...');
						console.debug("location: " + this.location.href);
						return false;
					}
					if (!window.allOnReadiesExecuted) { //at the bottom of the page in footer.jsp, this variable is initialized
						console.debug('allOnReadies not executed...');
						console.debug("location: " + this.location.href);
						return false;
					}
					if ($.active > 0) { //any active ajax requests pending?
						console.debug('active AJAX exists...');
						console.debug("location: " + this.location.href);
						return false;
					}
					var loadmask = $(".loadmask:visible");
					retVal = retVal && loadmask.length == 0;
					var datatables = $(".dataTable:visible:not(.drawn)").length == 0;
					retVal = retVal && datatables; //this assumes that when all the tables are drawn completely, we consider page complete. reportsbase.js adds drawn class ontablerendercallback
					console.log('waitfor condition is...' + retVal);
					return retVal;
				});
			},
			function() {
				window.setTimeout(function() {
					for (var i = 0; i < pageAlreadyLoadedArray.length; i++) {
						if (pageAlreadyLoadedArray[i] == otherPage.url) {
							return;
						}
					}
					//check for index
					var height = otherPage.evaluate(function(server ,createLocalLinks, codeHackwkhtmltopdf) {
						createLocalLinks(server);
						/*****************************************************************************
						 * Start code hack to get around this bug of wkhtmltopdf
						 * https://github.com/wkhtmltopdf/wkhtmltopdf/issues/1524
						 ****************************************************************************/
						codeHackwkhtmltopdf();
						/*****************************************************************************
						 * END of code hack to get around the bug of wkhtmltopdf
						 ****************************************************************************/
						console.log('getting height');
						if (!document.body.className) {
							document.body.className = "";
						}
						$('a.disableOnEmbedded').contents().unwrap();
						document.body.className += " embedded exportfriendly";
						return document.body.scrollHeight;
					}, screenshot.server , phantomExportUtility.createLocalLinks, codeHackwkhtmltopdf);

					totalRowsInPage += otherPage.evaluate(function() {
						return $(".dataTable tbody tr").filter(':visible').length;
					});
					var index = heights.length;
					heights.push(height);
					var otherFileName = outputFileName + "test" + index;
					phantomExportUtility.writeFile(fs, otherPage, otherFileName);
					pageAlreadyLoadedArray.push(otherPage.url);
					phantomExportUtility.closePage(otherPage);
					linksLoaded++;
				}, 500); //end settimeout
			}, DEFAULT_SUB_PAGE_TIMEOUT);//5 min. timeout - end waitfor
}

// we do not need CSV to execute in phantom with new code
//it could be done in servlet layer using data model itself but then we'd lose any client side rendering
function csv(page, outFile, isAlarm, alarmOnTableId, isReportMetadataCall , hideCSVAdditionalInfo) {
	var fs = require('fs');
	var delimiter = ',';
	var nl = '\r\n';
	var csv = page.evaluate(function(delimiter, nl, isAlarm, alarmOnTableId, isReportMetadataCall,hideCSVAdditionalInfo) {
		console.log("exporting to csv");
		$(".hideInCsv").remove();
		window.htmlDecode = function(input) {return input;};
		//any report specific changes should be done by creating a function in reports js file and then calling that from here say, if(window.tasksummary) tasksummary(); and tasksummary can be defined in your js file
		var csv = '';

		function addLine(text) {
			if (text != '""') {
				csv += text + nl;
			}
		}
		function createField(text) {
			var field = text.trim().replace(/\"/g, "\"\"");
			if (isNaN(field)) {
				return '"' + field + '"';
			} else {
				return '"' + " " + field + '"'; //  space is given in the begining to stop Excel from formatting the data automatically
			}
		}
		var breadcrumb = '';
		$.each($("#breadcrumbMain a").filter(":visible"), function(index, value) {
			breadcrumb += htmlDecode($(value).text()) + " > ";
		});

		console.log("isAlarm " + isAlarm);
		console.log("isReportMetadataCall: " + isReportMetadataCall);

		if (!isAlarm) {
			csv += '\uFEFF';
		}
		if ((breadcrumb != '' && !isAlarm)) {
			addLine(createField(breadcrumb));
		}
		if (!isAlarm && !hideCSVAdditionalInfo) {
			if ($("#welcomeMsg").text() != null && $("#welcomeMsg").text().length != 0) {
				addLine(createField($("#welcomeMsg").text()));
			}

			if (isReportMetadataCall !== 'true') {
				if (($("#custName").text() != null && $("#custName").text().length != 0)) {
					addLine(createField($("#custName").text()));
				}
			}

			if (($("#surveyName").text() != null && $("#surveyName").text().length != 0)) {
				var title = $("#surveyName").text();
				addLine(createField(title));
			}
			var reportDesc = "";
			if ($('#reportDesc').length && $("#reportDesc").text() != null && $("#reportDesc").is(":visible")) {
				reportDesc = $("#reportDesc").text();

			}
			if (($("#sectionTitle").text() != null && $("#sectionTitle").text().length != 0)) {
				addLine(createField(htmlDecode($("#sectionTitle").text())));
			}
			if ($("#generatedOnText").is(":visible")) {
				if (($("#generatedOnText").text() != null && $("#generatedOnText").text().length != 0)) {
					addLine(createField(htmlDecode($("#generatedOnText").text())));
					csv += nl; // add an extra line after title
				}
			}
			if (($("#commcelltext").text() != null && $("#commcelltext").text().length != 0)) {
				addLine(createField(htmlDecode($("#commcelltext").text().trim())));

			}
			if (($("#asoftext").text() != null && $("#asoftext").text().length != 0)) {
				addLine(createField(htmlDecode($("#asoftext").text().trim())));
			}
			if (($("#infoDiv").text() != null && $("#infoDiv").text().length != 0)) {
				addLine(createField(htmlDecode($("#infoDiv").text().trim())));
			}
			var line = '';
			$.each($("#reportTaskSummary li").filter(":visible"), function(index, value) {
				if ($(value).hasClass("col1")) {
					csv += createField(line) + nl;
					line = htmlDecode($(value).text());
				} else {
					line += htmlDecode($(value).text());
				}
			});
			if (line.length > 0) {
				csv += createField(line) + nl;
			}

			if ($("#searchKeywordid").is(":visible")) {
				if (($("#searchKeywordid").text() != null && $("#searchKeywordid").text().length != 0)) {
					csv += nl;
					addLine(createField(htmlDecode($("#searchKeywordid").text())).trim().replace(/(\r\n|\n|\r)/gm, '')
							.trim());
				}
			}
			var line = '';
			if ($(".appliedFiltersDiv").is(":visible")) {
				csv += nl;
				csv += createField(htmlDecode($(".filterText").text()).trim());
				$.each($("#appliedFilterLst li").filter(":visible"), function(index, value) {
					var line = '';
					csv += nl;
					line += createField((htmlDecode($(value).text()).trim().replace(/(\r\n|\n|\r)/gm, '').trim()));
					csv += (line);
				});
				csv += nl;
			}
			var line = '';
			if ($(".additionalFilters").is(":visible")) {
				csv += nl;
				csv += createField(htmlDecode($(".additionalFilterText").text()).trim());
				$.each($("#additionalFilterLst li").filter(":visible"), function(index, value) {
					var line = '';
					csv += nl;
					line += createField((htmlDecode($(value).text()).trim().replace(/(\r\n|\n|\r)/gm, '').trim()));
					csv += (line);
				});
				csv += nl;
			}

		}

		var sectionTitle = "";
		var summaryExist = false;
		$.each($("table").filter(":visible"), function(index, table) {
			var tableId = $(table).attr("id");
			if (isAlarm && alarmOnTableId != tableId) {
				return true; // continue; $.each -- continue to next iteration
			}
			if (hideCSVAdditionalInfo && (tableId == "reportSummary" || $(table).parent("#statusRow").length > 0)) {
				summaryExist = true;
				return true;
			}
			var repSubtitlePresent = false;
			var newsectionTitle = $(table).closest(".repHolder").find(".repSubTitle").text();
			if (newsectionTitle.length > 0 && newsectionTitle != sectionTitle) {
				csv += nl + createField(newsectionTitle) + nl;
				sectionTitle = newsectionTitle;
			}
			if (newsectionTitle.length > 0) {
				repSubtitlePresent = true;
			}
			var tabletitle = $(table).attr("id");
			if (tabletitle && tabletitle.indexOf("_table") > -1 && !isAlarm && !hideCSVAdditionalInfo) {
				if (reportDesc) {
					csv += nl + reportDesc;
				}
				if (repSubtitlePresent) {
					csv += nl + delimiter + $("#" + $(table).attr("id").replace("_table", "_title")).text() + nl;
				} else if ($("#" + $(table).attr("id").replace("_table", "_title")).text() != null &&
						$("#" + $(table).attr("id").replace("_table", "_title")).text().trim() != "") {
					csv += nl + $("#" + $(table).attr("id").replace("_table", "_title")).text() + nl;
				} else {
					csv += nl;
				}

			}
			else if(hideCSVAdditionalInfo && index !=0){
				if (!(summaryExist && index == 1)) {
					csv += nl;
				}
			}
			if (isReportMetadataCall === 'true') {
				console.log("getting metadata");
				if ($(table).parent("#statusRow").length > 0) {
					//do nothing
				} else {
					console.log("table" + $(table), "tablename: " + table);

					var ar = $.map($(table).dataTable().fnSettings().aoColumns, function(colMetaData, index2) {
						console.log("metadata: " + colMetaData + ", " + index2 + ", " + colMetaData.sTitle + ", renderer:" + colMetaData.rendererType);
						return createField(htmlDecode(colMetaData.sTitle));
					});

					csv += ar.join(delimiter) + nl;
					ar = $.map($(table).dataTable().fnSettings().aoColumns, function(colMetaData, index2) {
						console.log("sType: " + colMetaData.sType + ", sqlType: " + colMetaData.sqlType);
						var colType = colMetaData.sType;
						if (colType == null) {
							if (colMetaData.sqlType != null) {
								colType = colMetaData.sqlType.toLowerCase();
							} else {
								colType = "string";
							}
						}

						if (colMetaData.rendererType === 'capacityMB' || colType == 'html' || colMetaData.rendererType === 'percentNA' ||
								colMetaData.rendererType === 'MMMDDYYYYNEVER' || colMetaData.rendererType === 'DifferenceMB' ||
								colMetaData.rendererType === 'DifferenceNA' ) {
							colType = "string";
						}
						return createField(htmlDecode(colType));
					});
					csv += ar.join(delimiter) + nl;
				}
			} else {
				if ($(table).parent("#statusRow").length > 0) {
					//do nothing
				}else{
					$.each($(table).find('tr'), function(index2, tr) {
						var ar = $.map($(tr).find('th'), function(th, index3) {
							return createField(htmlDecode($(th).text()));
						});
						if (repSubtitlePresent) {
							csv += delimiter + ar.join(delimiter);
						} else {

							csv += ar.join(delimiter);
						}
						var tid = $(tr).attr("id") || "";
						var fIndx = tid.indexOf("filterRow");
						if (fIndx == -1) {
							ar = $.map($(tr).find('td'), function(td, index3) {
								if (isAlarm) {
									if ($(td).hasClass("dataTables_empty")) {
										// no rows in the table.. return empty;
										csv = "";
										return "";
									}
									return createField(htmlDecode($(td).html())); //do not remove any inner html
								} else {
									var renderedHtml = $(td).find(".csvText"); //if there is a renderer with some html then text value should be inside span/div with class csvText which will be displayed in csv
									if (renderedHtml.length > 0) {
										return createField(htmlDecode(renderedHtml.text()));
									} else {
										return createField(htmlDecode($(td).text().replace(/<[^>]*>/g, ""))); //remove any inner html, just display text
									}

								}
							});
							csv += ar.join(delimiter) + nl;
						}
					});
				}
			}
		});
		if (!isAlarm) {
			if ($("#error_message").text() != null && $("#error_message").is(":visible")) {
				addLine(nl + createField($("#error_message").contents(':not(div)').text()));
			}

			if ($('.exportError').text() != null && $.trim($('.exportError').text()) != '') {
				addLine(nl + createField($('.exportError').text()));
			}
		}

		console.log("returning csv string: " + csv);
		return csv;
	}, delimiter, nl, isAlarm, alarmOnTableId, isReportMetadataCall,hideCSVAdditionalInfo);
	fs.write(outFile, csv, "w");
	try {
		var columnModel = page.evaluate(function() {
			if (window.Reports) {
				return Reports.columnModelForExport;
			}
		});
		if (columnModel) {
			fs.write(screenshot.outputDir + screenshot.sessionid + 'temp.txt', JSON.stringify(columnModel), "w");
		}
	} catch (e) {
		console.error("Error while writing columnModel for big export " + e);
		console.error("Ignoring error and continuing with export");
	}
}

//check if we still need this .Not needed for the new kendo grid. So adding one more selector to make it only for ngtable. Will have to remove this once we move completely to the new grid.
function codeHackwkhtmltopdf(){
	$.each($(".reportTable.reportView thead"), function(index, value) {
        var el = $(value);
        el.next().prepend(el.contents());
        el.remove();
    });
    $(".reportTable.reportView th").each(function(index) {
        var thisTD = this;
        var newElement = $("<td></td>");
        $.each(this.attributes, function(index) {
            $(newElement).attr(thisTD.attributes[index].name, thisTD.attributes[index].value);
        });
        $(newElement).addClass('tableheadertd');
        $(newElement).html($(this).html());
        $(this).after(newElement).remove();
    });
}