function CommCellGroupEditor() {

	this.report = new Reports.initReport({
		type : 'commCellGroup',
		reportTitle : 'Create CommCell Group',
		scaleReport : true

	});

	this.requestString = report.requestString;

	const ASSOCIATE_TO_COMMCELL_GROUP = "Associate to CommCell Group";
	const COMMCELL_NAME = "CommCell Name";
	const COMMCELL_ID = "CommCell ID";
	const COMMCELL_ID_INT = "CommCell ID INT";
	const DISPLAY_NAME = "Display Name";
	const COMMCELL_GUID = "CommCell Guid";
	const GROUP_RESPONSE = "groupResponse";
	const COMMCELL_LIST_RESPONSE = "commCellListResponse";
	const ASSOCIATE_USER_TO_COMMCELL_GROUP = "Associate User to CommCell Group";
	const DELETE_USER_ACTION = "Action";
	const ASSOCIATED = "associated";
	const USER_NAME = "User Name";
	const USER_EMAIL = "User Email";
	const DONE_TYPING_INTERVAL= 300;
	const USERS_TABLE = "commCellGroupUser";
	const COMMCELLS_TABLE = "commCellGroup";
	const PREFIX = "Prefix";
	const CV_PREFIXES = ["CASP Partner", "CV", "CVRMS", "ESP", "UTL"];


	var commCellGroupController = {};
	commCellGroupController.name = "";
	commCellGroupController.groupdescription = "";
	commCellGroupController.ccGroupId = -1;
	commCellGroupController.commcellHolder = {};
	commCellGroupController.responseCtr = 0;
	commCellGroupController.responses = {};
	commCellGroupController.isEdit = false;
	commCellGroupController.savedCommCells = {};
	commCellGroupController.selectedCommCells = {};
	commCellGroupController.savedUsers = {};
	commCellGroupController.selectedUsers = {};
	var allPrefixList = [];
	var typingTimer=0;                //timer identifier


	this.report.initComponentState = function() {
		console.log("editor initComponentState");

	};

	report.beforeLoadReport = function() {
		console.log("editor beforeLoadReport");
	}

	report.initializeDefaults = function() {
		console.log("editor initialize");
	}

	commCellGroupController.checkLength = function(o, min, max, idName) {
		if (o.val().length > max) {
			var msg = "";
			msg = cvUtil.cvLocalize('CommCell_group_length_check_error_message', idName, max);
			$('#validateTips').text(msg);
			o.addClass("ui-state-error");
			setTimeout(function() {
				o.removeClass("ui-state-error", 3500);
			}, 500);
			return false;
		} else if (o.val().length < min) {
			$('#validateTips').text(cvUtil.cvLocalize('Please_Enter_CommCell_group_name'));
			o.addClass("ui-state-error");
			setTimeout(function() {
				o.removeClass("ui-state-error", 3500);
			}, 500);
			return false;
		} else {
			return true;
		}
	};



	commCellGroupController.init = function() {

		$("#prefixMenuWrapper").hide();


		$("#commCellGroup .tableFilter").click(function(event) {
			let gridId = $(this).attr('id');
			gridId = gridId.substring(0, gridId.lastIndexOf("_"));
			Reports.filterClick(gridId, 'toggle');
		});

		$("#commCellGroup #commcellsTab").click(function(event) {
			$("#usersTab").removeClass("selected");
			$("#userList").hide();
			$(this).addClass("selected");
			$("#commcellList").show();
		});

		$("#commCellGroup #usersTab").click(function(event) {
			$("#commcellsTab").removeClass("selected");
			$("#commcellList").hide();
			$(this).addClass("selected");
			$("#userList").show();
		});


		$("#prefixWrapper").on("click",  function(e) {
			$("#prefixMenuWrapper").show();
			$("#commCellGroup #prefixWrapper").removeClass("invalidEntry");
			e.stopPropagation();
		});



		$("#commCellGroup #companyName").on("click", function(e) {
			$("#commCellGroup #nameWrapper").removeClass("invalidEntry");
		});

		$("#commCellGroup #searchBox").on("keyup", function (evt) {
			clearTimeout(typingTimer);
			let search = $("#searchBox").val();
			typingTimer = setTimeout(function(){
				commCellGroupController.loadCompanies(search);
		    }, DONE_TYPING_INTERVAL);

		});

		$("#commCellGroup #clearSearchBox").on("click", function(e) {
			$("#searchBox").val("");
			commCellGroupController.loadCompanies("");
			$("#prefixMenuWrapper").hide();
			e.stopPropagation();
		});

		$("#commCellGroup #prefixMenuWrapper").on("click", ".prefix-item", function(e) {
			$("#prefixMenuWrapper").hide();
			$("#ccPrefix").val($(this).text());
			e.stopPropagation();
		});

		$(document).keyup(function(e) {
			if (e.key === "Escape") {
				$(".dropdown-wrapper").hide();
			}
		});

		$("body").on("click", function(e) {
			$(".dropdown-wrapper").hide();
		});

		$("#commCellGroup #submit_new_group_Details").click(function(event){
			commCellGroupController.submitRequest();
		});


		$("#userList #addCompanyUser").click(function(event) {
			commCellGroupController.showAddNewUserDialog();
		});

		$('#commcellList').on('change', 'input.associatedToGroup:checkbox', function(event) {
			let tr = $(this).closest('tr');
			let data = $("#commCellGroup_table").dataTable().fnGetData(tr[0]);

			if ($(this).is(':checked')) {
				//Add selected commcell to holder
				let commCellInfo = {};
				commCellInfo.commCellId = data[COMMCELL_ID_INT];
				commCellInfo.csGUID = data[COMMCELL_GUID];
				commCellInfo.commCellName = data[COMMCELL_NAME];
				commCellGroupController.selectedCommCells[commCellInfo.csGUID] = commCellInfo;
			} else {
				//Remove un-selected commcell from holder
				delete commCellGroupController.selectedCommCells[data[COMMCELL_GUID]];
			}
		});

		$("#userList").on("click", ".deleteUser", function(event) {
			let tr = $(this).closest('tr');
			let data = $("#commCellGroupUser_table").dataTable().fnGetData(tr[0]);
			delete commCellGroupController.selectedUsers[data[USER_EMAIL]];
			$("#commCellGroupUser_table").dataTable().fnDeleteRow(tr[0]);
		});

		$("#commCellGroup #commcellsTab").addClass("selected");
		$("#commCellGroup #userList").hide();
	}

	commCellGroupController.loadData = function() {
		console.log("load data");
		let ccGroupId = cvUtil.getParameter("companyId");
		let name = window.decodeURIComponent(cvUtil.getParameter("name"));
		let prefix = window.decodeURIComponent(cvUtil.getParameter("prefix"));
//		let isBillingGroup  = cvUtil.getParameter("bg");
		let title = cvUtil.cvLocalize('Commcell_group_Create_dialog_title');

		if (ccGroupId) {
			title = "Edit CommCell Group - " + name;
			commCellGroupController.isEdit = true;
			if (prefix && name.indexOf(prefix) != -1) {
				name =  name.substr(prefix.length + 3);
			}
			$("#companyName").val(name);
			$("#companyId").val(ccGroupId);
//			if (isBillingGroup) {
//				$("#isBillingGroup").click();
//			}

			if (prefix) {
				$("#ccPrefix").val(prefix);
			}
			$("#nameLabel").hide();
			$("#prefixWrapper").hide();
			$("#nameWrapper").hide();
			$("#commCellGroupbuttons").addClass("edit");
			$("#commcellGroupDetails").addClass("edit");

		} else {
			commCellGroupController.addLoggedInUser();
		}
		$('#welcomeMsg').html(title);

		cvUtil.mask();
		//load commcells associated to the group
		cvUtil.loadPageWithPOST(cvUtil.getContextPath() +
				"/survey.do?action=CommCellsForSearch&searchType=1&pageSize=-1&companyId=" + ccGroupId, null, function(data) {

			commCellGroupController.addResponse(GROUP_RESPONSE, data);

		}, function(e) {
			cvUtil.errorToast(localMsg.commcellListError);
		});

		//load all commcells
		cvUtil.loadPageWithPOST(cvUtil.getContextPath() +
				"/survey.do?action=CommCellsForSearch&searchType=1&pageSize=-1", null, function(data) {
			commCellGroupController.addResponse(COMMCELL_LIST_RESPONSE, data);
		}, function(err) {
			cvUtil.errorToast(localMsg.commcellListError);
		});


		//load users associated to the group
		cvUtil.loadPageWithPOST(cvUtil.getContextPath() + "/reports/getCustomerUsers.do", {
			companyId : ccGroupId
		}, function(data) {
			if (data && data.data && data.data.ccusers && data.data.ccusers.length > 0) {
				commCellGroupController.processUserResponse(data.data.ccusers[0].users);
			}

		}, function(e) {
			cvUtil.errorToast(localMsg.userListError);
		});

		commCellGroupController.loadCompanies("");

	}

	commCellGroupController.addLoggedInUser = function() {
		let email = window.sessionStorage.getItem('userEmail');
		let userInfo = {
				name : loggedInUserName,
				email : email,
				isCreator : 1,
				operation : 1
		}
		commCellGroupController.selectedUsers[email] = userInfo;

		let tableRows = [];
		let row = {};
		row[USER_NAME] = userInfo.name;
		row[USER_EMAIL] = userInfo.email;
		row[DELETE_USER_ACTION] = 2;
		tableRows.push(row);
		commCellGroupController.createUserTable(tableRows);
	}

	//load companies
	commCellGroupController.loadCompanies = function(keyword) {
		commCellGroupController.clearPrefixDropDown();

		if (isCommvaultUser) {
			setPrefixList(CV_PREFIXES);
			$("#prefixMenuWrapper").addClass("cv-user");
			$("#prefixSearchWrapper").addClass("cv-user");
			$("#ccPrefix").addClass("cv-user")
			return;
		} else {
			$("#prefixMenuWrapper").removeClass("cv-user");
			$("#prefixSearchWrapper").removeClass("cv-user");
			$("#ccPrefix").removeClass("cv-user");
		}

		cvUtil.loadPageWithPOST(cvUtil.getContextPath() + "/survey.do?action=CommCellsForSearch&searchType=2&pageSize=-1&keyword=" + keyword, null, function(data) {
			$.each(data, function(i, c) {
				allPrefixList.push(c.label);
			});

			setPrefixList(allPrefixList);
		},
		function(err) {
			console.log("Error loading companies.")
		});
	}

	commCellGroupController.addResponse = function(key, value) {
		commCellGroupController.responseCtr++;
		commCellGroupController.responses[key] = value;

		if (commCellGroupController.isEdit && commCellGroupController.responseCtr < 2) {
			return;
		}

		cvUtil.unmask();
		commCellGroupController.processResponse();
	}

	commCellGroupController.processResponse = function() {
		let tableData = {
			iDisplayLength : 100,
			aoColumns : getColumnDefs(),
			aaData : [],
			iTotalDisplayRecords : 0,
			iTotalRecords : 0
		}
		let tableRows = [];
		let respData = null;
		let ccMap = [];

		if (commCellGroupController.isEdit) {
			respData = commCellGroupController.responses[GROUP_RESPONSE];
			$.each(respData, function(i, rowData) {

				let row = {};
				row[ASSOCIATE_TO_COMMCELL_GROUP] = 1;
				row[COMMCELL_NAME] = rowData.ccName;
				row[COMMCELL_ID_INT] = rowData.ccId;
				row[COMMCELL_ID] = (rowData.ccId.toString(16)).toUpperCase();
				row[COMMCELL_GUID] = rowData.ccGuid;
				row[DISPLAY_NAME] = rowData.label;
				ccMap.push(rowData.ccId);
				tableRows.push(row);

				let cc = {
						commCellName: rowData.ccName,
						commCellId: rowData.ccId,
						csGUID: rowData.ccGuid
				}
				commCellGroupController.savedCommCells[rowData.ccGuid] = cc;
				commCellGroupController.selectedCommCells[rowData.ccGuid] = cc;
			});
		}

		respData = commCellGroupController.responses[COMMCELL_LIST_RESPONSE];
		$.each(respData, function(i, rowData) {
			if(ccMap.indexOf(rowData.ccId) !== -1) {
				return;
			}
			let row = {};
			row[ASSOCIATE_TO_COMMCELL_GROUP] = 0;
			row[COMMCELL_NAME] = rowData.ccName;
			row[COMMCELL_ID_INT] = rowData.ccId;
			row[COMMCELL_ID] = (rowData.ccId.toString(16)).toUpperCase();
			row[COMMCELL_GUID] = rowData.ccGuid;
			row[DISPLAY_NAME] = rowData.label;
			tableRows.push(row);
		});

		tableData.aaData = tableRows;
		tableData.iTotalRecords = tableRows.length;
		tableData.iTotalDisplayRecords = tableRows.length;
		Reports.configureClientSideTableWithPaging(COMMCELLS_TABLE, "", tableData);

		$("#commCellGroup .rptTblBtnEncDiv").append($("#commCellGroup_table_length"));
//		$("#ccTableWrapper_FilterButton").trigger("click");
		$("#commCellGroup_FilterButton").trigger("click");
	}

	commCellGroupController.processUserResponse = function(users) {
		let tableRows = [];
		if (users) {
			$.each(users, function(i, user) {
				let creator = false;
				if (user.description === "1") {
					creator = true;
				}

				let row = {};
				row[USER_NAME] = user.associatedUserOrUserGroup.userName;
				row[USER_EMAIL] = user.email;
				row[DELETE_USER_ACTION] = creator ? 2 : 1;
				tableRows.push(row);

				let userInfo = {
					isCreator:  creator ? 1 : 0,
					name: user.associatedUserOrUserGroup.userName,
					email: user.email
				}
				commCellGroupController.savedUsers[user.email] = userInfo;
				commCellGroupController.selectedUsers[user.email] = userInfo;
			});
		}

		commCellGroupController.createUserTable(tableRows);
	}

	commCellGroupController.createUserTable = function(tableRows) {
		let tableData = {
			iDisplayLength : 100,
			aoColumns : getUserColumnDefs(),
			aaData : [],
			iTotalDisplayRecords : tableRows.length,
			iTotalRecords : tableRows.length,
			aaData: tableRows,

		}
		Reports.configureClientSideTable(USERS_TABLE, "", tableData);
	}

	commCellGroupController.clearPrefixDropDown = function() {
		allPrefixList= [];
		let list = $("#prefixList");
		list.empty();
	}

	commCellGroupController.submitRequest = function() {
		if(!commCellGroupController.isValidRequest()) {
			return;
		}

		let params = {};
		let companyPrefix = $("#ccPrefix").val().trim();
		let companyName =  $("#companyName").val().trim();

		if (companyPrefix) {
			params.companyName = encodeURIComponent(companyPrefix + ' - ' + companyName);
		} else {
			params.companyName = encodeURIComponent(companyName);
		}
		params.companyPrefix = encodeURIComponent(companyPrefix);

		params.commcells = [];
		params.users = [];
		params.isEdit = commCellGroupController.isEdit ? 1 : 0;

		if ($("#companyId").val() !== "") {
			params.companyId = $("#companyId").val();
		}

		if ($("#isBillingGroup").is(":checked")) {
			params.isBillingGroup = true;
		}

		// we will send only the delta. newly added commcell with operation 1, delete with operation 2
		let selectedCCs = [];
		for(let guid in commCellGroupController.selectedCommCells) {
			if (commCellGroupController.selectedCommCells.hasOwnProperty(guid)) {
				selectedCCs.push(guid);
				let ccData = commCellGroupController.selectedCommCells[guid];
				if (!commCellGroupController.savedCommCells[guid]) {
					ccData.operation = 1;
					params.commcells.push(ccData);
				}
			}
		}

		for(let guid in commCellGroupController.savedCommCells) {
			if (commCellGroupController.savedCommCells.hasOwnProperty(guid)) {
				let ccData = commCellGroupController.savedCommCells[guid];
				if (selectedCCs.indexOf(guid) == -1) {
					ccData.operation = 2;
					params.commcells.push(ccData);
				}
			}
		}


		let selectedUsers = [];
		for(let email in commCellGroupController.selectedUsers) {
			if (commCellGroupController.selectedUsers.hasOwnProperty(email)) {
				selectedUsers.push(email);
				let userData = commCellGroupController.selectedUsers[email];
				if (!commCellGroupController.savedUsers[email]) {
					userData.operation = 1;
					params.users.push(userData);
				}
			}
		}

		for(let email in commCellGroupController.savedUsers) {
			if (commCellGroupController.savedUsers.hasOwnProperty(email)) {
				let userData = commCellGroupController.savedUsers[email];
				if (selectedUsers.indexOf(email) == -1) {
					userData.operation = 2;
					params.users.push(userData);
				}
			}
		}

		cvUtil.mask();
		cvUtil.loadPageWithPOST(cvUtil.getContextPath() + "/createUpdateCompany.do", {
			params : JSON.stringify(params)
		}, function(data) {
			if (data && data.data && data.data.errorCode > 0) {
				cvUtil.errorToast(data.data.message);
			} else {
				cvUtil.toast(data.data.message);
				window.location.href = cvUtil.getContextPath()  + "/survey/reports/viewReport.jsp?reportId=RegisteredCompanies";
			}
			cvUtil.unmask();
		}, function(e) {
			cvUtil.unmask();
			cvUtil.errorToast(localMsg.companyError);
		})
	}

	commCellGroupController.isValidRequest = function() {
		if (!commCellGroupController.isEdit && $("#ccPrefix").val().trim() == "") {
			$("#prefixWrapper").addClass("invalidEntry");
			$("#ccPrefix").focus();
			cvUtil.errorToast(localMsg.errorPrefix);
			return false;
		}

		if ($("#companyName").val().trim() == "") {
			$("#nameWrapper").addClass("invalidEntry");
			$("#companyName").focus();
			cvUtil.errorToast(localMsg.errorName);
			return false;
		}

		if (JSON.stringify(commCellGroupController.selectedCommCells) == "{}") {
			cvUtil.errorToast(localMsg.errorCommcell);
			return false;
		}

		return true;
	}

	commCellGroupController.showAddNewUserDialog = function() {
		$("#addUserDialogWrapper").removeClass('hide');
		commCellGroupController.clearUserDialog();
		$("#add-user-dialog").dialog({
			title : cvUtil.cvLocalize("Add_User"),
			width : 350,
			height : 230,
			modal : true,
			autoOpen : true,
			resizable : false,
			buttons : [ {
				text : cvUtil.cvLocalize("Cancel"),
				class : "ui-close-button",
				click : function() {
					$(this).dialog("close");
					$(this).dialog("destroy");
				}
			}, {
				text : cvUtil.cvLocalize("user.add"),
				class : "ui-ok-button",
				click : function() {
					let userName = $("#add-user-dialog #userName").val();
					let email = $("#add-user-dialog #email").val();
					if (!commCellGroupController.validateNewUser(userName, email)) {
						return;
					}
					commCellGroupController.addUser(userName,  email);
					$(this).dialog("close");
				}
			} ]
		});
	}

	commCellGroupController.clearUserDialog = function() {
		$("#add-user-dialog #userName").val("");
		$("#add-user-dialog #email").val("");
		markField("#add-user-dialog #userName", "");
		markField("#add-user-dialog #email", "");
	}

	commCellGroupController.validateNewUser = function() {
		commCellGroupController.validateField($("#add-user-dialog #userName"));
		commCellGroupController.validateField($("#add-user-dialog #email"));
		if ($('#add-user-dialog .errorField').length > 0) {
			return false;
		}

		let email = $("#add-user-dialog #email").val();
		for (let e in commCellGroupController.selectedUsers) {
			if (e === email){
				cvUtil.errorToast(localMsg.userExist);
				return false;
			}
		}

		return true;
	}

	commCellGroupController.validateField = function(ele) {
		var re = new RegExp(ele.data("validation"));
		var val = ele.val();
		var txId = ele.attr("id");
		var fn = ele.data("validationfn");
		var msg = '';
		if (fn !== undefined) {
			eval(fn);
			return;
		}
		if (re.test(val) === false) {
			msg = ele.data("msg");
		}
		markField('#' + txId, msg);
	}

	commCellGroupController.addUser = function(userName, email) {
		let userTableId = USERS_TABLE + CVTable.table;
		let userTable = $('#' + userTableId).dataTable();
		let newUser = {};

		newUser[USER_NAME] = userName;
		newUser[USER_EMAIL] = email;
		newUser[DELETE_USER_ACTION] = 1;

		userTable.fnAddData( newUser );
		userTable.fnSettings()._iDisplayLength = userTable.fnGetData().length;
		userTable.fnDraw();

		let userInfo = {
				isCreator: 0,
				name: userName,
				email: email
		}

		commCellGroupController.selectedUsers[email] = userInfo;
	}

	function addUserRow(name, email, isEdit, isCreator) {
		if (!isNoEmptySlot()) {
			return;
		}

		var eleClass = "form-group userEntry newUser";
		if (isEdit) {
			eleClass = "form-group userEntry savedUser";
		}
		var group = $("<div></div>", {
			'class' : eleClass,
		}).appendTo("#userWrapper #userListDiv");

		var userDivEle = $("<div></div>", {
			'class' : 'rowItem userName'
		}).appendTo(group);

		var userEle = $("<input/>", {
			'class' : 'form-control',
			type : 'text',
			placeholder : 'User Name',
			disabled : isEdit ? true : false
		}).appendTo(userDivEle);

		var emailDivEle = $("<div></div>", {
			'class' : 'rowItem userEmail'
		}).appendTo(group);

		var emailEle = $("<input/>", {
			'class' : 'form-control',
			type : 'text',
			placeholder : 'Email Address',
			disabled : isEdit ? true : false
		}).appendTo(emailDivEle);

		var deleteBtnEle = $("<span></span>", {
			'class' : 'rowItem deleteUser sprite icon-trash',
			title : 'Delete User'
		}).appendTo(group);

		if (isCreator) {
			deleteBtnEle.addClass("disabled");
		}

		if (name) {
			userEle.val(name);
			userEle.attr("data-user-name", name);
		}
		if (email) {
			emailEle.val(email);
			emailEle.attr("data-user-email", email);
		}

		return group;
	}

	function setPrefixList(data) {
		let ul = $("#prefixList");
		ul.empty();

		$.each(data, function(i, c) {
			var pref = $("<li></li>", {
				text : c,
				class:"prefix-item"
			}).appendTo(ul);
		});
	}


	function isNoEmptySlot() {
		var noEmpty = true;
		$.each($(".newUser"), function(i, user) {
			var name = $(".userName input", user);
			var email = $(".userEmail input", user);

			if (name.val() === '' && email.val() === '') {
				noEmpty = false;
				return false;
			}
		});

		return noEmpty;
	}

	function deleteUserRow(ele) {
		if (ele.parent().hasClass("newUser")) {
			ele.parent().remove();
		} else {
			ele.parent().addClass("deleted");
		}
	}
	function createColumn(colName, rendererType, sqlType, visible, donotShow) {
		return {
			bFilterRendered : true,
			bUseRendered : false,
			bVisible : visible,
			doNotShow : donotShow,
			mDataProp : colName,
			rendererType : rendererType,
			sOriginalName : colName,
			sTitle : colName,
			dataType : sqlType
		}
	}

	function getColumnDefs() {
		let cols = [];

		cols.push(createColumn(ASSOCIATE_TO_COMMCELL_GROUP, "AssociateToCommCellGroup", "Integer", true, false));
		cols.push(createColumn(COMMCELL_ID, "", "String", true, false));
		cols.push(createColumn(COMMCELL_NAME, "", "String", true, false));
		cols.push(createColumn(DISPLAY_NAME, "", "String", false, true));
		cols.push(createColumn(COMMCELL_GUID, "", "String", false, true));
		cols.push(createColumn(COMMCELL_ID_INT, "", "String", false, true));

		return cols;
	}

	function getUserColumnDefs() {
		let cols = [];

		cols.push(createColumn(USER_NAME, "", "String", true, false));
		cols.push(createColumn(USER_EMAIL, "", "String", true, false));
		cols.push(createColumn(DELETE_USER_ACTION, "RemoveUserFromCommCellGroup", "Integer", true, false));

		return cols;
	}

	commCellGroupController.init();
	commCellGroupController.loadData();

};

$(document).ready(function() {
	CommCellGroupEditor();

});
