// ============================================================================
//  Copyright (c) CommVault Systems,
//  All Rights Reserved
// 
//  THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF CommVault Systems
//  The copyright notice above does not evidence any
//  actual or intended publication of such source code.                       
// 
//  File name: CommCellGrowthReportBuilder.java
//  Author: Marilou Geverola
// ============================================================================

function CommCellGrowthReport() {

	this.report = new Reports.initReport({
		type : 'commcellgrowth',
		generateButtonId : 'btngenerate',
		chartIds : [ 'configuration_Chart', 'data_Chart', 'activity_Chart', 'license_Chart' ]
	});
	this.requestString = report.requestString;

	report.initializeDefaults = function() {
		requestString["type"] = "CommCellGrowth";
		requestString["growthType"] = [ "configuration", "data", "activity", "license" ];
		requestString.frequency = "Daily";
		requestString.lastN = 7;
		requestString.predictNext = 4;
		requestString["refreshCache"] = true;
		var endTime = new Date();
		endTime = "" + endTime.getFullYear() + pZ((endTime.getMonth() + 1)) + pZ(endTime.getDate()) +
				pZ(endTime.getHours()) + pZ(endTime.getMinutes());
		requestString.endTime = endTime;
		deleteAllDrillKeys();
	};

	report.initComponents = function() {
		$('#lastN').spinner({
			min : 1,
			max : 12
		});
		$('#predictNext').spinner({
			min : 1,
			max : 4
		});
		var growthTypes = $(".growthType");

		growthTypes.each(function() {
			$(this).bind('checkedState', function() {
				checkCb($(this));
			});

			$(this).bind('unCheckedState', function() {
				var forceCheck = true;
				var box = $(this);
				growthTypes.each(function() {
					if ($(this).attr("data-state") == "checked" && $(this) != box) {
						forceCheck = false;
						return false;
					}
				});
				if (forceCheck)
					checkCb($(this));
				else
					unCheckCb($(this));
			});
		});

		$('#frequency').change(function() {
			var freq = $('#frequency').val();
			if (freq === 'Yearly') {
				getDateSpinner();
				$('#lastN').val(1);
				$('#predictNext').val(1);
			} else {
				getDatePicker();
				setMaxDate();
				$('#lastN').val(7);
				$('#predictNext').val(4);
			}
		});
	};

	function getDateSpinner() {
		var endTime = '';
		if ($('#endTime').hasClass('picker')) {
			endTime = $("#endTime").datetimepicker('getDate');
		}
		$('#endTime').datepicker("destroy");
		$('#endTime').removeClass('picker');
		$('#endTime').addClass('spinner');
		var currYr = new Date().getFullYear();
		$('#endTime').spinner({
			min : currYr - 10,
			max : currYr
		});
		if (endTime == undefined || endTime == '') {
			$('#endTime').val(currYr);
		} else {
			$('#endTime').val(endTime.getFullYear());
		}
	}
	function getDatePicker() {
		var endTime = '';
		var currDate = new Date();
		var currYr = currDate.getFullYear();
		if ($('#endTime').hasClass('spinner') || !($('#endTime').hasClass('picker'))) {
			endTime = $('#endTime').val();
			endTime = new Date(endTime, currDate.getMonth(), currDate.getDate());
			$('#endTime').spinner("destroy");
			$('#endTime').addClass('picker');
			$('#endTime').removeClass('spinner');

			$('#endTime').datetimepicker({
				showButtonPanel : true,
				showTime : false,
				showMinute : false,
				showHour : false,
				changeMonth : true,
				changeYear : true,
				dateFormat : 'm/d/yy',
				timeFormat : '',
				yearRange : currYr - 10 + ":" + currYr,
				beforeShow : function(input, inst) {
					var freq = $('#frequency').val();
					switch (freq) {
					case 'Daily':
					case 'Weekly':
					case 'Yearly':
						$(inst.dpDiv).removeClass('calendar-off');
						break;
					case 'Monthly':
						$(inst.dpDiv).addClass('calendar-off');
						break;
					}
				},
				beforeShowDay : function(date) {
					var freq = $('#frequency').val();
					switch (freq) {
					case 'Daily':
						return [ true, "" ];
					case 'Weekly':
						return [ date.getDay() == 1, "" ];
					case 'Monthly':
						return [ date.getDate() == LastDayOfMonth(date.getFullYear(), date.getMonth()), "" ];
					case 'Yearly':
						break;
					}
					return [ false, "" ];
				},

				onClose : function() {
					var freq = $('#frequency').val();
					if (freq === 'Monthly') {
						var iMonth = $("#ui-datepicker-div .ui-datepicker-month :selected").val();
						iMonth = parseInt(iMonth);
						var iYear = $("#ui-datepicker-div .ui-datepicker-year :selected").val();
						var date = new Date(iYear, iMonth + 1, 0);
						$(this).datetimepicker('setDate', date);
					}
				},
				onChangeMonthYear : function(year, month, inst) {
					var freq = $('#frequency').val();
					if (freq === 'Monthly') {
						$(this).datetimepicker('option', 'defaultDate', LastDayOfMonth(year, month));
					}
				}

			});
			if (endTime != '') {
				$('#endTime').datetimepicker('setDate', endTime);
			}
		} else if ($('#endTime').hasClass('picker')) {
			endTime = $("#endTime").datetimepicker('getDate');
			var f = $('#frequency').val();

			if (f == "Monthly") {
				endTime.setMonth(endTime.getMonth() + 1, 0);
				$('#endTime').datetimepicker('setDate', endTime);
			} else if (f == "Weekly") {

				endTime.setDate(endTime.getDate() - endTime.getDay() + 1);
				$('#endTime').datetimepicker('setDate', endTime);
			}
		}
	}

	function LastDayOfMonth(Year, Month) {
		return (new Date((new Date(Year, Month + 1, 1)) - 1)).getDate();
	}

	report.initComponentState = function() {
		$.each(report.requestString.growthType, function(index, value) {
			checkCb($('#' + value));
		});
		initTimeRange();
	};

	function initTimeRange() {
		$('#frequency').val(requestString.frequency);
		$('#lastN').val(requestString.lastN);
		$('#predictNext').val(requestString.predictNext);
		$('#endTime').val(requestString.endTime);
		var endTime = requestString.endTime;
		if (endTime == undefined) {
			endTime = new Date();
			endTime = "" + endTime.getFullYear() + pZ((endTime.getMonth() + 1)) + pZ(endTime.getDate()) +
					pZ(endTime.getHours()) + pZ(endTime.getMinutes());
		}
		if (requestString.frequency == 'Yearly') {
			getDateSpinner();
			$('#endTime').val(endTime.substr(0, 4));
		} else {
			getDatePicker();
			var dt = new Date(endTime.substr(0, 4), endTime.substr(4, 2) - 1, endTime.substr(6, 2), endTime
					.substr(8, 2), endTime.substr(10, 2));
			$('#endTime').datetimepicker("setDate", dt);
			setMaxDate();

		}
	}

	function setMaxDate() {
		var freq = $('#frequency').val();
		var today = new Date();

		var lastDay = new Date();
		lastDay.setMonth(lastDay.getMonth() + 1, 0);

		if (freq == 'Monthly') {
			if (today.getDate() == lastDay.getDate()) {
				$('#endTime').datetimepicker('option', 'maxDate', today);
			} else {
				var lastMonthLastDay = new Date();
				lastMonthLastDay.setDate(0);
				$('#endTime').datetimepicker('option', 'maxDate', lastMonthLastDay);
			}
		} else if (freq == 'Weekly') {
			$('#endTime').datetimepicker('option', 'maxDate', getMonday(today));
		} else {
			$('#endTime').datetimepicker('option', 'maxDate', today);
		}
	}

	function getMonday(day) {
		var lastDay = new Date();
		lastDay.setMonth(lastDay.getMonth() + 1, 0);

		var monday = new Date();

		if (day.getDate() == lastDay.getDate() && day.getDay() == 0) {
			monday.setDate(day.getDate() - 6);
		} else {
			monday.setDate(day.getDate() - day.getDay() + 1);
		}
		return monday;
	}

	function updateTimeRange() {
		requestString.frequency = $('#frequency').val();
		requestString.lastN = $('#lastN').val();
		requestString.predictNext = $('#predictNext').val();
		//requestString.untilDate = $('#endTime').val();
		var endTime = '';
		if (requestString.frequency == 'Yearly') {
			endTime = $("#endTime").val();
			requestString.endTime = "" + endTime + pZ(1) + pZ(1) + pZ(0) + pZ(0);
		} else {
			endTime = $("#endTime").datetimepicker('getDate');
			requestString.endTime = "" + endTime.getFullYear() + pZ((endTime.getMonth() + 1)) + pZ(endTime.getDate()) +
					pZ(endTime.getHours()) + pZ(endTime.getMinutes());
		}
	}

	function pZ(value) {
		if (value < 10)
			return "0" + value;
		else
			return "" + value;
	}

	report.beforeLoadReport = function() {
		$(".commcell_charts").mask("");
	};

	report.onLoadReport = function(data) {
		showHideGrowthSections();
		$.each($(".growthType"), function() {
			var cid = $(this).attr('id');
			var i = 0;
			if ($(this).hasClass('checked')) {
				var drillVal = getDrillValueForGrowthType(requestString, cid);
				var charts = data["data"];

				if (charts != undefined) {

					for (i = 0; i < charts.length; i++) {
						var chartData = charts[i];
						var gType = chartData.gType;
						$("#" + gType + "_BreadCrumbPanel").show();
						updateBreadCrumb(gType, chartData["path"]);
						if (chartData["chartData"] != null && chartData["chartData"]["chart"] != null &&
								chartData["chartData"]["chart"].length != 0) {
							if (gType == cid) {
								$("#" + gType + "_ErrorMsg").hide();
								$("#" + gType + "_Chart").show();
								if (drillVal != null) {
									if (gType == 'configuration' || gType == 'data') {
										if (chartData["chartData"]["yMax"] <= 0) {
											$("#" + gType + "_ErrorMsg").show();
											$("#" + gType + "_Chart").hide();
										} else {
											$("#" + gType + "_Chart").show();
											$("#" + gType + "_ErrorMsg").hide();
											createRaphaelBarchart(chartData["chartData"], gType + "_Chart", "");
										}
									} else {
										var totalY = 0;
										$.map(chartData["chartData"]["chart"].points, function(obj, index) {
											totalY += obj.y;
										});
										if (totalY <= 0) {
											$("#" + gType + "_ErrorMsg").show();
											$("#" + gType + "_Chart").hide();
										} else {
											$("#" + gType + "_Chart").show();
											$("#" + gType + "_ErrorMsg").hide();
											createRaphaelPieChart(chartData, gType);
										}
									}
								} else {
									createRaphaelchart(chartData["chartData"], gType + "_Chart", gType);
								}
							}
						} else {
							// Show no data message
							$("#" + gType + "_ErrorMsg").show();
							$("#" + gType + "_Chart").hide(); // ????
							/*
							 * if ( drillVal == null ) { $("#" + gType + "_BreadCrumbPanel").hide(); //??? }
							 */
						}
						if (chartData["tableData"] != null && chartData["tableData"]["aaData"] != null &&
								chartData["tableData"]["aaData"].length > 0) {
							if (gType == cid) {
								Reports.configureClientSideTable(gType + "_Summary", '', chartData["tableData"]);
								$("#" + gType + "_Table").show(); // ????
							}
						} else {
							$("#" + gType + "_Table").hide(); // ????
						}

					}

				}

			}

		});

	};

	function updateBreadCrumb(gType, path) {
		//clear the breadcrumb
		var panelId = "#" + gType + "_BreadCrumbPanel";
		$(panelId).empty();

		var counter = 0;
		$.each(path, function(index, value) {
			var id = gType + '_path' + counter;
			$(panelId).append('<li class="breadcrumb" id=' + id + '>' + value + '</li>');
			$("#" + id).click(function() {
				breadCrumbClick(gType, this.id.substring(this.id.length - 1));
			});
			counter++;
		});
		$(panelId + ' li').last().css('cursor', 'default');
		$(panelId + ' li').last().css('background-image', 'none');
		$(panelId).append('<li class="endBreadcrumb"></li>');
	}

	this.breadCrumbClick = function(gType, index) {
		if (index == 0 && getDrillValueForGrowthType(requestString, gType) != null) {
			deleteKeys(requestString, gType);
			report.processReport();
		}
	};

	report.updateRequestString = function() {
		updateReportOptions();
		updateTimeRange();
	};

	report.init();

	report.editTableCallBack = function(dataTable, value, settings) {
		var rowObject = settings["rowData"];
		var period = rowObject["period"];
		var numOfPrediction = rowObject["numOfPrediction"];
		var columnIndex = settings["colIndex"] + 1;
		if (isNaN(value))
			return;
		var numOfPeriods = rowObject["numOfPeriods"];
		var initialValue = rowObject["Growth_date_" + (numOfPeriods - 1)];
		for (var i = 0; i < numOfPrediction; i++) {
			initialValue = initialValue + value;
			dataTable.fnUpdate(initialValue, settings["rowIndex"], columnIndex++);
		}
	};

	report.customizeTable = function(dataTable) {
		var header = $("tr:first", dataTable);
		var lastN = requestString["lastN"];
		if (lastN != undefined) {
			lastN++;
			var predHeader = $("th:gt(" + lastN + ")", dataTable);
			$.each(predHeader, function(index, value) {
				value.style.fontWeight = "bold";
				value.style.color = "#66E066";
			});

			var rateHeader = $("th:eq(" + lastN + ")", dataTable);
			$.each(rateHeader, function(index, value) {
				value.style.fontWeight = "bold";
				value.style.color = "#6CB5FF";
			});
		}

	};

	function updateReportOptions() {
		requestString.growthType = new Array();

		$.each($(".growthType"), function() {
			var cid = $(this).attr('id');
			if ($(this).hasClass('checked')) {
				requestString.growthType.push(cid);
			} else {
				deleteKeys(requestString, cid);
			}

		});
	}

	function deleteAllDrillKeys() {
		$.each($(".growthType"), function() {
			var cid = $(this).attr('id');
			deleteKeys(requestString, cid);
		});
		deleteKeys(requestString, 'drill');
		deleteKeys(requestString, 'path');
	}

	function deleteKeys(obj, prop) {
		var re = new RegExp("^" + prop);
		var key = '';
		for (key in obj) {
			if (re.test(key)) {
				delete obj[key];
			}
		}
	}

	function getDrillValueForGrowthType(obj, prop) {
		var re = new RegExp("^" + prop);
		var key = '';
		for (key in obj) {
			if (re.test(key)) {
				return obj[key];
			}
		}
		return null;
	}

	function createRaphaelchart(data, frameId, gType) {
		var chartData = data["chart"];

		$("#" + frameId).empty();
		var r = Raphael(frameId);
		var txtattr = {
			fill : "#fff",
			stroke : '#333',
			'stroke-width' : 1,
			"font-weight" : 100,
			font : "12px  Fontin-Sans, sans-serif"
		};

		var temp = [];
		var temp2 = [];
		var popupLabels = [];
		var pointData = [];
		var seriesLabels = [];
		var yPos = 50;
		var thresholdValues = [];
		for (var i = 0; i < chartData.length; i++) {
			var s = chartData[i];
			//	    	r.text(500, yPos, s["name"]).attr(txtattr);
			temp[i] = s["xValues"];
			temp2[i] = s["yValues"];
			popupLabels[i] = s["popupLabels"];
			pointData[i] = s["pointData"];
			seriesLabels[i] = s["name"];
			if (s.maxEntityValue && s.maxEntityValue != "" && s.maxEntityValue > 0) {
				thresholdValues[i] = [];
				//s["maxEntityValue"]=48;
				thresholdValues[i][0] = [ 0.95 * s["maxEntityValue"] ];
				thresholdValues[i][1] = [ 0.8 * s["maxEntityValue"] ];
			}
			yPos += 20;
		}

		var xLabels = data["xAxisLabels"];
		var xStepInterval = data["xStepInterval"];

		var delta = 0;
		if (data["unitStr"])
			delta = data["unitStr"].length * 10;

		var isClickable = gType != "license";

		var lines = r.linechart(20 + delta, 10, 600, 300, temp, temp2, {
			thresholdValues : thresholdValues,
			nostroke : false,
			axis : "0 0 1 1",
			symbol : "circle",
			totalPoints : data["totalPoints"],
			axisystep : 5,
			miny : -1,
			maxy : data["yMax"],
			serieslabels : seriesLabels,
			labelxpos : 640 + delta,
			labelypos : 85,
			unitstr : data["unitStr"],
			minfactor : data["minFactor"],
			clickable : isClickable,
			dataLabelPos : 330
		}, popupLabels, pointData);

		for (var x = 0; x < lines.axis[0].text.items.length; x++)
			lines.axis[0].text.items[x].attr({
				'text' : xLabels[x * xStepInterval]
			});

		//	    lines.hover(function () {
		//    	this.tags = r.set();
		//    	
		//    				if (this.pointIndex ==0)
		//    					 this.tags.push(r.popup(this.x, this.y, this.popupLabel, "right", 10).attr(txtattr));
		//    				else
		//    					this.tags.push(r.popup(this.x, this.y, this.popupLabel, "up", 10).attr(txtattr));
		//    	    }, function () {
		//    	        this.tags && this.tags.remove();
		//      });

		if (gType != "license") {
			lines.click(function() {
				requestString[gType + "DrillBy"] = this.pointData.entityType;
				requestString[gType + "StartTime"] = this.pointData.startDate;
				requestString[gType + "EndTime"] = this.pointData.endDate;
				requestString["refreshCache"] = false;

				report.processReport();
			});
		}

		lines.symbols.attr({
			r : 4
		});
		r.raphael.g.txtattr = txtattr;

	}

	function createRaphaelBarchart(data, frameId, title) {

		$("#" + frameId).empty();
		var r = Raphael(frameId);
		var txtattr = {
			font : "12px 'Fontin Sans', Fontin-Sans, sans-serif"
		};

		var xLabels = data["xAxisLabels"];
		var xStepInterval = data["xStepInterval"];

		var len = data["totalPoints"] * 40; //70
		if (len < 200)
			len = 200;
		var chart = r.barchart(50, 10, len, 250, data["chart"], {
			nostroke : false,
			axis : "0 0 1 1",
			symbol : "circle",
			smooth : true,
			maxy : data["yMax"],
			type : "soft",
			axisxstep : 1,
			axisystep : 5,
			totalpoints : data["totalPoints"],
			serieslabels : data["seriesLabels"],
			unitstr : data["unitStr"],
			labelxpos : len + 100,
			labelypos : 45,
			prefix : "MB",
			minfactor : 1024
		}, data["popupLabels"]);

		var hovertxt = '';
		var g = '';
		chart.hover(function() {

			var hoverPosY = 280;// cy+r-55;
			var cx = 0; //cx-200;
			hovertxt = r.text(20, hoverPosY, this.bar.popupLabel).attr({
				fill : '#555',
				stroke : "none",
				"font-size" : 12,
				"text-anchor" : 'start',
				"font-weight" : 500,
				"font-family" : "Helevetica,Arial,Sans-serif"
			});
			var hoverPosX = cx - (hovertxt.getBBox().width / 2);
			hoverPosX = hoverPosX - cx + 100;
			hovertxt.transform('t' + hoverPosX + ',0');
			//g2 = sqre.glow({width:4});
			//e.stopPropagation();
			g = this.bar.glow({
				width : 4
			});

			//this.tooltips = r.set();

			//			if (this.pointIndex ==0)
			//this.tooltips.push(r.popup(this.bar.x, this.bar.y,this.bar.popupLabel, "right", 6).attr({ fill: "#fff", stroke: '#333', 'stroke-width': 1}));

			//			else
			//				this.tooltips.push(r.popup(this.bar.x, this.bar.y, this.bar.value || "0", "up", 6).attr({ fill: "#fff", stroke: '#333', 'stroke-width': 1}));
		}, function() {
			//this.tooltips && this.tooltips.remove();
			g.remove();
			if (hovertxt)
				hovertxt.attr("opacity", 0);
		});

		if (xLabels.length > 1) {
			chart.label(data["xAxisLabels"], true);
		} else {
			chart.axis[0].text.items[0].attr({
				'text' : xLabels[0]
			});
		}

	}

	function createRaphaelPieChart(data, frameId, title) {
		$("#" + frameId + "_Chart").empty();
		var pieData = data["chartData"]["chart"]["points"];
		var values = $.map(pieData, function(node) {
			return node["y"];
		});

		pieData.values = values;

		var labels = $.map(pieData, function(node) {
			return node["name"] + " - " + node["y"];
		});

		var events = {};
		events.clickFtn = [];

		Raphael(frameId + "_Chart", 700, 400).pieChart(250, 180, 150, pieData, labels, 500, 40, "#fff", events);
	}

	function showHideGrowthSections() {
		var growthTypes = $(".growthType");
		growthTypes.each(function() {
			var cId = $(this).attr('id');
			if ($(this).attr("data-state") == "checked") {
				$('#' + cId + '_ContentWrapper').show();
			} else {
				$('#' + cId + '_ContentWrapper').hide();
			}
		});
	}
}

$(document).ready(function() {
	CommCellGrowthReport();
});
