var repTitle = 'Backup Server Search';

function MSServerStatusReport() {
	var type = cvUtil.getParameter('type', 'backup');
	var entityType = cvUtil.getParameter('entityType', 1);
	var startDate = cvUtil.getParameter('startDate');
	var endDate = cvUtil.getParameter('endDate');
	var servers = cvUtil.getParameter('servers');

	this.report = new Reports.initReport({
		type : 'backup',
		reportTitle : repTitle
	});

	this.requestString = report.requestString;
	report.initializeDefaults = function() {
		report.requestString.type = 'backup';
		if (entityType) {
			report.requestString["entityType"] = entityType;
		}
		if (servers) {
			report.requestString["servers"] = servers;
		}
	};

	report.initComponents = function() {
		$("#from").datepicker({
			onClose : function(selectedDate) {
				$("#to").datepicker("option", "minDate", selectedDate);
			}
		});
		$("#to").datepicker({
			onClose : function(selectedDate) {
				$("#from").datepicker("option", "maxDate", selectedDate);
			}
		});
		$('#settingsLink').hide();
		setDefaultDates();

	};

	report.initComponentState = function() {

		if (entityType) {
			$('input:radio[name=entityType][value=' + entityType + ']').prop('checked', true);
		}
		if (servers) {
			$("#serversList").val(servers);
			updateInputsDisplay(servers);
		}
		if (startDate && endDate) {
			$("#from").val(startDate);
			$("#to").val(endDate);
		}

		$(".msGenerate").click(function(el) {
			updateTableData();
		});

		updateFilters();
	};

	report.getExportFilePrefix = function() {
		var filePre = repTitle;
		return filePre;
	};

	report.beforeLoadReport = function() {

	};
	report.getRequestUrl = function() {
		return cvUtil.getContextPath() + '/server/msbackup.do?type=backup&action=msserverstatus&json=' +
				window.encodeURIComponent(JSON.stringify(this.requestString));
	};

	report.onLoadReport = function(data) {
		var servers = $('#serversList').val().trim();
		if (servers == '' || servers.length <= 0) {
			return;
		}

		processResponse(data, true);
	};

	report.init();

	//------------------------------------------functions----------------------------------------------------------

	function processResponse(data, initialLoad) {

		if (data == undefined || data == null) {
			$("#ContentWrapper").hide();
			$("#noDataNotificationPanel").html(cvUtil.cvLocalize('Something_went_wrong_please_try_again'));
			$("#noDataNotificationPanel").show();
		} else if (data && data.error) {
			$("#ContentWrapper").hide();
			$("#noDataNotificationPanel").html(data.error);
			$("#noDataNotificationPanel").show();
		} else {
			var resCnt = 0;
			if (data.statusTable) {
				var view = {};
				view.members = data.statusTable;
				view.members.iDisplayLength = 100;
				view.collapsible = false;
				view.filters = true;
				view.exportExcel = true;
				//view.alarm=true;
				//view.title = getSelectedCriteriaText()+": "+data.statusTable.iTotalRecords;
				resCnt = data.statusTable.iTotalRecords;
				var tableDiv = 'statusTable';
				if (resCnt > 0) {
					if ($("#" + tableDiv).length == 0) {
						$('#ContentWrapper').append(SurveyTemplates.makeReportTable(tableDiv, view));
					}
					var configTable = true;
					if ((!data.statusTable.aoColumns || data.statusTable.aoColumns.length == 0) &&
							(!data.statusTable.aoColumnDefs || data.statusTable.aoColumnDefs.length == 0)) {
						configTable = false;
					}
					if (configTable) {
						setFilterRendered(view.members.aoColumns);
						Reports.configureClientSideTableWithPaging(tableDiv, view.title, view.members);
					}
					$('#statusTable_title').text(view.title);
					//$("#"+tableDiv+" .rptTblBtnEncDiv").append($("#"+tableDiv+"_table_length"));
					$('.dataTables_filter').hide();
					$("#" + tableDiv + " .rptTblBtnEncDiv").find('.dataTables_length').remove();
					var pageLengthDiv = $("#" + tableDiv + "_table_wrapper .dataTables_length").detach();
					pageLengthDiv.appendTo($("#" + tableDiv + " .rptTblBtnEncDiv"));
					$("#" + tableDiv).show();
				} else {
					if ($("#" + tableDiv).length == 1) {
						$("#" + tableDiv).hide();
					}
				}
				report.updateHash(true);
			}
			showResultText(resCnt);
			addSummarySection('summaryTable');

			report.afterLoadReport();
		}
	}//end of processResponse

	function showResultText(cnt) {
		$('#resultDiv').show();
		$('#resultText').text(getSelectedCriteriaText());
		$('#resultCount').text(cnt);

	}

	function isValidData() {
		var servers = $('#serversList').val().trim();
		var msg = '';
		if (servers == '' || servers.length <= 0) {
			msg = 'At least one server name is required.';
		}
		markField('#serversList', msg);

		msg = '';
		var entityType = $('input:radio[name=entityType]:checked').val();
		if (entityType == 4) {
			var startDate = $('#from').val();
			var endDate = $('#to').val();
			if (startDate == '' || !moment(startDate, 'MM/DD/YYYY').isValid()) {
				msg = 'Enter valid start date.';
			}
			markField('#from', msg);
			msg = '';
			if (endDate == '' || !moment(endDate, 'MM/DD/YYYY').isValid()) {
				msg = 'Enter valid end date.';
			}
			markField('#to', msg);
		} else {
			markField('#from', msg);
			markField('#to', msg);
		}

		if ($('.errorField').length > 0) {
			return false;
		}

		return true;
	}
	function updateTableData() {
		if (!isValidData()) {
			return;
		}

		delete requestString["startDate"];
		delete requestString["endDate"];

		Reports.clearTableParams();
		var entityType = $('input:radio[name=entityType]:checked').val();
		requestString["entityType"] = entityType;
		if (entityType == 4) {
			var startDate = $.datepicker.formatDate('mm/dd/yy', $('#from').datepicker('getDate'));
			requestString["startDate"] = startDate;

			var endDate = $.datepicker.formatDate('mm/dd/yy', $('#to').datepicker('getDate'));
			requestString["endDate"] = endDate;
		}
		var servers = $('#serversList').val().trim();
		var serversList = servers.replace(/[\s,;\n]+/g, ';');//replace all whitespaces, comma and semicolons with a semicolon
		if (serversList.slice(-1) !== ';') {
			serversList += ';';
		}
		updateInputsDisplay(serversList);
		requestString["servers"] = serversList;
		var configURL = cvUtil.getContextPath() + '/server/msbackup.do?type=backup&action=msserverstatus&json=' +
				window.encodeURIComponent(JSON.stringify(report.requestString));
		report.mask("");
		cvUtil.loadPageWithPOST(configURL, "", function(data) {
			report.updateHash(false);
			processResponse(data, false);
			report.afterLoadReport();
			var filterOn = $("#detailsTable_FilterButton").hasClass("filterOn");
			if (filterOn) {
				$("#detailsTable_FilterButton").toggleClass("filterOn");
			}
			updateFilters();
			report.unmask();
		});
	}//end of updateTableData

	function updateInputsDisplay(serversList) {
		$('#inputsDiv').show();
		$('#inputsCount').text(serversList.split(';').length - 1);
	}

	function getSelectedCriteriaText() {
		var type = $('input:radio[name=entityType]:checked').val();
		if (type == 1) {
			return cvUtil.cvLocalize('ms.search.servers_24_hours');
		} else if (type == 2) {
			return cvUtil.cvLocalize('ms.search.servers_not_24_hours');
		} else if (type == 3) {
			return cvUtil.cvLocalize('ms.search.servers_never');
		} else if (type == 4) {
			var startDate = $.datepicker.formatDate('mm/dd/yy', $('#from').datepicker('getDate'));
			var endDate = $.datepicker.formatDate('mm/dd/yy', $('#to').datepicker('getDate'));
			return cvUtil.cvLocalize('ms.search.servers_protected', startDate, endDate);
		}
	}
	function updateFilters() {
		$(document).ajaxStart(function() {
			//report.mask("");
		}).ajaxStop(function() {
			// If the ajax completes but tables are not rendered attacheTableActions may fail.
			// This is not observed but just be sure, waiting for 100ms.
			setTimeout(function() {
				report.attachTableActions();
				if (!cvUtil.getParameter('exportType') && !$("body").hasClass("exportfriendly")) {
					$('.tableFilter').each(function(idx, elem) {
						if (!$(this).hasClass("filterOn") &&
								$(this).closest(".reportTable").find("table").hasClass("initialized")) {
							$(elem).click();
						}
					});
				}
				report.unmask();
			}, 100);
			report.unmask();
		});
	}//end of updateFilters
	function addSummarySection(tableDiv) {
		var showSummary = false;
		var summary = "<table id=\"reportSummary\">";
		summary += "<tr><td><u>Summary</u></td><td></td></tr>";
		if (exportRequest) {
			showSummary = true;
			summary += "<tr><td>Servers: </td><td> " + servers + "</td></tr>";
			summary += "<tr><td>Number of servers input: </td><td> " + (servers.split(';').length - 1) + "</td></tr>";
			summary += "<tr><td>" + $('#resultDiv').text().trim() + "</td></tr>";
		}
		summary += "</table>";
		if (showSummary) {
			$('#resultDiv').hide();
			$("#msSummaryDiv").append(summary);
			$("#msSummaryDiv").show();
		} else {
			$("#msSummaryDiv").hide();
		}
	}
}

function convertMapToList(map) {
	var list = [];
	if (map) {
		$.each(map, function(key, val) {
			var temp = {};
			temp.id = key;
			temp.name = val;
			list.push(temp);
		});
		sortList(list);
	}
	return list;
}

function sortList(list) {
	if (list) {
		list.sort(function(a, b) {
			var nameA = a.name.toLowerCase();
			var nameB = b.name.toLowerCase();
			return (nameA < nameB) ? -1 : (nameA > nameB) ? 1 : 0;
		});
	}
}

function setFilterRendered(aoColumns) {
	var renderersResultingURLs = [ "MSSuccessJobs", "MSFailedJobs", "MSJobsRate", "MSSearchServerJobs" ];
	for (var i = 0; i < aoColumns.length; i++) {
		var renderer = aoColumns[i].rendererType;
		if (typeof renderer != 'undefined') {
			if ($.inArray(renderer, renderersResultingURLs) > -1) {
				aoColumns[i].bFilterRendered = false;
				aoColumns[i].bUseRendered = false;
			}
		}
	}
}

function setDefaultDates() {
	var today = new Date();
	var fromday = new Date();
	fromday.setFullYear(today.getFullYear() - 1)
	$("#from").datepicker('setDate', fromday);
	$("#to").datepicker('setDate', today);
}

$(document)
		.ready(function() {
			MSServerStatusReport();

			var r = "<span id='rep_link' class='breadcrumbMain'><a class='withSeparatorMain' href='index.jsp#0'>Reports</a></span>";
			$(r).prependTo($('#headerLinks'));

			$('#actionsButton').hide();
			$('#buildYourOwnReport').hide();
			$('#sectionTitle').html(repTitle);
			$(document).attr("title", repTitle);
		});
