// ============================================================================
//  Copyright (c) CommVault Systems,
//  All Rights Reserved
//
//  THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF CommVault Systems
//  The copyright notice above does not evidence any
//  actual or intended publication of such source code.
//
//  Author: Hari Ravula
// ============================================================================

/*
 * This should be the home of all report tables' renderers
 * This should have format and unformat methods of a renderer.
 */
if(typeof document != "undefined"){
	$(document).ready(function() {
		exportRequest = cvUtil.getParameter('exportType', "");
	});
}

Reports.Renderers = {
	"capacity" : function(o, val, id, opts) {
		var format = function(o, val, id) {
			return autoConvert(o.aData[o.oSettings.aoColumns[o.iDataColumn].mDataProp]);
		};
		var unformat = function(val) {
			if (val == '0 bytes' || val == "0 b") {
				return 0;
			}
			var r = /^[0-9\.]+\s*[KMGTPE]?B(ytes)*$/i; //Capacity - e.g. 2MB
			if (val == "n/a") {
				return -1;
			}
			if (r.test(val)) {
				$.each(capacity.units, function(key, unit) {
					if (endsWith(val, key.toLowerCase())) {
						if (!isNaN(val.substr(0, val.length - key.length))) {
							val = parseFloat(val) * unit;
						}

						return false;
					}
				});
				return val;
			}
			// should not reach this..
			return -100;
		};
		var validateVals = function() {
			return Reports.Renderers["capacityMB"](null, null, null, {
				validateVals : true
			})();
		};

		if (!!opts) {
			if (!!opts.unformat) {
				return unformat;
			} else if (!!opts.format) {
				return format;
			} else if (!!opts.validateVals) {
				return validateVals;
			} else {
				return null;
			}
		}
		// call format by default
		return format(o, val, id);
	},
	"licenseCapacityUsage" : function(o, val, id, opts) {
		var format = function(o, val, id) {
			var licType = o.aData.licenseType;
			if (licType === 'vmsockets' || licType === 'protectedvms' || licType === 'users' ||
					licType === 'archived_mailboxes' || licType === 'ci_objects') {
				if (!isNaN(o.aData[o.oSettings.aoColumns[o.iDataColumn].mDataProp])) {
					return autoConvertNumber(parseInt(o.aData[o.oSettings.aoColumns[o.iDataColumn].mDataProp]));
				}
			}
			return autoConvert(o.aData[o.oSettings.aoColumns[o.iDataColumn].mDataProp]);
		};
		var unformat = function(val) {
			if (val == '0 bytes' || val == "0 b") {
				return 0;
			}
			var r = /^[0-9\.]+\s*[KMGTPE]?B(ytes)*$/i; //Capacity - e.g. 2MB
			if (val == "n/a") {
				return -1;
			}
			if (r.test(val)) {
				$.each(capacity.units, function(key, unit) {
					if (endsWith(val, key.toLowerCase())) {
						if (!isNaN(val.substr(0, val.length - key.length))) {
							val = parseFloat(val) * unit;
						}

						return false;
					}
				});
				return val;
			}
			// should not reach this..
			return -100;
		};
		var validateVals = function() {
			return Reports.Renderers["capacityMB"](null, null, null, {
				validateVals : true
			})();
		};

		if (!!opts) {
			if (!!opts.unformat) {
				return unformat;
			} else if (!!opts.format) {
				return format;
			} else if (!!opts.validateVals) {
				return validateVals;
			} else {
				return null;
			}
		}
		// call format by default
		return format(o, val, id);
	},
	"capacityMB" : function(o, val, id, opts) {
		var format = function(o, val, id) {
			return autoConvertFromMB(parseFloat(o.aData[o.oSettings.aoColumns[o.iDataColumn].mDataProp]));
		};
		var unformat = function(val, negative) {
			if (val == '0 bytes' || val == "0 b") {
				return 0;
			}
			var r = /^[0-9\.]+\s*[KMGTPE]?B(ytes)*$/i; //Capacity - e.g. 2MB
			if (negative) {
				r = /^-?[0-9\.]+\s*[KMGTPE]?B(ytes)*$/i;
			} else if (val == "n/a") {
				return -1;
			}
			val = $.trim(val);
			if (r.test(val)) {
				if (endsWith(val, "kb")) {
					val = parseFloat(val) / 1024;
				} else if (endsWith(val, "bytes")) {
					val = parseFloat(val) / 1024 / 1024;
				} else {
					$.each(capacity.unitsMB, function(key, unit) {
						if (endsWith(val, key.toLowerCase())) {
							if (!isNaN(val.substr(0, val.length - key.length))) {
								val = parseFloat(val) * unit;
							}

							return false;
						}
					});
				}
				return val;
			}
			// should not reach this..
			return -100;
		};
		var validateVals = function() {
			return {
				pattern : /^[0-9\.]+\s*[KMGTPE]?B(ytes)*$/i,
				allowedValues : [ "n/a" ],
				desc : "bytes, KB, MB, GB, TB, PB or EB",
				hint : "Capacity (in bytes, KB, MB, GB, TB, PB or EB)"
			};
		};

		if (!!opts) {
			if (!!opts.unformat) {
				return unformat;
			} else if (!!opts.format) {
				return format;
			} else if (!!opts.validateVals) {
				return validateVals;
			} else {
				return null;
			}
		}
		// call format by default
		return format(o, val, id);
	},

	"LicenseSizeMB" : function(o, val, id, opts) {
		var format = function(o, val, id) {
			return convertLicenseCapacity(parseFloat(o.aData[o.oSettings.aoColumns[o.iDataColumn].mDataProp]));
		};
		var unformat = function(val, negative) {
			if (val == '0 bytes' || val == "0 b") {
				return 0;
			}
			var r = /^[0-9\.]+\s*[KMGTPE]?B(ytes)*$/i; //Capacity - e.g. 2MB
			if (negative) {
				r = /^[\s\-]*[0-9\.]*\s*[KMGTPE]?B(ytes)$/i;
			} else if (val == "n/a") {
				return -1;
			}
			if (r.test(val)) {
				if (endsWith(val, "kb")) {
					val = parseFloat(val) / 1024;
				} else if (endsWith(val, "bytes")) {
					val = parseFloat(val) / 1024 / 1024;
				} else {
					$.each(capacity.unitsMB, function(key, unit) {
						if (endsWith(val, key.toLowerCase())) {
							if (!isNaN(val.substr(0, val.length - key.length))) {
								val = parseFloat(val) * unit;
							}

							return false;
						}
					});
				}
				return val;
			}
			// should not reach this..
			return -100;
		};
		var validateVals = function() {
			return {
				pattern : /^[0-9\.]+\s*[KMGTPE]?B(ytes)*$/i,
				allowedValues : [ "n/a" ],
				desc : "bytes, KB, MB, GB, TB, PB or EB",
				hint : "Capacity (in bytes, KB, MB, GB, TB, PB or EB)"
			};
		};

		if (!!opts) {
			if (!!opts.unformat) {
				return unformat;
			} else if (!!opts.format) {
				return format;
			} else if (!!opts.validateVals) {
				return validateVals;
			} else {
				return null;
			}
		}
		// call format by default
		return format(o, val, id);
	},
	"capacityMB_Neg" : function(o, val, id, opts) {
		var format = function(o, val, id) {
			return autoConvertFromMB(o.aData[o.oSettings.aoColumns[o.iDataColumn].mDataProp], true);
		};
		var unformat = function(val, negative) {
			return Reports.Renderers["capacityMB"](null, null, null, {
				unformat : true
			})(val, true);
		};
		var validateVals = function() {
			return {
				pattern : /^[\s\-]*[0-9\.]*\s*[KMGTPE]?B(ytes)$/i,
				desc : "bytes, KB, MB, GB, TB, PB or EB",
				hint : "Capacity (in bytes, KB, MB, GB, TB, PB or EB)"
			};
		};

		if (!!opts) {
			if (!!opts.unformat) {
				return unformat;
			} else if (!!opts.format) {
				return format;
			} else if (!!opts.validateVals) {
				return validateVals;
			} else {
				return null;
			}
		}
		// call format by default
		return format(o, val, id);
	},

	"capacityMBPercent" : function(o, val, id, opts) {
		return Reports.Renderers["capacityMB"](o, val, id, opts);
	},

	"strikeCount" : function(o, val, id, opts) {
		var format = function(o, val, id) {
			val = o.aData[o.oSettings.aoColumns[o.iDataColumn].mDataProp];
			var exportRequest = cvUtil.getParameter('exportType', "");
			if (exportRequest == 'csv') {
				return val;
			}
			if (val == 3) {
				return '<div style="padding:0; margin:0;text-align:left">'
						+ '<svg width="29" height="9" xmlns="http://www.w3.org/2000/svg" xmlns:svg="http://www.w3.org/2000/svg"><g><ellipse stroke="#000000" fill="#E44C4C" stroke-width="3" cx="4.47954" cy="4.50856" rx="4.2" ry="4.2" stroke-opacity="0" id="svg_2"/><ellipse stroke="#000000" fill="#E44C4C" stroke-width="3" cx="14.47954" cy="4.50856" rx="4.2" ry="4.2" stroke-opacity="0" id="svg_3"/><ellipse stroke="#000000" fill="#E44C4C" stroke-width="3" cx="24.47954" cy="4.50856" rx="4.2" ry="4.2" stroke-opacity="0" id="svg_4"/></g></svg>';
						+ '</div>';
			} else if (val == 2) {
				return '<div style="padding:0; margin:0;text-align:left">'
						+ '<svg width="19" height="9" xmlns="http://www.w3.org/2000/svg" xmlns:svg="http://www.w3.org/2000/svg"><g><ellipse id="svg_2" stroke-opacity="0" ry="4.2" rx="4.2" cy="4.50856" cx="4.47954" stroke-width="3" fill="#F69C09" stroke="#000000"/><ellipse id="svg_3" stroke-opacity="0" ry="4.2" rx="4.2" cy="4.50856" cx="14.47954" stroke-width="3" fill="#F69C09" stroke="#000000"/></g></svg>';
						+ '</div>';
			} else if (val == 1) {
				return '<div style="padding:0; margin:0;text-align:left">'
						+ '<svg width="9" height="9" xmlns="http://www.w3.org/2000/svg" xmlns:svg="http://www.w3.org/2000/svg"><g><ellipse stroke="#000000" fill="#FFF700" stroke-width="3" cx="4.54015" cy="4.49605" rx="4.2" ry="4.2" stroke-opacity="0" id="svg_2"/></g></svg>';
						+ '</div>';
			} else {
				return val;
			}
		};
		var unformat = function(val) {
			// nothing to unformat
			return val;
		};
		var validateVals = function() {
			return {
				allowedValues : [ 1, 2, 3 ],
				desc : "Strike Count (1, 2 or 3)",
				hint : "Strike Count (1, 2 or 3)"
			};
		};

		if (!!opts) {
			if (!!opts.unformat) {
				return unformat;
			} else if (!!opts.format) {
				return format;
			} else if (!!opts.validateVals) {
				return validateVals;
			} else {
				return null;
			}
		}
		// call format by default
		return format(o, val, id);
	},

	"NumberNA" : function(o, val, id, opts) {
		var format = function(o, val, id) {
			val = parseFloat(val);
			if (val == -2) {
				return "Never Protected";
			} else if (val < 0) {
				return "N/A";
			}
			return Reports.Renderers.FORMATNUMBERNA(o, val, id, opts);
		};
		var unformat = function(val) {
			if (val == "n/a") {
				return -1;
			} else if (val == "never protected") {
				return -2;
			}
			return Reports.Renderers.FORMATNUMBERNA(o, val, id, opts)(val);
		};
		var validateVals = function() {
			return {
				allowedValues : [ "n/a", "never protected", function(val) {
					return $.isNumeric(val) && val >= 0;
				} ],
				desc : "non-negative numeric",
				hint : "Non-negative numeric, Never Protected or N/A"
			};
		};

		if (!!opts) {
			if (!!opts.unformat) {
				return unformat;
			} else if (!!opts.format) {
				return format;
			} else if (!!opts.validateVals) {
				return validateVals;
			} else {
				return null;
			}
		}
		// call format by default
		return format(o, val, id);
	},

	"NumberUnlimitedNA" : function(o, val, id, opts) {
		var format = function(o, val, id) {
			val = parseInt(val, 10);
			if (val == -2) {
				return "N/A";
			} else if (val == -1) {
				return "Unlimited";
			}
			return Reports.Renderers.FORMATNUMBERNA(o, val, id, opts);
		};
		var unformat = function(val) {
			if (val == "unlimited") {
				return -1;
			} else if (val == "n/a") {
				return -2;
			}
			return Reports.Renderers.FORMATNUMBERNA(o, val, id, opts)(val);
		};
		var validateVals = function() {
			return {
				allowedValues : [ "unlimited", "n/a", function(val) {
					return $.isNumeric(val) && val >= 0;
				} ],
				desc : "non-negative numeric",
				hint : "Non-negative numeric value, Unlimited or N/A"
			};
		};

		if (!!opts) {
			if (!!opts.unformat) {
				return unformat;
			} else if (!!opts.format) {
				return format;
			} else if (!!opts.validateVals) {
				return validateVals;
			} else {
				return null;
			}
		}
		// call format by default
		return format(o, val, id);
	},

	"percentNA" : function(o, val, id, opts) {
		var format = function(o, val, id) {
			return getPercentValue(o.aData[o.oSettings.aoColumns[o.iDataColumn].mDataProp]);
		};
		var unformat = function(val) {
			if (val == "n/a") {
				return -1;
			}
			return val;
		};
		var validateVals = function() {
			return {
				allowedValues : [ "n/a", function(val) {
					return $.isNumeric(val) && (val >= 0 && val <= 100);
				} ],
				desc : "percent(0 - 100)",
				hint : "Percent value (0 to 100)"
			};
		};

		if (!!opts) {
			if (!!opts.unformat) {
				return unformat;
			} else if (!!opts.format) {
				return format;
			} else if (!!opts.validateVals) {
				return validateVals;
			} else {
				return null;
			}
		}
		// call format by default
		return format(o, val, id);
	},

	"formattedInteger" : function(o, val, id, opts) {
		var format = function(o, val, id) {
			var x1 = "" + o.aData[o.oSettings.aoColumns[o.iDataColumn].mDataProp];
			var rgx = /(\d+)(\d{3})/;
			while (rgx.test(x1)) {
				x1 = x1.replace(rgx, '$1' + ',' + '$2');
			}
			return x1;
		};
		var unformat = function(val) {
			return val.replace(/[,]/g, "");
		};
		var validateVals = function() {
			return {
				allowedValues : [ function(val) {
					return $.isNumeric(val.replace(/[,]/g, ""));
				} ],
				desc : "numeric",
				hint : "numeric value"
			};
		};

		if (!!opts) {
			if (!!opts.unformat) {
				return unformat;
			} else if (!!opts.format) {
				return format;
			} else if (!!opts.validateVals) {
				return validateVals;
			} else {
				return null;
			}
		}
		// call format by default
		return format(o, val, id);
	},

	"booleanRenderer" : function(o, val, id, opts) {
		var format = function(o, val, id) {
			var html = "";
			var str = val.toLowerCase();
			if (exportRequest == 'csv') {
				return val;
			}
			if (str == 'yes') {
				html = "<img src='" + cvUtil.getContextPath() +
						"/survey/images/cb_selected.png' alt='Yes' style='vertical-align: middle' title='Yes'>";
			} else if (str == 'no') {
				html = "";
			}
			return html;
		};
		var unformat = function(val) {
			if (val == 'yes' || val == 'true' || val == '1') {
				return 'Yes';
			} else if (val == 'no' || val == 'false' || val == '0') {
				return "No";
			}
		};
		var validateVals = function() {
			return {
				allowedValues : [ "yes", "no", "true", "false", "1", "0" ],
				desc : "Yes/No",
				hint : "Yes or No"
			};
		};

		if (!!opts) {
			if (!!opts.unformat) {
				return unformat;
			} else if (!!opts.format) {
				return format;
			} else if (!!opts.validateVals) {
				return validateVals;
			} else {
				return null;
			}
		}
		// call format by default
		return format(o, val, id);
	},

	"HEALTH_STATUS" : function(o, val, id, opts) {
		var format = function(o, val, id) {
			var images = [ 'question', 'check-circle', 'info', 'warning', 'critical', 'check-circle-bulb' ];
			var healthStatusDesc = encodeURIComponent(o.aData["HealthStatusDescription"]);
			var tokens = val.split('_');

			if (exportRequest == "csv") {
				return tokens[1];
			}

			if (tokens[0] == '1a') {
				tokens[0] = 5;
			}

			var html = "<span class='healthStatus' style='text-align:left'><span class='sprite icon-" +
					images[tokens[0]] + "' alt='" + tokens[1] + "' style='vertical-align: middle' data-healthdesc='" +
					healthStatusDesc + "'></span>";

			var cellVal = tokens[1];

			var urlval = "healthparams.jsp#";
			var paramId = o.aData["ParamId"];
			if (paramId != undefined && paramId != "") {
				var hsh = "param" + paramId;
				urlval = urlval + hsh;
			}

			if (!exportRequest) {
				cellVal = "<a class='disableOnEmbedded' href='" + urlval + "'>" + cellVal + "</a>";
			}
			return html + "<span style='vertical-align: bottom;padding-left:3px;'>" + cellVal + "</span><span>";
		};
		var unformat = function(val) {
			if (val == "unknown") {
				return "0_Unknown";
			} else if (val == "good") {
				return "1_Good";
			} else if (val == "info") {
				return "2_Info";
			} else if (val == "warning") {
				return "3_Warning";
			} else if (val == "critical") {
				return "4_Critical";
			}
			return val;
		};
		var validateVals = function() {
			return {
				allowedValues : [ "unknown", "good", "info", "warning", "critical" ],
				desc : "Status",
				hint : "Status (Unknown, Good, Info, Warning or Critical)"
			};
		};
		var customValueFn = function(val) {
			var tokens = val.split('_');
			if (tokens.length > 1) {
				return tokens[1];
			}
			return "";
		};

		if (!!opts) {
			if (!!opts.unformat) {
				return unformat;
			} else if (!!opts.format) {
				return format;
			} else if (!!opts.validateVals) {
				return validateVals;
			} else if (!!opts.customValueFn) {
				return customValueFn;
			} else {
				return null;
			}
		}
		// call format by default
		return format(o, val, id);
	},

	/// Renderer for elements requiring dot icons to indicate status
	"DOT_STATUS" : function(o, val, id, opts) {
		var format = function(o, val, id) {
			/// Dot colors corresponding to status values
			var images = [ "white", "green", "none", "orange", "red" ];
			var svg = '<svg width="20" height="20" xmlns="http://www.w3.org/2000/svg" xmlns:svg="http://www.w3.org/2000/svg"><g><ellipse id="svg_2" stroke-opacity="0" ry="9.7" rx="9.7" cy="10" cx="10" stroke-width="3" fill-opacity="{{OPACITY}}" fill="{{COLOR}}" stroke="#000000"/></g></svg>';
			var colors = ['#FFFFFF', '#75C350', '#000000', '#F69C09', '#E44C4C'];
			var opacity = 1;

			/// Statuses come in the form <1-4>_<StatusString>; anything else
			/// is unknown
			var tokens = val.split('_');
			if (tokens[0] > 4) {
				tokens[0] = 0;
			}

			/// Do not render icons in export
			if (exportRequest == "csv") {
				return tokens[1];
			}

			if (tokens[1] === "Info") {
				tokens[1] = "";
			}

			if (tokens[0] == 2) {
				opacity = 0;
			}

			svg = svg.replace("{{COLOR}}", colors[tokens[0]]).replace("{{OPACITY}}", opacity);

			/// Define rendered cell contents
			var html = "<div style='text-align: center' title='"+ tokens[1] +"'>" + svg + "<span style='display:none'>" + tokens[1] + "</span></div>";

			return html;
		};

		/// Below this point taken directly from HEALTH_STATUS

		var unformat = function(val) {
			val = val.toLowerCase();

			if (val == "unknown") {
				return "0_Unknown";
			} else if (val == "good") {
				return "1_Good";
			} else if (val == "") {
				return "2_Info";
			} else if (val == "warning") {
				return "3_Warning";
			} else if (val == "critical") {
				return "4_Critical";
			}

			return val;
		};
		var validateVals = function() {
			return {
				allowedValues : [ "unknown", "good", "", "warning", "critical" ],
				desc : "Status",
				hint : "Status (Unknown, Good, Warning or Critical)"
			};
		};
		var customValueFn = function(val) {
			var tokens = val.split('_');

			if (tokens.length > 1) {
				return tokens[1];
			}

			return "";
		};

		if (!!opts) {
			if (!!opts.unformat) {
				return unformat;
			} else if (!!opts.format) {
				return format;
			} else if (!!opts.validateVals) {
				return validateVals;
			} else if (!!opts.customValueFn) {
				return customValueFn;
			} else {
				return null;
			}
		}
		// call format by default
		return format(o, val, id);
	},

	"statusRenderer" : function(o, val, id, opts) {
		return Reports.Renderers["HEALTH_STATUS"](o, val, id, opts);
	},

	"remarkRenderer" : function(o, val, id) {
		var surveyId = parseInt(o.aData["SurveyId"], 10);
		var currentMonitor = o.aData["Parameter"];
		var paramId = o.aData["ParamId"];
		var ccuid = o.aData["CommUniId"];
		var ccGroupId = cvUtil.getParameter('ccGroupId', -1);
		var companyId = cvUtil.getParameter("companyId", 0);
		var valueAssessment = [ 97, 98, 99, 100, 101, 102, 111, 112, 113, 114, 171 ];
		var followForExport = 'followForExport';
		if (val.indexOf("<VDL>") != -1) {
			val = val.replace("<VDL>", "<DELIM>");
			var groupParam = "";
			if (ccGroupId > 0) {
				groupParam = "&ccGroupId=" + ccGroupId;
			}
			if (companyId > 0) {
				groupParam = "&companyId=" + companyId;
			}
			if (currentMonitor == "Concurrent LAN Backup Settings" || surveyId == 32) {
				val = val +
						"\t" +
						"<a class='hideInCsv hc-cell-a' href='http://documentation.commvault.com/commvault/release_9_0_0/books_online_1/english_us/prod_info/features.htm?var1=http://documentation.commvault.com/commvault/release_9_0_0/books_online_1/english_us/features/media_agent/media_agent.htm' target='_blank'>View Documentation  </a>" +
						" , ";
			} else if (surveyId == 25 || surveyId == 23 || surveyId == 30) {
				val = val + "<a class='hideInCsv hc-cell-a " + followForExport + "' href='" + cvUtil.getContextPath() +
						"/survey/reports/operations.jsp?surveyId=" + surveyId + "&commUniId=" + ccuid + groupParam +
						"&dd=false" + "'>View Details</a>";

			} else if (surveyId == 27) {//SLA: VDL and then doc link
				val = "<a class='hideInCsv hc-cell-a " + followForExport + "' href='" + cvUtil.getContextPath() +
						"/survey/reports/indcommcellsurvey.jsp?commUniId=" + ccuid + "&surveyId=" + surveyId +
						groupParam + "&dd=false" + "'>View Details</a>" + val;
			} else {
				val = val + "<a class='hideInCsv hc-cell-a " + followForExport + "' href='" + cvUtil.getContextPath() +
						"/survey/reports/indcommcellsurvey.jsp?commUniId=" + ccuid + "&surveyId=" + surveyId +
						groupParam + "&dd=false" //+ currentMonitor
						+ "'>View Details</a>";
			}

		}

		if (val.indexOf("<DELIM>") != -1) {
			var tokens = val.split("<DELIM>");
			val = "<ul class='result-list'>";
			for (var i = 0; i < tokens.length; i++) {
				var token = tokens[i];
				if (token.length != 0) {
					val = val + "<li class='result-parts'>" + token + "</li>";
				}
			}
			val = val + "</ul>";
		}

		// for actions like configuring alarms etc..
		if ($.inArray(surveyId, valueAssessment) == -1 && o.aData["Status"].indexOf("Info") == -1) {
			var tmplt = "<span id='cAlarm' class='action-separator' data-alarmtype='0' data-paramname='" +
					currentMonitor + "' data-paramid='" + paramId + "'>Add Alert</span>";
			//+ "<span id='vAlarms' data-paramname='" + currentMonitor + "'>View Alarms</span>";
			val = val + "<div class='hideInCsv health-actions'>" + tmplt + "</div>";
		}
		return val;
	},

	"outcomeRenderer" : function(o, val, id) {
		var currentMonitor = o.aData["Parameter"];
		var surveyId = o.aData["SurveyId"];
		var ccuid = o.aData["CommUniId"];

		if (val.indexOf("<DELIM>") != -1) {
			var tokens = val.split("<DELIM>");
			val = "<ul class='result-list multi-line-Column'>";
			for (var i = 0; i < tokens.length; i++) {
				var token = tokens[i];
				if (token.length != 0) {
					val = val + "<li class='result-parts'>" + token + "</li>";
				}
			}
			val = val + "</ul>";
		} else {
			val = "<span class = 'multi-line-Column'>" + val + "</span>";
		}

		if (currentMonitor == "Client Strike Count" || surveyId == 25) {
			if (val.indexOf("Strike 3 :") != -1) {
				val = val
						.replace("Strike 3 :",
								"<span style='padding:0; margin:0'><span class='sprite icon-dot-red' alt='strike'></span>"
										+ "<span style='padding:0; margin:0'><span class='sprite icon-dot-red' alt='strike'></span>"
										+ "<span style='padding:0; margin:0'><span class='sprite icon-dot-red' alt='strike'></span>"
										+ "</span>");
			}
			if (val.indexOf("Strike 2 :") != -1) {
				val = val
						.replace("Strike 2 :",
								"<span style='padding:0; margin:0'><span class='sprite icon-dot-orange' alt='strike'></span>"
										+ "<span style='padding:0; margin:0'><span class='sprite icon-dot-orange' alt='strike'></span>"
										+ "</span>");
			}
			if (val.indexOf("Strike 1 :") != -1) {
				val = val.replace("Strike 1 :",
						"<span style='padding:0; margin:0'><span class='sprite icon-dot-yellow' alt='strike'></span>"
								+ "</span>");
			}
		}

		return val;
	},

	"Quarter" : function(o, val, id, opts) {
		var format = function(o, val, id) {
			return "Q" + val;
		};
		var unformat = function(val) {
			return val.replace("q", '');
		};
		var validateVals = function() {
			return {
				pattern : /^q[1-9]\d*$/i,
				hint : "Quarter value (as in Q1, Q2, Q3, Q4 etc)",
				desc : "Quarter (as in Q1, Q2, Q3, Q4 etc)"
			};
		};

		if (!!opts) {
			if (!!opts.unformat) {
				return unformat;
			} else if (!!opts.format) {
				return format;
			} else if (!!opts.validateVals) {
				return validateVals;
			} else {
				return null;
			}
		}
		// call format by default
		return format(o, val, id);
	},

	"dateTimeRenderer" : function(o, val, id, opts) {
		return getFormattedDateTimeFromUTC(o, val, id, opts, "MMM D, YYYY hh:mm A");
	},

	"MMMDDYYYYHrMinSecAM" : function(o, val, id, opts) {
		return getFormattedDateTimeFromUTC(o, val, id, opts, "MMM D, YYYY hh:mm:ss A");
	},

	"MMDDYYYYHrMinSecAM" : function(o, val, id, opts) {
		return getFormattedDateTimeFromUTC(o, val, id, opts, "MM/DD/YYYY hh:mm:ss A");
	},

	"DDMMYYYYHrMinSecAM" : function(o, val, id, opts) {
		return getFormattedDateTimeFromUTC(o, val, id, opts, "DD/MM/YYYY hh:mm:ss A");
	},

	"MMMYYYY" : function(o, val, id, opts) {
		var format = function(o, val, id) {
			val = parseInt(val, 10);
			if (val == -50) {
				return "N/A";
			} else if (val == -100) {
				return "";
			} else if (val == -3) {
				return "Never";
			}

			//return moment.unix(val  + (new Date().getTimezoneOffset()*60) ).format("MMM YYYY");
			return getDate(val, "MMM YYYY");
		};
		var unformat = function(val) {
			if (val == "n/a") {
				return -50;
			} else if (val == "never") {
				return -3;
			} else if (val == "") {
				return -100;
			} else if (val == "#EMPTY") {
				// token for empty values.. need a way to implement it. alarms xml may not accept empty values.
			}

			var seconds = Reports.unformatRelativeDate(val);
			if (seconds != null) {
				return seconds;
			}
			return moment(val, "MMM YYYY").unix() - getTimezoneOffset();
		};
		var validateVals = function() {
			return {
				// allowedValues: [ function(val) { return moment(val, "MMM YYYY").isValid() || Reports.unformatRelativeDate(val) != null; }],
				pattern : /^([0-9\.]+)\s*(year|month|week|day|hour|minute|second+)(s)?$/i,
				desc : "Relative Time (e.g. 5 Hours or 5 Days)",
				hint : "Relative Time (e.g. 5 Hours or 5 Days)"
			};
		};

		if (!!opts) {
			if (!!opts.unformat) {
				return unformat;
			} else if (!!opts.format) {
				return format;
			} else if (!!opts.validateVals) {
				return validateVals;
			} else {
				return null;
			}
		}
		// call format by default
		return format(o, val, id);
	},

	"MMMDDYYYY" : function(o, val, id, opts) {
		var format = function(o, val, id) {
			if (val == null || val == undefined || val === 'null') {
				return "";
			}
			val = parseInt(val, 10);
			if (val == -50) {
				return "N/A";
			} else if (val == -100) {
				return "";
			} else if (val == -3) {
				return "Never";
			} else if (val == -2) {
				return "Nothing to Copy";
			} else if (val == -4) {
				return "History unavailable to estimate time";
			} else if (val == -5) {
				return "Jan 01, 2038"; // Why 2038?
			} else if (val == -6) {
				return "Not Protected"; // Why 2038?
			} else if (val == -7) {
				return "No Jobs";
			}
			//return moment.unix(val  + (new Date().getTimezoneOffset()*60) ).format("MMM DD, YYYY");
			return getDate(val, "MMM DD, YYYY");
		};
		var unformat = function(val) {
			if (val == "n/a") {
				return -50;
			} else if (val == "never") {
				return -3;
			} else if (val == "") {
				return -100;
			} else if (val == "#EMPTY") {
				// token for empty values.. need a way to implement it. alarms xml may not accept empty values.
			} else if (val == "nothing to copy") {
				return -2;
			} else if (val == "history unavailable to estimate time") {
				return -4;
			} else if (val == "jan 01, 2038" || val == "jan 1, 2038") {
				return -5;
			}
			var seconds = Reports.unformatRelativeDate(val);
			if (seconds != null) {
				return seconds;
			}

			//			return moment(val, "MMM DD, YYYY").unix() - new Date().getTimezoneOffset() * 60;
			return (Date.parse(val) / 1000) - getTimezoneOffset();
		};
		var validateVals = function() {
			return {
				//allowedValues : [ "n/a", "never", "nothing to copy", "history unavailable to estimate time", function(val) {
				//					return moment(val, "MMM DD, YYYY").isValid() || Reports.unformatRelativeDate(val) != null;
				//return moment(val, "MMM D, YYYY hh:mm A").isValid() || Reports.unformatRelativeDate(val) != null;
				//}],
				pattern : /^([0-9\.]+)\s*(year|month|week|day|hour|minute|second+)(s)?$/i,
				desc : "Relative Time (e.g. 5 Hours or 5 Days)",
				hint : "Relative Time (e.g. 5 Hours or 5 Days)"
			};
		};

		if (!!opts) {
			if (!!opts.unformat) {
				return unformat;
			} else if (!!opts.format) {
				return format;
			} else if (!!opts.validateVals) {
				return validateVals;
			} else {
				return null;
			}
		}
		// call format by default
		return format(o, val, id);
	},

	"MMMDDYYYYNEVER" : function(o, val, id, opts) {
		var format = function(o, val, id) {
			if (val == null || val == undefined || val === 'null') {
				return "";
			}
			val = parseInt(val, 10);
			console.log(val);
			if (val == 549755813887) {
				return "N/A";
			} else if (val == -100) {
				return "";
			} else if (val == 549755813886) {
				return "Never";
			} else if (val == -2) {
				return "Nothing to Copy";
			} else if (val == -4) {
				return "History unavailable to estimate time";
			} else if (val == -5) {
				return "Jan 01, 2038"; // Why 2038?
			} else if (val == -6) {
				return "Not Protected"; // Why 2038?
			} else if (val == -7) {
				return "No Jobs";
			}
			//return moment.unix(val  + (new Date().getTimezoneOffset()*60) ).format("MMM DD, YYYY");
			return getDate(val, "MMM DD, YYYY");
		};
		var unformat = function(val) {
			if (val == "n/a") {
				return 549755813887;
			} else if (val == "never") {
				return 549755813886;
			} else if (val == "") {
				return -100;
			} else if (val == "#EMPTY") {
				// token for empty values.. need a way to implement it. alarms xml may not accept empty values.
			} else if (val == "nothing to copy") {
				return -2;
			} else if (val == "history unavailable to estimate time") {
				return -4;
			} else if (val == "jan 01, 2038" || val == "jan 1, 2038") {
				return -5;
			}
			var seconds = Reports.unformatRelativeDate(val);
			if (seconds != null) {
				return seconds;
			}

			//			return moment(val, "MMM DD, YYYY").unix() - new Date().getTimezoneOffset() * 60;
			return (Date.parse(val) / 1000) - getTimezoneOffset();
		};
		var validateVals = function() {
			return {
				//allowedValues : [ "n/a", "never", "nothing to copy", "history unavailable to estimate time", function(val) {
				//					return moment(val, "MMM DD, YYYY").isValid() || Reports.unformatRelativeDate(val) != null;
				//return moment(val, "MMM D, YYYY hh:mm A").isValid() || Reports.unformatRelativeDate(val) != null;
				//}],
				pattern : /^([0-9\.]+)\s*(year|month|week|day|hour|minute|second+)(s)?$/i,
				desc : "Relative Time (e.g. 5 Hours or 5 Days)",
				hint : "Relative Time (e.g. 5 Hours or 5 Days)"
			};
		};

		if (!!opts) {
			if (!!opts.unformat) {
				return unformat;
			} else if (!!opts.format) {
				return format;
			} else if (!!opts.validateVals) {
				return validateVals;
			} else {
				return null;
			}
		}
		// call format by default
		return format(o, val, id);
	},

	"DWDDMMM" : function(o, val, id, opts) {
		var format = function(o, val, id) {
			if (val == null || val == undefined || val === 'null') {
				return "";
			}
			val = parseInt(val, 10);
			if (val == -50) {
				return "N/A";
			} else if (val == -100) {
				return "";
			} else if (val == -3) {
				return "Never";
			}
			//return moment.unix(val  + (new Date().getTimezoneOffset()*60) ).format("dddd (MMM D)");
			return getDate(val, "dddd (MMM D)");
		};
		var unformat = function(val) {
			if (val == "n/a") {
				return -50;
			} else if (val == "never") {
				return -3;
			} else if (val == "") {
				return -100;
			} else if (val == "#EMPTY") {
				// token for empty values.. need a way to implement it. alarms xml may not accept empty values.
			}
			var seconds = Reports.unformatRelativeDate(val);
			if (seconds != null) {
				return seconds;
			}
			return (Date.parse(val) / 1000) - getTimezoneOffset();
			//			return moment(val, "dddd (D MMM)").unix() - new Date().getTimezoneOffset() * 60;
		};
		var validateVals = function() {
			return {
				//allowedValues : [ "n/a", "never", function(val) {
				//return moment(val, "MMM D, YYYY hh:mm A").isValid() || Reports.unformatRelativeDate(val) != null;

				//}],
				pattern : /^([0-9\.]+)\s*(year|month|week|day|hour|minute|second+)(s)?$/i,
				desc : "Relative Time (e.g. 5 Hours or 5 Days)",
				hint : "Relative Time (e.g. 5 Hours or 5 Days)"
			};
		};

		if (!!opts) {
			if (!!opts.unformat) {
				return unformat;
			} else if (!!opts.format) {
				return format;
			} else if (!!opts.validateVals) {
				return validateVals;
			} else {
				return null;
			}
		}
		// call format by default
		return format(o, val, id);
	},

	"MMMDDYYYYHrMinAM" : function(o, val, id, opts) {
		var format = function(o, val, id) {
			if (val == null || val == undefined || val === 'null') {
				return "";
			}
			val = parseInt(val, 10);
			if (val == -50) {
				return "N/A";
			} else if (val == -100) {
				return "";
			} else if (val == -3) {
				return "Never";
			}

			//return moment.unix(val  + (new Date().getTimezoneOffset()*60) ).format("MMM D, YYYY hh:mm A");
			return getDate(val, "MMM D, YYYY hh:mm A");
		};
		var unformat = function(val) {
			if (val == "n/a") {
				return -50;
			} else if (val == "never") {
				return -3;
			} else if (val == "") {
				return -100;
			} else if (val == "#EMPTY") {
				// token for empty values.. need a way to implement it. alarms xml may not accept empty values.
			}
			var seconds = Reports.unformatRelativeDate(val);
			if (seconds != null) {
				return seconds;
			}
			return moment(val, "MMM D, YYYY hh:mm A").unix() - getTimezoneOffset();
		};
		var validateVals = function() {
			return {
				/*
				 * allowedValues : [ "n/a", "never", function(val) { return moment(val, "MMM D, YYYY hh:mm
				 * A").isValid() || Reports.unformatRelativeDate(val) != null; }],
				 */
				pattern : /^([0-9\.]+)\s*(year|month|week|day|hour|minute|second+)(s)?$/i,
				desc : "Relative Time (e.g. 5 Hours or 5 Days)",
				hint : "Relative Time (e.g. 5 Hours or 5 Days)"
			};
		};

		if (!!opts) {
			if (!!opts.unformat) {
				return unformat;
			} else if (!!opts.format) {
				return format;
			} else if (!!opts.validateVals) {
				return validateVals;
			} else if (!!opts.desc) {
				return desc;
			} else {
				return null;
			}
		}
		// call format by default
		return format(o, val, id);
	},

	"HHMMSS" : function(o, val, id, opts) {
		var format = function(o, val, id) {
			if (val == null || val == undefined || val === 'null') {
				return "";
			}
			val = parseInt(val, 10);
			if (val == -50) {
				return "N/A";
			} else if (val == -100) {
				return "";
			} else if (val == -3) {
				return "Never";
			}
			//return moment.unix(val  + (new Date().getTimezoneOffset()*60) ).format("HH:mm:ss");
			return getDate(val, "HH:mm:ss");
		};
		var unformat = function(val) {
			if (val == "n/a") {
				return -50;
			} else if (val == "never") {
				return -3;
			} else if (val == "") {
				return -100;
			} else if (val == "#EMPTY") {
				// token for empty values.. need a way to implement it. alarms xml may not accept empty values.
			}
			var seconds = Reports.unformatRelativeDate(val);
			if (seconds != null) {
				return seconds;
			}
			return moment(val, "HH:mm:ss").unix() - getTimezoneOffset();
		};
		var validateVals = function() {
			return {
				/*
				 * allowedValues : [ "n/a", "never", function(val) { return moment(val, "HH:mm:ss").isValid() ||
				 * Reports.unformatRelativeDate(val) != null; }],
				 */
				pattern : /^([0-9\.]+)\s*(year|month|week|day|hour|minute|second+)(s)?$/i,
				desc : "Relative Time (e.g. 5 Hours or 5 Days)",
				hint : "Relative Time (e.g. 5 Hours or 5 Days)"
			};
		};

		if (!!opts) {
			if (!!opts.unformat) {
				return unformat;
			} else if (!!opts.format) {
				return format;
			} else if (!!opts.validateVals) {
				return validateVals;
			} else if (!!opts.desc) {
				return desc;
			} else {
				return null;
			}
		}
		// call format by default
		return format(o, val, id);
	},

	"editable" : function(o, val, id) {
		return '<input style="width: 100px; height: 14px; " autocomplete="off" value="' + val + '">';
	},

	"editableCapacity" : function(o, val, id) {
		val = autoConvert(o.aData[o.oSettings.aoColumns[o.iDataColumn].mDataProp]);
		return '<input style="width: 100px; height: 14px; " autocomplete="off" value="' + val + '">';
	},

	"commcellName" : function(o, val, id) {
		var urlval = "";
		var retval = '';
		var uniqId = o.aData[o.oSettings.aoColumns[0].mDataProp];
		if (uniqId == 0) {
			//Registered CommCell but data not uploaded.. no hyper link for this case.
			var enableMetricsDocUrl = 'http://documentation.commvault.com/commvault/v11/article?p=features/reports/metrics/cloud_reporting_server.htm';
			var docTooltip = cvUtil.cvLocalize('commcell.not_uploaded.msg.public');
			if (!SurveyGlobals.isPublicMetrics) {
				enableMetricsDocUrl = 'http://documentation.commvault.com/commvault/v11/article?p=features/reports/metrics/setup.htm';
				docTooltip = cvUtil.cvLocalize('commcell.not_uploaded.msg.private');
			}
			return "<span class='disabledIcon' title='Disabled'></span><span class='statusEnable cc-name-content' title='" +
					docTooltip + "'><a href='" + enableMetricsDocUrl + "' target='_blank'>" + val + "</a></span>";
		}
		var fromDashboardSLA = cvUtil.getParameter('fromDashboardSLA', 0);
		var createCCGroup = cvUtil.getParameter('createGroup', 'false');
		var surveyId = cvUtil.getParameter('surveyId', '0');
		var ccGroupId = cvUtil.getParameter('ccGroupId', '-1');
		var companyId = cvUtil.getParameter('companyId', 0);
		var tenant = encodeURIComponent(cvUtil.getParameter('tenant',""));
		if(window.location.pathname.indexOf("CommCellSurvey.jsp")>-1 && surveyId !='0' && surveyId !='137'){//Job Details(137) doesn't have indcommcellsurvey view
			urlval = cvUtil.getContextPath() + "/survey/reports/indcommcellsurvey.jsp?surveyId="+surveyId+"&commUniId="+uniqId;
			if( tenant.length>0 ){
				urlval += "&tenant=" + tenant;
			}
		}else{
			urlval = cvUtil.getContextPath() + "/survey/reports/dashboard.jsp?commUniId=" + uniqId;
		}
		if (ccGroupId > 0 && createCCGroup === 'false') {
			urlval += "&ccGroupId=" + ccGroupId;
		}
		if (companyId > 0) {
			if(CvUserDefaultDashboardGroupId <= 0 || (companyId != CvUserDefaultDashboardGroupId)){
				urlval = urlval + "&companyId=" + companyId;
			}
		}
		if (exportRequest) {
			return val;
		} else {
			var addDeleteButton = false;
			if (o.aData["Status"] != undefined && fromDashboardSLA == 0 && surveyId <= 0) {
				if (o.aData["Status"].toLowerCase() == "enabled") {
					retval = "";
				} else {
					retval = "<span class='disabledIcon' title='Disabled'></span>";
				}

				retval = retval + "<a href='" + urlval + "' class='statusEnable cc-name-content'>" + val + "</a>";
				addDeleteButton = true;

			} else {
				retval = "<a href='" + urlval + "'>" + val + "</a>";
			}
			if (!!SurveyGlobals && SurveyGlobals.isAdminUser) {
				retval = retval + '<input class="new-cc-name" data-uniqid="' + uniqId + '" data-name="' + val +
						'" value="' + val + '">';
				retval = retval + '<span class="sprite icon-edit edit-ccname" data-uniqid="' + uniqId +
						'" data-name="' + val + '" title="' + cvUtil.cvLocalize('Edit_CommCellName') + '"></span>';
				retval = retval + '<span class="sprite icon-save save-cc-name" title="' +
						cvUtil.cvLocalize('Save_CommCellName') + '"></span>';
				retval = retval + '<span class="sprite icon-delete cancel-cc-name" title="' +
						cvUtil.cvLocalize('Cancel') + '"></span>';
			}

			if (addDeleteButton) {
				retval = retval + "<span class='deleteIcon' data-uniqId='" + uniqId + "' data-cname='" + val +
						"' title='Delete'></span>";
				;
			}
			return retval;
		}
	},

	"commceServ" : function(o, val, id) {
		var namevalue = new Array();
		var csvalue = val.split('CommServid=');
		for (var i = 0; i < csvalue.length; i++) {
			if (csvalue[i] != "") {
				var name = csvalue[i].split('name=');
				for (var j = 0; j < name.length; j++) {
					if (name[j] != "") {
						namevalue.push(name[j]);
					}
				}
			}
		}

		var urlval = window.location.search;
		var returnvalue;

		//var uniqId = o.aData[o.oSettings.aoColumns[0].mDataProp];
		for (var k = 0; k < namevalue.length; k++) {
			if (urlval.toString().indexOf("?") == -1) {
				urlval = window.location.href + "?surveyId=25&commUniId=" + namevalue[k];
			} else {
				urlval = window.location.href + "&commUniId=" + namevalue[k];
			}
			if (k == 0) {
				if (exportRequest) {
					returnvalue = namevalue[k + 1];
				} else {
					returnvalue = "<a href='" + urlval + "'>" + namevalue[k + 1] + "</a>";
				}

			} else {
				if (exportRequest) {
					returnvalue = returnvalue + ' , ' + namevalue[k + 1];
				} else {
					returnvalue = returnvalue + ' , ' + "<a href='" + urlval + "'>" + namevalue[k + 1] + "</a>";
				}
			}
			k++;
		}
		return returnvalue;
	},

	"throughput" : function(o, val, id) {
		val = o.aData[o.oSettings.aoColumns[o.iDataColumn].mDataProp];
		if (val == " 0 - 10") {
			val = '<span  class="throughput">' + val + '</span>';
			return val;
		} else if (val == " 10 - 20") {
			val = '<span  class="throughput">' + val + '</span>';
			return val;
		} else if (val == '>= 20' || val === '20 and Above') {
			val = '<span class="throughput">' + val + '</span>';
			return val;
		}
		return val;
	},

	"dashboard" : function(o, val, id) {
		var urlval = "";
		var uniqId = o.aData[o.oSettings.aoColumns[0].mDataProp];
		urlval = cvUtil.getContextPath() + "/survey/reports/dashboard.jsp?commUniId=" + uniqId;
		if (cvUtil.getParameter('ccGroupId', -1) > 0) {
			urlval = urlval + "&ccGroupId=" + cvUtil.getParameter('ccGroupId');
		}
		if (cvUtil.getParameter('companyId', 0) > 0) {
			urlval = urlval + "&companyId=" + cvUtil.getParameter('companyId');
		}
		if (exportRequest) {
			return urlval;
		} else {
			return "<a class='dashboardLink' target='_blank' href='" + urlval + "'>dashboard</a>";
		}
	},
	"COMPANYNAME" : function(o, val, id) {
		var urlval = "";
		var companyId = o.aData[o.oSettings.aoColumns[0].mDataProp];
		urlval = cvUtil.getContextPath() + "/survey/reports/dashboard.jsp?companyId=" + companyId;
		if (cvUtil.getParameter('ccGroupId', -1) > 0) {
			urlval = urlval + "&ccGroupId=" + cvUtil.getParameter('ccGroupId');
		}
		if (exportRequest) {
			return val;
		} else {
			return "<a class='dashboardLink' href='" + urlval + "'>"+val+"</a>";
		}
	},

	"commCellId" : function(o, val, id) {
		return decimalToHex(val, 2).toUpperCase();
	},

	"CustomerName" : function(o, val, id) {
		var urlval = window.location.search;
		var uniqId = o.aData[o.oSettings.aoColumns[0].mDataProp];
		urlval = "commcellmonitoring.jsp?commUniId=" + uniqId;
		if (cvUtil.getParameter('ccGroupId', -1) > 0) {
			urlval = urlval + "&ccGroupId=" + cvUtil.getParameter('ccGroupId');
		}
		if (cvUtil.getParameter('companyId', 0) > 0) {
			urlval = urlval + "&companyId=" + cvUtil.getParameter('companyId');
		}

		if (exportRequest) {
			return val;
		} else {
			return "<a href='" + urlval + "'>" + val + "</a>";
		}
	},

	"CommCellName" : function(o, val, id) {
		var urlval = window.location.search;
		var uniqId = o.aData[o.oSettings.aoColumns[0].mDataProp];
		urlval = "dashboard.jsp?commUniId=" + uniqId;
		if (cvUtil.getParameter('ccGroupId', -1) > 0) {
			urlval = urlval + "&ccGroupId=" + cvUtil.getParameter('ccGroupId');
		}
		if (cvUtil.getParameter('companyId', 0) > 0) {
			urlval = urlval + "&companyId=" + cvUtil.getParameter('companyId');
		}

		if (exportRequest) {
			return val;
		} else {
			return "<a href='" + urlval + "'>" + val + "</a>";
		}
	},

	"failureReason" : function(o, val, id) {
		return val.replace(/\\n\\n/gi, "\n\n");
	},

	"spNameLinkRenderer" : function(o, val, id) {
		if (val) {
			var urlval = window.location.href;
			urlval = urlval + "&spName=" + val;
			if (exportRequest) {
				return val;
			} else {
				return "<a href='" + urlval + "'>" + val + "</a>";
			}
		}
		return val;
	},

	"CATEGORY_LINK" : function(o, val, id) {
		if (val) {
			var urlval = window.location.href;
			var category = o.aData[o.oSettings.aoColumns[0].mDataProp];
			urlval = urlval + "&entity=" + category;
			if (exportRequest) {
				return val;
			} else {
				return "<a href='" + urlval + "'>" + val + "</a>";
			}
		}
		return val;
	},

	"DAYS" : function(o, val, id) {
		if (val < 0) {
			return "Never";
		} else if (val == 0) {
			return "0 days";
		}
		return (function(total_days) {
			var date_current = new Date();
			var utime_target = date_current.getTime() + total_days * 86400 * 1000;
			var date_target = new Date(utime_target);

			var diff_year = parseInt(date_target.getUTCFullYear() - date_current.getUTCFullYear());
			var diff_month = parseInt(date_target.getUTCMonth() - date_current.getUTCMonth());
			var diff_day = parseInt(date_target.getUTCDate() - date_current.getUTCDate());

			var days_in_month = [
					31,
					parseInt((date_target.getUTCFullYear() % 4 ? 29 : 28), 10),
					31,
					30,
					31,
					30,
					31,
					31,
					30,
					31,
					30,
					31 ];
			var date_string = "";
			while (true) {
				date_string = "";
				//adding year
				if (diff_year > 0) {
					date_string += diff_year + (diff_year > 1 ? " years" : " year");
				}

				if (diff_month < 0) {
					diff_year -= 1;
					diff_month += 12;
					continue;
				}
				var seperate = "";
				if (date_string.length) {
					seperate = " ";
				}
				if (diff_month > 0) {
					date_string += seperate + diff_month + (diff_month > 1 ? " months" : " month");
				}

				if (diff_day < 0) {
					diff_month -= 1;
					diff_day += days_in_month[((11 + date_target.getUTCMonth()) % 12)];
					continue;
				}

				if (date_string.length) {
					seperate = " and ";
				} else {
					seperate = "";
				}
				if (diff_day > 0) {
					date_string += seperate + diff_day + (diff_day > 1 ? " days" : " day");
				}
				break;
			}
			return date_string;
		})(val);
	},

	"SCALE_STATUS" : function(o, val, id, opts) {
		return Reports.Renderers["HEALTH_STATUS"](o, val, id, opts);
	},

	"VA_REMARKS_LINK" : function(o, val, id) {
		var result = val;
		url = window.location.href, surveyId = o.aData[o.oSettings.aoColumns[1].mDataProp], pattern = /surveyId=\d+/;

		if (val.indexOf("<VDL>") != -1) {
			result = result.replace("<VDL>", "");
			url = url.replace().replace(/surveyId=\d+/, "surveyId=" + surveyId);
			var cssClass = " class='followForExport' ";
			result += "<br><a" + cssClass + "href='" + url + "'>View Details</a>";
		}

		result = "<span class = 'multi-line-Column'>" + result + "</span>";

		return result;
	},

	"SCALE_REMARKS_LINK" : function(o, val, id) {
		if (val.indexOf("<VDL>") != -1) {
			val = val.replace("<VDL>", "");
			var urlval = window.location.href;
			var param = o.aData[o.oSettings.aoColumns[1].mDataProp];
			var paramId = o.aData[o.oSettings.aoColumns[5].mDataProp];
			urlval = urlval + "&param=" + param + "&paramId=" + paramId;
			var cssClass = " class='followForExport' ";
			val = val + "<a" + cssClass + "href='" + urlval + "'>View Details</a>";
		}
		if (val.indexOf("<DELIM>") !== -1) {
			var tokens = val.split("<DELIM>");
			val = "<ul class='result-list'>";
			for (var i = 0; i < tokens.length; i++) {
				var token = tokens[i];
				if (token.length !== 0) {
					val = val + "<li class='result-parts'>" + token + "</li>";
				}
			}
			val = val + "</ul>";
		}
		if(val.indexOf("<RECSLInt>") !== -1 && val.indexOf("</RECSLInt>") !== -1 ){
			var str=val;
			var startTag="<RECSLInt>";
		    var endTag="</RECSLInt>";
		    var start1 = str.indexOf(startTag);
		    var start2 = str.indexOf(endTag);
		    var linkInside = str.substring( start1 + startTag.length , start2);
		    var surveyId = parseInt(cvUtil.getParameter('surveyId'));
		    linkInside = linkInside.replace(/ /g, '_')+'_'+surveyId;
		    val = val.substring(0,start1-1)+"<br/><a href='#"+linkInside+"' class='moveToInternalLink'>View Details</a> below"+val.substring(start2 + endTag.length ,val.length);
		}
		return val;
	},

	"JobsAffected" : function(o, val, id) {

		var errorCode = o.aData[o.oSettings.aoColumns[0].mDataProp];
		errorCode = $("<div/>").html(errorCode).text();
		var jobs = o.aData[o.oSettings.aoColumns[1].mDataProp];
		var ccID = cvUtil.getParameter('commUniId', 0);
		var params="";
		if (ccID > 0) {
			params = 'commUniId=' + ccID;
		}
		if (cvUtil.getParameter('ccGroupId', -1) > 0) {
			params += "&ccGroupId=" + cvUtil.getParameter('ccGroupId');
		}
		if (cvUtil.getParameter('companyId', 0) > 0) {
			params += "&companyId=" + cvUtil.getParameter('companyId');
		}
		var urlval = 'operations.jsp?';
		if(params.length > 0){
			urlval += params + '&surveyId=103&filteredTable-0=details103&details103-filterClause=Error%20Code-LIKE-' + encodeURIComponent(errorCode) + '%3B';
		}else{
			urlval += 'errorCode=' + errorCode;
		}
		if (exportRequest) {
			return jobs;
		} else {
			return "<a href='" + urlval + "' target='_blank'>" + jobs + "</a>";
		}
	},
	"HOURS" : function(o, val, id, opts) {

		var format = function(o, val, id, opts) {
			if (val == null || val == undefined || val === 'null') {
				return "N/A";
			}
			var hh = val / 60;
			var aa = hh >= 12 ? "PM" : "AM";
			hh = (hh > 12) ? hh - 12 : (hh == 0) ? 12 : hh;
			return hh + ":00 " + aa;
		};

		var unformat = function(val) {
			if ((val.toLowerCase().indexOf("am") != -1) || (val.toLowerCase().indexOf("pm") != -1)) {
				val = val.trim();
				var index = (val.toLowerCase().indexOf("am") != -1) ? (val.toLowerCase().indexOf("am")) : (val
						.toLowerCase().indexOf("pm"));
				if (index + 2 != val.length) {
					return;
				}
				var incrementInTime = (val.toLowerCase().indexOf("am") != -1) ? 0 : (12 * 60);
				var hh = val.slice(0, val.indexOf(":"));
				var min = val.slice(val.indexOf(":") + 1, index - 1);
				if (!isNaN(hh) && !isNaN(min)) {//}  &&
					hh = parseInt(hh);
					min = parseInt(min);
					if ((hh >= 0 && hh <= 12) && (min >= 0 && min <= 60)) {
						if (hh == 12) {
							hh = 0;
						}
						return hh * 60 + min + incrementInTime;
					}
				}
				return;

			}
		};

		var validateVals = function() {
			return {
				allowedValues : [ function(val) {
					if (val.toLowerCase() == "n/a") {
						return val;
					} else {
						val = val.trim();
						if (val.indexOf(":") == -1) {
							return null;
						}
						var index = val.toLowerCase().indexOf("pm") != -1 ? val.toLowerCase().indexOf("pm") : val
								.toLowerCase().indexOf("am");
						if (index + 2 != val.length) {
							return null;
						}
						var hh = val.slice(0, val.indexOf(":"));
						var min = val.slice(val.indexOf(":") + 1, index);
						if (isNaN(hh) || isNaN(min)) {
							return null;
						} else {
							hh = parseInt(hh);
							min = parseInt(min);
							if ((hh >= 0 && hh <= 12) && (min >= 0 && min <= 60)) {
								return val;
							}
							return null;

						}
					}
					return null;
				} ],
				desc : "Please enter time as 10:00 AM",
				hint : "Please enter time as 10:00 AM"
			};
		};

		if (!!opts) {
			if (!!opts.unformat) {
				return unformat;
			} else if (!!opts.format) {
				return format;
			} else if (!!opts.validateVals) {
				return validateVals;
			} else {
				return null;
			}
		}
		// call format by default
		return format(o, val, id);
	},

	"alarmCheckBoxRenderer" : function(o, val, id) {
		var aId = o.aData["alarmId"];
		var cellVal = '<div class="selectCB custom-cb unchecked" data-state="unchecked" data-alarmid="' + aId +
				'"></div>';
		return cellVal;
	},

	"alarmNameLinkRenderer" : function(o, val, id) {
		var alarmURL = decodeURIComponent(o.aData["alarmURL"]);
		var reportURL = o.aData["reportURL"];
		var reportName = o.aData["report"];
		var reportEnum = o.aData["reportEnum"];
		var reportUniqId = o.aData["reportUniqId"];
		var aid = o.aData["alarmId"];
		var email = o.aData["email"];
		var emailCc = o.aData["emailCc"];
		var emailBcc = o.aData["emailBcc"];
		var description = o.aData["description"];
		var notificationTypes = o.aData["notificationTypes"];
		var type = o.aData["type"];
		var snmpdescription = o.aData["SNMPDescription"];
		var alertSeverity = o.aData["alertSeverity"];
		var alertStatus = o.aData["status"];
		var headerTemplate = o.aData["headerTemplate"];
		var isTileAlert = o.aData["isTileAlert"];
		var tileID = o.aData["tileID"];
		var alertCriteriaIdForTile = o.aData["alertCriteriaIdForTile"];
		var paramValue = o.aData["paramValue"];
		var operator = o.aData["alarmOperationType"];
		var commUniId = o.aData["commUniId"];
		var templateId = o.aData['templateId'];
		var ccGroupId = o.aData['ccGroupId'];
		var companyId = o.aData['companyId'];

		if (!val) {
			val = "No Name";
		}
		val= val.replace(/\&lt\;/g,"\<");	//changes for supporting alert notification through email
		val= val.replace(/\&gt\;/g,">");
		var tmpl = "<a class='alarmNameLink' data-alarmurl='" + alarmURL + "' data-reporturl='" + reportURL +
				"' data-report='" + reportName + "' data-aid='" + aid + "' data-name='" + _.escape(val) + "' data-email='" +
				email + "' data-emailcc='" + emailCc + "' data-emailbcc='" +emailBcc +
				"' data-description='" + description + "' data-notificationtypes='" + notificationTypes +
				"' data-type='" + type + "' data-snmpdescription='" + snmpdescription + "' data-alertseverity='" +
				alertSeverity + "' data-reportenum='" + reportEnum + "' data-reportuniqid='" + reportUniqId +
				"' data-alertstatus='" + alertStatus + "' data-headertemplate='" + headerTemplate +
				"' data-tile-alert='" + isTileAlert + "' data-tile-id='" + tileID + "' data-alert-criteria-id='" +
				alertCriteriaIdForTile + "' data-param-value='" + paramValue + "' data-operator='" + operator + "' data-comm-uni-id='" + commUniId + "' data-template-id='" + templateId +
				"' data-cc-group-id='" + ccGroupId +"' data-company-id='" + companyId +"' >#ALARMNAME#</a>";
		return tmpl.replace("#ALARMNAME#", $("<div/>").text(val).html());
	},

	"alarmStatusRenderer" : function(o, val, id) {
		if (val == 'Yes') {
			return "<span style='text-align:left'><span class='sprite icon-dot-green' alt='Enabled' style='vertical-align: middle' title='Enabled'></span>";
		} else {
			return "<span style='text-align:left'><span class='sprite icon-dot-gray' alt='Disabled' style='vertical-align: middle' title='Disabled'></span>";
		}

	},

	"largest25ClientsAgentsLink" : function(o, val,id){
		if(o && o.aData && !exportRequest){
			if(o.aData["CommServ UniqueId"] && o.aData["ClientId"] && o.aData["AgentCheck"]!=0){
				return '<a href="'+cvUtil.getContextPath()+'/reportsplus/reportViewer.jsp?reportId=clientapptypereport&input.CommCell='+o.aData["CommServ UniqueId"]+'&input.Client=%5B'+o.aData["ClientId"]+'%5D&commUniId='+o.aData["CommServ UniqueId"]+'">'+val+'</a>';
			}
		}
		return val;
	},

	"alarmCCName" : function(o, val, id) {
		var i = 0;
		if (!isNaN(val)) {
			var uniqId = parseInt(val, 10);
			if (!!window.commcellSearchData) {
				var rows = window.commcellSearchData;
				for (i = 0; i < rows.length; i++) {
					var uniqIdFrmSearchData = rows[i].commUniId;
					var name = rows[i].ccName;
					if (uniqId == uniqIdFrmSearchData) {
						return name;
					}
				}
			}
		} else if (val == "ww") {
			return "All CommCells(Worldwide)";
		} else if (val == "Custom Report") {
			return val;
		}
		return val;
	},

	"BillingTagsActions" : function(o, val, id) {
		var html = val + '<span class="tagActionIcons">';
		html += '<span href="" class="actionLinks edit sprite icon-edit" title="Edit"></span>';
		html += '<span class="actionLinks delete sprite  icon-trash" title="Delete"></span>';
		html += '</span>';
		return html;
	},

	"BillingAssociationsActions" : function(o, val, id) {
		var html = val + '<span class="tagActionIcons  sprite icon-trash actionLinks delete"/>';
		html += '</span>';
		return html;
	},

	"AssociateToCommCellGroup" : function(o, val, id) {
		var checked = parseInt(val) > 0 ? 'checked' : '';
		var html = '<span class="AssociateToCommCellGroup">';
		html += '<input type="checkbox" class="associatedToGroup" ' + checked + '>';
		html += '</span>';
		return html;
	},
	"RemoveUserFromCommCellGroup" : function(o, val, id) {
		var disabled = parseInt(val) == 2 ? 'disabled' : '';
		var html = '<span class="deleteUser sprite icon-trash ' + disabled + '" title="Remove User"> </span>';
		return html;
	},

	"chargebackCommCellName" : function(o, val, id) {
		return getChargebackDrilldownURL(o, val, 'CommCell Name');
	},

	"chargebackClientGroup" : function(o, val, id) {
		return getChargebackDrilldownURL(o, val, 'Client Groups');
	},

	"chargebackClient" : function(o, val, id) {
		return getChargebackDrilldownURL(o, val, 'Client');
	},
	"chargebackBillingTag" : function(o, val, id) {
		return getChargebackDrilldownURL(o, val, 'BillingTag');
	},
	"chargebackTenantName" : function(o, val, id) {
		return getChargebackDrilldownURL(o, val, 'Tenant');
	},
	"HHMM" : function(o, val, id, opts) {

		var format = function(o, val, id) {
			if (val == null || val == undefined || val === 'null') {
				return "N/A";
			}
			val = parseInt(val);
			if (val == -1) {
				return "N/A";
			}

			var mm = val % 60;
			var hh = (val - mm) / 60;
			return hh + ":" + mm;
		};
		var unformat = function(val) {
			if (val == "n/a") {
				return -1;
			}
			var seconds = Reports.unformatRelativeDate(val);
			if (seconds != null) {
				return seconds / 60;
			}
			return moment(val, "HH:mm").unix() - getTimezoneOffset();
		};
		var validateVals = function() {
			return {
				pattern : /^([0-9\.]+)\s*(year|month|week|day|hour|minute|second+)(s)?$/i,
				desc : "Relative Time (e.g. 5 Hours or 5 minutes)",
				hint : "Relative Time (e.g. 5 Hours or 5 minutes)"
			};
		};

		if (!!opts) {
			if (!!opts.unformat) {
				return unformat;
			} else if (!!opts.format) {
				return format;
			} else if (!!opts.validateVals) {
				return validateVals;
			} else if (!!opts.desc) {
				return desc;
			} else {
				return null;
			}
		}
		// call format by default
		return format(o, val, id);

	},
	"jobIdColumn" : function(o, val, noStatus) {
		var retval = '';
		if (!o.aData || ( !o.aData.status && !o.aData["Job Status"] )) {
			return val;
		}
		var status = "";
		if(o.aData.status){
			status = o.aData.status.toLowerCase();
		}else{
			status = o.aData["Job Status"].toLowerCase();
		}

		var statusIcon = "icon-dot-white";

		STATUS_COLOR_MAP = {
		  "completed": "icon-dot-green",
		  "completed with errors": "icon-dot-orange",
		  "completed with warnings": "icon-dot-yellow",
		  "failed": "icon-dot-red-lighter",
		  "killed": "icon-dot-burgundy",
		  "delayed": "icon-dot-purple",
		  "running": "icon-dot-blue",
		  "killed": "icon-dot-burgundy",
		  "no run": "icon-dot-gray-darker",
		  "committed": "icon-dot-cyan"
		};

		if (STATUS_COLOR_MAP[status]) {
			statusIcon = STATUS_COLOR_MAP[status];
		}

		retval = '<div>';
		if (!noStatus) {
			retval = retval + '<span class="jobStatusIcon sprite ' + statusIcon + '"></span>';
		}
		retval = retval + '<a href="/adminconsole/#/jobs/' + val + '">' + val + '</a></div>';
		return retval;

	},

	"parentJobIdColumn" : function(o, val) {
		if (!!val && val != 'N/A') {
			return Reports.Renderers["jobIdColumn"](o, val, true);
		}
		return val;
	},

	"CommCellGroup" : function(o, val, id) {
		var ccGroupId = o.aData[o.oSettings.aoColumns[0].mDataProp];
		var urlval = cvUtil.getContextPath() + "/survey/reports/dashboard.jsp?ccGroupId=" + ccGroupId;
		if (exportRequest) {
			return val;
		} else {
			return "<a href='" + urlval + "' class='ccGroupDrillDown'>" + val + "</a>";
		}
	},

	"FORMATNUMBERNA" : function(o, val, id, opts) {
		var format = function(o, val, id) {
			val = val.toString();
			var pattern = /(-?\d+)(\d{3})/;
			while (pattern.test(val)) {
				val = val.replace(pattern, "$1,$2");
			}
			return val;
		};

		var unformat = function(val) {
			return val.toString().replace(/,/g, "");
		};
		var validateVals = function() {
			return {
				allowedValues : [function(val) {
					return $.isNumeric(val);
				} ],
				desc : "numeric value",
				hint : "Numeric value"
			};
		};

		if (!!opts) {
			if (!!opts.format) {
				return format;
			} else if (!!opts.unformat) {
				return unformat;
			} else if (!!opts.validateVals) {
				return validateVals;
			}  else {
				return null;
			}
		}
		return format(o, val, id);
	},

	"CommcellStatusRenderer" : function(o, val, id, opts) {
		var format = function(o, val, id) {
			var html = "";
			var str = val.toLowerCase();
			if (exportRequest == 'csv') {
				return val;
			}
			if (str == 'enabled') {
				html = '<span title="Enabled" alt="Enabled" class="sprite icon-dot-green"></span>';
			} else if (str == 'disabled') {
				html = '<span title="Disabled" alt="Disabled" class="sprite icon-dot-red"></span>';
			}
			return html;
		};
		var unformat = function(val) {
			if (val == 'Enabled' || val == '1') {
				return 'Enabled';
			} else if (val == 'Disabled' || val == '0') {
				return "Disabled";
			}
		};
		var validateVals = function() {
			return {
				allowedValues : [ "yes", "no", "true", "false", "1", "0" ],
				desc : "Disabled;/Enabled",
				hint : "Disabled or Enabled"
			};
		};

		if (!!opts) {
			if (!!opts.unformat) {
				return unformat;
			} else if (!!opts.format) {
				return format;
			} else if (!!opts.validateVals) {
				return validateVals;
			} else {
				return null;
			}
		}
		// call format by default
		return format(o, val, id);
	},
	"MSCustomer" : function(o, val, id) {
		var custId = o.aData['CustomerId<DONOTSHOW>'];
		var dcId = cvUtil.getParameter('dcId', 0);
		var type = cvUtil.getParameter('type', 'backup');
		var dateType = cvUtil.getParameter('dateType', 1);
		var lastNdays = cvUtil.getParameter('lastNdays', -1);
		var startDate = cvUtil.getParameter('startDate');
		var endDate = cvUtil.getParameter('endDate');

		var params = "type=" + type + "&custId=" + custId + "&dcId=" + dcId + "&dateType=" + dateType + "&lastNdays=" +
				lastNdays;
		if (startDate) {
			params += "&startDate=" + startDate;
		}
		if (endDate) {
			params += "&endDate=" + endDate;
		}

		var urlval = cvUtil.getContextPath() + "/reports/msbackupstats.jsp?" + params;
		if (exportRequest) {
			return val;
		} else {
			return "<a href='" + urlval + "'>" + val + "</a>";
		}
	},
	"MSServer" : function(o, val, id) {
		var type = cvUtil.getParameter('type', 'backup');
		var custId = o.aData['CustomerId<DONOTSHOW>'];
		var dcId = o.aData['DataCenterId<DONOTSHOW>'];
		var serverId = o.aData['ServerId<DONOTSHOW>'];
		var serverName = o.aData['Server<MSServer>'];
		var dateType = cvUtil.getParameter('dateType', 1);
		var lastNdays = cvUtil.getParameter('lastNdays', -1);
		var startDate = cvUtil.getParameter('startDate');
		var endDate = cvUtil.getParameter('endDate');
		var params = "type=" + type + "&custId=" + custId + "&dcId=" + dcId + "&serverId=" + serverId + "&dateType=" +
				dateType + "&lastNdays=" + lastNdays + "&serverName=" + serverName;
		if (startDate) {
			params += "&startDate=" + startDate;
		}
		if (endDate) {
			params += "&endDate=" + endDate;
		}
		var urlval = cvUtil.getContextPath() + "/reports/msbackupstats.jsp?" + params;
		if (exportRequest) {
			return val;
		} else {
			return "<a href='" + urlval + "'>" + val + "</a>";
		}
	},
	"MSCustomerJobs" : function(o, val, id) {
		var custId = o.aData['CustomerId<DONOTSHOW>'];
		var dcId = cvUtil.getParameter('dcId', 0);
		var type = cvUtil.getParameter('type', 'backup');
		var dateType = cvUtil.getParameter('dateType', 1);
		var lastNdays = cvUtil.getParameter('lastNdays', -1);
		var startDate = cvUtil.getParameter('startDate');
		var endDate = cvUtil.getParameter('endDate');
		var params = "type=" + type + "&custId=" + custId + "&dateType=" + dateType + "&lastNdays=" + lastNdays +
				"&showJobs=true&dcId=" + dcId;
		if (startDate) {
			params += "&startDate=" + startDate;
		}
		if (endDate) {
			params += "&endDate=" + endDate;
		}
		var urlval = cvUtil.getContextPath() + "/reports/msbackupstats.jsp?" + params;
		if (exportRequest) {
			return val;
		} else {
			return "<a href='" + urlval + "'>" + val + "</a>";
		}
	},
	"MSSearchServerJobs" : function(o, val, id) {
		var type = cvUtil.getParameter('type', 'backup');
		var custId = o.aData['CustomerId<DONOTSHOW>'];
		var dcId = o.aData['DataCenterId<DONOTSHOW>'];
		var serverId = o.aData['ServerId<DONOTSHOW>'];
		var serverName = o.aData['Server<MSSearchServerJobs>'];
		var dateType = cvUtil.getParameter('dateType', 2);
		var lastNdays = cvUtil.getParameter('lastNdays', -1);
		var startDate = cvUtil.getParameter('startDate');
		var endDate = cvUtil.getParameter('endDate');
		var params = "type=" + type + "&custId=" + custId + "&dcId=" + dcId + "&serverId=" + serverId + "&dateType=" +
				dateType + "&lastNdays=" + lastNdays + "&serverName=" + serverName;
		if (startDate) {
			params += "&startDate=" + startDate;
		}
		if (endDate) {
			params += "&endDate=" + endDate;
		}
		var urlval = cvUtil.getContextPath() + "/reports/msbackupstats.jsp?" + params;
		if (exportRequest) {
			return val;
		} else {
			return "<a href='" + urlval + "'>" + val + "</a>";
		}
	},
	"MSSuccessJobs" : function(o, val, id) {
		return getMSJobStatusLinkByMonth(o, val, id, 1);
	},
	"MSFailedJobs" : function(o, val, id) {
		return getMSJobStatusLinkByMonth(o, val, id, 2);
	},
	"MSOtherJobs" : function(o, val, id) {
		return getMSJobStatusLinkByMonth(o, val, id, 3);
	},
	"MSSuccessJobsDay" : function(o, val, id) {
		return getMSJobStatusLinkByDay(o, val, id, 1);
	},
	"MSFailedJobsDay" : function(o, val, id) {
		return getMSJobStatusLinkByDay(o, val, id, 2);
	},
	"MSOtherJobsDay" : function(o, val, id) {
		return getMSJobStatusLinkByDay(o, val, id, 3);
	},
	"MSJobsRate" : function(o, val, id) {
		val = parseFloat(val);
		if (exportRequest) {
			return val;
		} else {
			if (val >= 99.5) {
				return "<font color='green'>" + val + "</font>";
			} else {
				return "<font color='red'>" + val + "</font>";
			}
		}
		return val;
	},
	"strikesCCName" : function(o, val, id) {
		var url = window.location.href;
		var uniqId = o.aData[o.oSettings.aoColumns[0].mDataProp];
		url = url + "&commUniId=" + uniqId;
		if (exportRequest || (uniqId && uniqId == -10)) {
			return val;
		} else {
			return "<a href='" + url + "'>" + val + "</a>";
		}
	},
	"CC_DRILLDOWN" : function(o, val, id) {
		/// This will turn the cell into a link to the current URL with the
		/// CommCell ID appended; designed to drilldown in tables
		var url = cvUtil.getContextPath() + "/health/health.jsp";
		var params = window.location.search;
		var uniqId = o.aData[o.oSettings.aoColumns[0].mDataProp];

		url = url + params;

		if (params !== "") {
			url = url + "&commUniId=" + uniqId;
		} else {
			url = url + "?commUniId=" + uniqId;
		}

		if (exportRequest) {
			return val;
		} else {
			return "<a href='" + url + "' class='ccDrillDown'>" + val + "</a>";
		}
	},
	"TagEntityLevel" : function(o, val, id) {
		val = parseInt(val);
		if (val == -1) {
			return "N/A";
		} else if (val == 0) {
			return "SubClient";
		} else if (val == 1) {
			return "Client";
		}
		return val;
	},
	"BillingTagSizeType" : function(o, val, id) {
		val = parseInt(val);
		if (val === 0) {
			return "Any";
		} else if (val === 1) {
			return "Frontend Backup";
		} else if (val === 2) {
			return "Frontend Archive";
		} else if (val === 3) {
			return "Application";
		} else if (val === 4) {
			return "Media";
		}
		return val;
	},
	"URL" : function(o, val, id) {
		if (val == undefined || val == '') {
			return '';
		}
		if (exportRequest) {
			return val;
		}
		var urllink = val;
		if (typeof urllink === 'string') {
			if (urllink.indexOf('http') < 0) {
				urllink = 'http://' + val;
			}
		}
		return "<a href='" + urllink + "' target='_blank'>" + val + "</a>";
	},
	"Task" : function(o, val, id) {
		if (exportRequest) {
			return val;
		}
		var id = o.aData["Task Id"];
		if (id) {
			var ccGroupId = cvUtil.getParameter('ccGroupId', 0);
			var companyId = cvUtil.getParameter('companyId', 0);
			var href = cvUtil.getContextPath() + '/survey/profileTask.do?ccGroupId=' + ccGroupId + '&companyId='+ companyId +'&taskId=' + id;
			return "<a class='taskLink' data-tid='" + id + "' href='" + href + "'>" + val + "</a>";
		} else {
			return val;
		}
	},
	"TaskStatus" : function(o, val, id) {
		return reportsUtil.getTaskStatus(val);
	},
	"TaskType" : function(o, val, id) {
		return reportsUtil.getTaskType(val);
	},

	"reportConfigurationActions" : function(o, val, id, opts) {
		var format = function(o, val, id) {
			if (!window.location.origin) {
				window.location.origin = window.location.protocol + "//" + window.location.host;
			}
			var reportStatus = o.aData["Status"];
			if (reportStatus === "Enabled") {
				var url = window.location.origin + cvUtil.CONTEXT_PATH + o.aData["Hyperlink<DONOTSHOW>"];
				if(surveySingleCommcellUser){
					var ccParams =(url.indexOf('?') > 0 ? '&':'?') + 'commUniId='+surveySingleCommcellUser;
					url = url + ccParams;
					url = url.replace('CommCellSurvey.jsp', 'indcommcellsurvey.jsp');
				}
				val = '<a target="_blank" href=' + url + '>' + val + '</a>';
			}
			var html = val + '<span class="tagActionIcons">';
			html += '<span href="" class="actionLinks edit sprite icon-edit" title="Edit"></span>';
			html += '</span>';
			return html;
		};
		var unformat = function(val) {
			return $("<div/>").html(val).text();
		};
		if (!!opts) {
			if (!!opts.unformat) {
				return unformat;
			} else if (!!opts.format) {
				return format;
			} else if (!!opts.customValueFn) {
				return unformat;
			} else {
				return null;
			}
		}
		return format(o, val, id);
	},
	"DATEUNIT" : function(o, val, id, opts) {
		var flag = o.aData["Flags<DONOTSHOW>"];
		var dayUnit = "";

		if (flag & 4) {
			if (parseInt(val) > 1) {
				dayUnit = " Months"
			} else {
				dayUnit = " Month";
			}
		} else {
			if (parseInt(val) > 1) {
				dayUnit = " Days"
			} else {
				dayUnit = " Day"
			}
		}
		return (val + dayUnit);
	},
	"TAMLocation" : function(o, val, id) {
		var loc = o.aData["TAM Location<TAMLocation>"];
		if (loc == 1) {
			return 'On-site';
		} else {
			return 'Remote'
		}
	},
	"FeedbackNotes" : function(o, val, id) {
		var notes = o.aData["Notes<FeedbackNotes>"];
		try {
			notes = window.decodeURIComponent(notes);
		} catch (err) {
			console.debug('Plain feedback notes.');
		}
		return notes;
	},
	"DBINDEXFRAG1" : function(o, val, id) {
		return getDBIndexFragmentationURL(o, val, 1);
	},
	"DBINDEXFRAG2" : function(o, val, id) {
		return getDBIndexFragmentationURL(o, val, 2);
	},

	"DifferenceNA" : function(o, val, id) {
		val = parseFloat(val);
		if (!val) {
			return "N/A";
		}
		return val;
	},
	"DifferenceMB" : function(o, val, id, opts) {
		if (!opts || !opts.unformat) {
			if (!val) {
				return "N/A";
			}
		}
		return Reports.Renderers["capacityMB_Neg"](o, val, id, opts);
	},
	"partitionInfo" : function(o, val, id){
        if (exportRequest) {
            return val;
        } else {
			var uniqId = o.aData[o.oSettings.aoColumns[0].mDataProp];
			var storeId = o.aData[o.oSettings.aoColumns[6].mDataProp];
			var urlval = cvUtil.getContextPath() + "/survey/reports/indcommcellsurvey.jsp?surveyId=65&commUniId=" + uniqId + "&storeId=" + storeId ;
            return "<a href='" + urlval + "'>" + val + "</a>";
        }
	},
	"activityJobDetails" : function(o, val, id){
        if (exportRequest) {
            return val;
        } else {
			var uniqId = o.aData[o.oSettings.aoColumns[2].mDataProp];
			var clientId = o.aData[o.oSettings.aoColumns[3].mDataProp];
			var strikeSinceUnformatted = o.aData[o.oSettings.aoColumns[16].mDataProp];
			var strikeSince = moment(strikeSinceUnformatted*1000).format('YYYY-MM-DD hh:mm:ss').replace(' ','%20');
			var currentDate = moment(Date.now()).format('YYYY-MM-DD hh:mm:ss').replace(' ','%20');
			var urlval = cvUtil.getContextPath() + "/survey/reports/indcommcellsurvey.jsp?commUniId=" + uniqId + "&surveyId=137&jobStatus=0&startDate=" + strikeSince + "&endDate=" + currentDate + "&repType=hourly&clientId=" + clientId;
            return "<a href='" + urlval + "'>" + val + "</a>";
        }
	},
	"TagPrice" : function(o, val, id, opts) {
		var format = function(o, val, id) {
			val = parseFloat(val);
			if (val == -1) {
				return "Global Price";
			} else if (val < 0) {
				return "N/A";
			}
			return Reports.Renderers.FORMATNUMBERNA(o, val, id, opts);
		};
		var unformat = function(val) {
			if (val == "n/a") {
				return -2;
			} else if (val == "Global Price") {
				return -1;
			}
			return Reports.Renderers.FORMATNUMBERNA(o, val, id, opts)(val);
		};
		var validateVals = function() {
			return {
				allowedValues : [ "n/a", "global price", function(val) {
					return $.isNumeric(val) && val >= 0;
				} ],
				desc : "non-negative numeric",
				hint : "Non-negative numeric, Global Price or N/A"
			};
		};

		if (!!opts) {
			if (!!opts.unformat) {
				return unformat;
			} else if (!!opts.format) {
				return format;
			} else if (!!opts.validateVals) {
				return validateVals;
			} else {
				return null;
			}
		}
		// call format by default
		return format(o, val, id);
	},
	"MetricsToCustomLink" : function(o, val, id){
        if (exportRequest) {
            return val;
        } else {
			var uniqId = o.aData[o.oSettings.aoColumns[0].mDataProp];
			var reportId = o.aData[o.oSettings.aoColumns[3].mDataProp];
			var urlval = cvUtil.getContextPath() + "/reportsplus/reportViewer.jsp?reportId=" + reportId + "&input.CommservUniqueId=" + uniqId;
            return "<a href='" + urlval + "'>" + val + "</a>";
        }
	}

/*
 * Template "" : function(o, val, id, opts) { var format = function(o, val, id) { }; var unformat =
 * function(val) { }; var validateVals = function() { return { }; };
 * 
 * if(!!opts) { if(!!opts.unformat) { return unformat; } else if(!!opts.format) { return format; } else
 * if(!!opts.validateVals) { return validateVals; } else if(!!opts.desc) { return desc; } else { return null; } } //
 * call format by default return format(o, val, id); },
 */
};

/*
 * This below function is used to validate filter input value or the value entered while creating alert.
 */
Reports.validateValue = function(value, validationObj, type) {
	/*
	 * input is a string of non-zero length validation object contains the data type of input, allowed values,
	 * and patterns if any.
	 */
	value = value.toString().toLowerCase();
	if (validationObj) {
		var allowedValues = validationObj.allowedValues;
		var pattern = validationObj.pattern;

		if (!!allowedValues && $.isArray(allowedValues)) {
			var retval = {
				status : false
			};
			for (var i = 0; i < allowedValues.length; i++) {
				var av = allowedValues[i];
				if ($.isFunction(av)) {
					if (av.call(this, value)) {
						retval = {
							status : true
						};
						break;
					}
				} else if (av == value) {
					retval = {
						status : true
					};
					break;
				}
			}
			if (retval.status) {
				return retval;
			}
		}

		if (!!pattern && pattern instanceof RegExp) {
			if (pattern.test(value)) {
				return {
					status : true
				};
			} else {
				return {
					status : false,
					desc : validationObj.desc
				};
			}
		}
		return {
			status : false,
			desc : validationObj.desc
		};
	} else if (!!type) {
		type = type.toLowerCase();
		var numericSqlTypes = [
				"bit",
				"tinyint",
				"smallint",
				"int",
				"bigint",
				"numeric",
				"decimal",
				"float",
				"real",
				// H2 types mapping for Custom Reports
				"integer",
				"mediumint",
				"int4",
				"signed",
				"tinyint",
				"smallint",
				"int2",
				"year",
				"bigint",
				"int8",
				"identity",
				"decimal",
				"number",
				"dec",
				"numeric",
				"double",
				"precision",
				"float",
				"float8",
				"real",
				"float4"
		// char types , "varchar", "varchar_ignorecase", "char"
		];
		if ($.inArray(type, numericSqlTypes) > -1) {
			return {
				status : $.isNumeric(value),
				desc : "numeric"
			};
		} else if (value.length) {
			return {
				status : true
			};
		}
	}
	// shouldn't reach here..
	return {
		status : false,
		desc : "some"
	};
};

Reports.unformatRelativeDate = function(val) {
	var relativeRegex = /^([0-9\.]+)\s*(year|month|week|day|hour|minute|second+)(s)?$/i;
	if (relativeRegex.test(val)) {
		var numAndUnits = relativeRegex.exec(val);
		var num = parseFloat(numAndUnits[1]);
		var unit = numAndUnits[2] + "s";
		var seconds = moment.duration(parseInt(num, 10), unit).asMilliseconds() / 1000;
		return seconds;
	}
	return null;
};

function getFormattedDateTimeFromUTC(o, val, id, opts, pattern) {
	var format = function(o, val, id) {
		val = parseInt(val, 10);
		if (val <= 0) {
			return " ";
		}
		return getDate(val, pattern);
	};
	var unformat = function(val) {
		var seconds = Reports.unformatRelativeDate(val);
		if (seconds != null) {
			return seconds;
		}
		// returning UTC timestamp
		var dval = (Date.parse(val) / 1000) - getTimezoneOffset();
		return dval;
	};
	var validateVals = function() {
		return {
			pattern : /^([0-9\.]+)\s*(year|month|week|day|hour|minute|second+)(s)?$/i,
			desc : "Relative Time (e.g. 5 Hours or 5 Days)",
			hint : "Relative Time (e.g. 5 Hours or 5 Days)"
		};
	};

	if (!!opts) {
		if (!!opts.unformat) {
			return unformat;
		} else if (!!opts.format) {
			return format;
		} else if (!!opts.validateVals) {
			return validateVals;
		} else {
			return null;
		}
	}

	// call format by default
	return format(o, val, id);
}

function getChargebackDrilldownURL(o, val, filterCol) {
	if (exportRequest || val == 'Total of All Rows') {
		return val;
	}

	var urlval = window.location.href;
	urlval = urlval.replace(window.location.search, "");

	var str = "?";
	var commUniId = cvUtil.getParameter('commUniId', 0);
	var startDate = cvUtil.getParameter('startDate', 1);
	var sizeUnit = cvUtil.getParameter('sizeUnit', 'TB');
	var groupBy = cvUtil.getParameter('groupBy', 'ClientGroup');
	var repType = cvUtil.getParameter('repType', 1);

	str += "startDate=" + startDate + "&sizeUnit=" + sizeUnit + "&repType=" + repType;
	if (commUniId) {
		str += "&commUniId=" + commUniId;
	}
	if (cvUtil.getParameter('ccGroupId', -1) > 0) {
		str = str + "&ccGroupId=" + cvUtil.getParameter('ccGroupId');
	}
	if (cvUtil.getParameter('companyId', 0) > 0) {
		str = str + "&companyId=" + cvUtil.getParameter('companyId');
	}

	if(filterCol === 'BillingTag'){
		str += "&groupBy=BillingTag&drillFrom=" + groupBy+"&drillEntity=" + encodeURIComponent(val);
	}else if(filterCol === 'Tenant'){
		str += "&groupBy=SubClient&drillFrom=" + groupBy+"&drillEntity=" + encodeURIComponent(val);
	}else{
		str += "&groupBy=SubClient&drillFrom=" + groupBy;
		var filterClause = filterCol + '-LIKE-=' + val + ';';
		var entities = val;
		if (filterCol == 'Client') {
			var ccName = o.aData["CommCell Name"];
			if (ccName) {
				filterClause += 'CommCell Name-LIKE-=' + ccName + ';';
				entities += ';' + ccName;
			}
		}
		str += '&drillEntity=' + encodeURIComponent(entities);
		var filter = '&filteredTable-0=entityTable&entityTable-filterClause=' + encodeURIComponent(filterClause);
		str += filter;
	}

	var cost = o.aData["Total Cost"];
	if (cost) {
		str += '&totalCost=' + cost;
	}

	urlval += str;
	return "<a href='" + urlval + "'>" + val + "</a>";
}

function getMSJobStatusLinkByMonth(o, val, id, jobStatus) {

	if (exportRequest) {
		return val;
	} else {
		var intVal = parseInt(val);
		var type = cvUtil.getParameter('type', 'backup');
		var dcId = o.aData['DataCenterId<DONOTSHOW>'];
		var dcName = o.aData['DC'];
		var startDate = cvUtil.getParameter('startDate');
		var params = "type=" + type + "&jobStatus=" + jobStatus + "&dcId=" + dcId + "&dcName=" + dcName;
		if (startDate) {
			params += "&startDate=" + startDate;
		}
		var urlval = cvUtil.getContextPath() + "/reports/msbackupscorecard.jsp?" + params;
		if (jobStatus == 1) {//success
			if (intVal > 0) {
				return "<a href='" + urlval + "'>" + intVal + "</a>";
			} else {
				return "<font>" + intVal + "</font>";
			}
		} else if (jobStatus == 2) {//failed
			if (intVal > 0) {
				return "<a href='" + urlval + "'><font color='red'>" + intVal + "</font></a>";
			} else {
				return "<font color='green'>" + intVal + "</font>";
			}
		} else if (jobStatus == 3) {//others
			if (intVal > 0) {
				return "<a href='" + urlval + "'>" + intVal + "</a>";
			} else {
				return "<font>" + intVal + "</font>";
			}
		}

	}
	return val;
}

function getMSJobStatusLinkByDay(o, val, id, jobStatus) {

	if (exportRequest) {
		return val;
	} else {
		var intVal = parseInt(val);
		var type = cvUtil.getParameter('type', 'backup');
		var params = "type=" + type + "&jobStatus=" + jobStatus;
		var startDate = o.aData[o.oSettings.aoColumns[0].mDataProp];
		if (startDate) {
			var mm = moment(new Date(startDate));//, 'YYYY-MM-DD');
			startDate = mm.format('MM/DD/YYYY');
			var eDate = mm.toDate();
			eDate.setDate(eDate.getDate() + 1);
			var endDate = moment(eDate).format('MM/DD/YYYY');
			params += "&startDate=" + startDate + "&endDate=" + endDate;
		}
		var urlval = cvUtil.getContextPath() + "/reports/msbackupscorecard.jsp?" + params;
		if (jobStatus == 1) {//success
			if (intVal > 0) {
				return "<a href='" + urlval + "'>" + intVal + "</a>";
			} else {
				return "<font>" + intVal + "</font>";
			}
		} else if (jobStatus == 2) {//failed
			if (intVal > 0) {
				return "<a href='" + urlval + "'><font color='red'>" + intVal + "</font></a>";
			} else {
				return "<font color='green'>" + intVal + "</font>";
			}
		} else if (jobStatus == 3) {//others
			if (intVal > 0) {
				return "<a href='" + urlval + "'>" + intVal + "</a>";
			} else {
				return "<font>" + intVal + "</font>";
			}
		}
	}
	return val;
}

function getDBIndexFragmentationURL(o, val, percentInterval) {
	var database = o.aData['Database'];
	var params = "database=" + database + "&indexInterval=" + percentInterval;
	var surveyId = cvUtil.getParameter('surveyId', 144);
	var commUniId = cvUtil.getParameter('commUniId', 0);
	var ccGroupId = cvUtil.getParameter('ccGroupId', -1);
	var companyId = cvUtil.getParameter("companyId", 0);
	if (commUniId > 0) {
		params += "&commUniId=" + commUniId;
	} else {
		var cId = o.aData['CommServ UniqueId'];
		if (cId) {
			params += "&commUniId=" + cId;
		}
	}
	if (ccGroupId > 0) {
		params += "&ccGroupId=" + ccGroupId;
	}
	if (companyId > 0) {
		params += "&companyId=" + companyId;
	}
	params += "&surveyId=" + surveyId;
	var urlval = cvUtil.getContextPath() + "/survey/reports/indcommcellsurvey.jsp?";
	urlval = urlval + params;
	if (exportRequest) {
		return val;
	} else {
		return "<a href='" + urlval + "'>" + val + "</a>";
	}
};