/*
 3 way checkbox tree using dynatree
 */

function StorageFilter(options) {
	this.options = options;
	this.requestString = options.requestString;
	if (!options.clientSideSource) {
		if (!options.ajaxsource)
			options.ajaxsource = cvUtil.getContextPath() + // "/server/report?type=storageinformation&component=StoragePoliciesTree";
			'/server/report?type=storageinformation&json=' +
					window.encodeURIComponent('{"invisible":{"initial":true},"component":"StoragePoliciesTree"}');

		cvUtil.loadPageWithGET(options.ajaxsource, "", function(data) {
			buildSPTree(options.treeid, data["spTree"]);
			buildTree(options.librariesid, data["libraries"], requestString.libraryFilter);
			buildTree(options.locationsid, data["locations"], requestString.locationFilter);
			initCheckboxes();
		});

	}
	function buildSPTree(treeid, data) {
		$(treeid).dynatree({
			checkbox : true,
			selectMode : 3,
			children : data[0],
			onKeydown : function(node, event) {
				if (event.which == 32) {
					node.toggleSelect();
					return false;
				}
			},
			autoCollapse : false,
			onPostInit : function(isReloading, isError) {
			},
			onCreate : function(node, nodeSpan) {
				initSPTree(node);
			}

		});
	}

	this.buildPolicyTree = function(treeid, data) {
		buildSPTree(treeid, data);
	};

	function buildTree(id, data, filter) {
		$(id).dynatree({
			checkbox : true,
			selectMode : 3,
			children : data[0],
			onKeydown : function(node, event) {
				if (event.which == 32) {
					node.toggleSelect();
					return false;
				}
			},
			autoCollapse : false,
			onPostInit : function(isReloading, isError) {
			},
			onCreate : function(node, nodeSpan) {
				initTree(id, node, filter);
			}
		});
	}

	function initSPTree(node) {
		if ($(options.treeid).dynatree("getTree").isReloading() == false) {
			if (typeof requestString.copyFilter[0] === 'string' && requestString.copyFilter[0] == 'all') {
				node.select();
			} else {
				for ( var i in requestString.copyFilter) {
					var obj = requestString.copyFilter[i];
					if (obj.policy)
						obj.policy = decodeURI(obj.policy);
					if (obj.copy)
						obj.copy = decodeURI(obj.copy);

					var copies = obj.copy.split('~');

					var children = $(options.treeid).dynatree("getTree").getNodeByKey(obj.policy).getChildren();

					for ( var j in children) {
						var child = children[j];
						for (var k = 0; k < copies.length; k++) {
							if (child.data.key == copies[k]) {
								child.select();
								break;
							}
						}
					}

				}
			}
		}
	}

	function initTree(id, node, filter) {
		if ($(id).dynatree("getTree").isReloading() == false) {
			if (typeof filter[0] === 'string' && filter[0] == 'all') {
				node.select();
			} else {
				for ( var i in filter) {
					var obj = filter[i];
					obj = decodeURI(obj);
					$(id).dynatree("getTree").getNodeByKey(obj).select(true);
				}
			}
		}
	}

	this.updatePolicies = function() {
		requestString.copyFilter = [];
		var allPoliciesSelected = true;
		var children = $(options.treeid).dynatree("getTree").getRoot().getChildren();
		for ( var i in children) {
			var node = children[i];
			if (node.isSelected() == false) {
				allPoliciesSelected = false;
				break;
			}
			;
		}
		if (allPoliciesSelected) {
			requestString.copyFilter.push('all');
			return false;
		}
		var selectedNodes = $(options.treeid).dynatree("getTree").getSelectedNodes(true);
		var selectedPolicies = $.map(selectedNodes, function(node) {
			var pObj = {};
			if (node.data.type == 'policy') {
				pObj.policy = node.data.key;
				var copies = node.getChildren();
				pObj.copy = "";
				for ( var i in copies) {
					var child = copies[i];
					pObj.copy = child.data.key + '~' + pObj.copy;
				}
			} else {
				pObj.policy = node.getParent().data.key;
				pObj.copy = node.data.key;
			}
			return pObj;
		});
		if (selectedPolicies.length == 0) {
			selectedPolicies.push('all');
		}
		requestString.copyFilter = selectedPolicies;
	};

	this.updateLibsAndLocs = function() {
		var treeIds = [ options.librariesid, options.locationsid ];
		for ( var i in treeIds) {
			var id = treeIds[i];
			var filter = [];
			var allSelected = true;
			var children = $(id).dynatree("getTree").getRoot().getChildren();
			for ( var j in children) {
				var node = children[j];
				if (node.isSelected() == false) {
					allSelected = false;
					break;
				}
				;
			}
			if (allSelected) {
				filter.push('all');
			} else {
				var selectedNodes = $(id).dynatree("getTree").getSelectedNodes(true);
				var selectedItems = $.map(selectedNodes, function(node) {
					return node.data.key;
				});
				if (selectedItems.length == 0) {
					selectedItems.push('all');
				}
				filter = selectedItems;
			}

			if (id == options.librariesid) {
				requestString.libraryFilter = filter;
			} else {
				requestString.locationFilter = filter;
			}
		}
	};
}

function storageFilterList(options) {
	this.policyUl = options.id;
	this.list = options.list;

	this.buildList = function(policyUl, list) {
		$.each(list, function(key, value) {
			if (key == 0) {
				$("#" + policyUl).html('<option value="all">' + value + '</option>');
				return;
			}
			$("#" + policyUl).append('<option value="' + value + '">' + value + '</option>');
		});

	};

	this.updateList = function(list) {
		this.buildList(this.policyUl, list);
		this.list = list;
		$("#storagePolicies").multiselect({
			header : false,
			selectedList : 2
		});
		$("#storagePolicies").bind("multiselectclick",
				function(event, ui) {
					if (ui.value == "all") {
						if (ui.checked) {
							$("#storagePolicies").multiselect("checkAll");
						} else {
							$("#storagePolicies").multiselect("uncheckAll");
						}
					} else {
						if (ui.checked) {
							if ($("#storagePolicies").multiselect("getChecked").length == (list.length - 1)) {
								$("#storagePolicies").multiselect("checkAll");
							}
						} else {
							$($("#storagePolicies").multiselect("widget").find("ul li")[1]).find("input")
									.removeAttr("aria-selected").attr("checked", false);
						}
					}

					var selections = $("#storagePolicies").multiselect("getChecked");
					if (selections.length == 0) {
						$("#storagePolicies").trigger('selectionChanged', false);
					} else {
						if (selections.length == list.length) {
							$("#storagePolicies").multiselect({
								selectedText : "All selected"
							});
						} else {
							$("#storagePolicies").multiselect({
								selectedText : selections.length + " selected"
							});
						}
						$("#storagePolicies").trigger('selectionChanged', true);
					}

				});

	};

	if (this.list)
		this.buildList(this.policyUl, this.list);

	this.getSelectedList = function() {
		return $("#storagePolicies").multiselect("getChecked");
	};

	this.setSelectedList = function(arr) {
		$("#storagePolicies").multiselect("widget").find(":checkbox").each(function() {
			if ($.inArray(this.value, arr) > -1) {
				if (!this.checked) {
					this.click();
				}
			}
		});
	};

	this.populateRequestString = function() {
		requestString.copyFilter = [];
		var selection = this.getSelectedList();
		if ($(selection[0]).attr("value") == "all")
			requestString.copyFilter.push('all');
		else {
			$.each(selection, function(key, value) {
				requestString.copyFilter.push($(value).attr("value"));
			});

		}

	};

}
