// ============================================================================
//  Copyright (c) CommVault Systems,
//  All Rights Reserved
// 
//  THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF CommVault Systems
//  The copyright notice above does not evidence any
//  actual or intended publication of such source code.                       
// 
//  File name: storageinformation.js
//  Author: Hari Ravula
// ============================================================================

var chartData = null;
var chartOtherDrillIndex = 0;
var MAX_NUM_PIE_SLICE = 10;
var librariesFilter = new Array();

function StorageInformationReport() {
	this.detailsClicked = false;
	this.librariesFilterPanel = null;
	this.locationsFilterPanel = null;
	this.policiesFilterPanel = null;
	this.savedFilters = null;

	this.report = new Reports.initReport({
		type : 'storageinformation',
		generateButtonId : 'btngenerate',
		chartIds : [ 'piechartcontainer' ]
	});

	this.requestString = report.requestString;

	report.initializeDefaults = function() {
		report.requestString["type"] = "StorageInformation";
		report.requestString["mode"] = 1;
		report.requestString["distribution"] = "MediaType";
		report.requestString["groupBy"] = "MediaType";
		report.requestString["orderBy"] = "BARCODE";
		report.requestString["mediaStatus"] = [ "all" ];
		report.requestString["mediaGroup"] = [ "all" ];
		report.requestString["mediaState"] = "all";
		report.requestString["mediaType"] = [ "all" ];
		report.requestString["exportedOption"] = "ALL";
		report.requestString["libraryFilter"] = [ "all" ];
		report.requestString["locationFilter"] = [ "all" ];
		report.requestString["copyFilter"] = [ "all" ];
		report.requestString["copyType"] = [ "all" ];
		report.requestString["existsIn"] = "and";
		report.requestString.reportFilters = [];
		report.requestString.invisible.drilldown = false;

		var groupBy = $('#groupByPanel li');
		var selectedLI = groupBy.index(groupBy.filter('.selectedOption'));
		if (selectedLI != 0 && savedFilters == null) {
			$('#groupByPanel :nth-child(1)').click();
		}

	};

	var librariesAll, librariesBoxes, locationsAll, locationsBoxes, error_note, generateBtn;
	function minimumSelectionCheck(checkall, boxes, note, errorClass, libraryFlag) {
		checkall.bind('checkedState', function() {
			boxes.each(function() {
				checkCb($(this));
			});
			$("." + errorClass).remove();
			error_note.children(":first").css("display", "block");

			if (error_note.is(":empty")) {
				error_note.css("visibility", "hidden");
				generateBtn.attr("disabled", false);
				generateBtn.removeClass("btnDisabled");
				error_note.append("<span id='removeMe'>&nbsp;</span>");
			}
			if (libraryFlag) {
				checkall == librariesAll ? librariesFlag = true : locationsFlag = true;
			}
		});
		checkall.bind('unCheckedState', function() {
			boxes.each(function(index) {
				unCheckCb($(this));
			});
			if (libraryFlag) {
				checkall == librariesAll ? librariesFlag = false : locationsFlag = false;
				if (librariesFlag || locationsFlag) {
					return;
				}
			}
			error_note.children("*").css("display", "none");
			$("#removeMe").remove();
			error_note.prepend("<span class='paintred " + errorClass + "'>" + note + "</span>");
			error_note.css("visibility", "visible");
			generateBtn.attr("disabled", true);
			generateBtn.addClass("btnDisabled");
		});

		// check 'all' if all are checked
		boxes.each(function() {
			$(this).bind('checkedState', function() {
				var all = true;
				if (checkall.attr('id') == 'mediaTypeAll') {
					all = false;
				}
				boxes.each(function() {
					if ($(this).attr("data-state") == "unchecked") {
						all = false;
						return false;
					}
				});
				if (all) {
					if (checkall.attr("data-state") == "unchecked")
						checkCb(checkall);
				}

				$("." + errorClass).remove();
				error_note.children(":first").css("display", "block");
				if (error_note.is(":empty")) {
					error_note.css("visibility", "hidden");
					generateBtn.attr("disabled", false);
					generateBtn.removeClass("btnDisabled");
					error_note.append("<span id='removeMe'>&nbsp;</span>");
				}
				if (libraryFlag) {
					checkall == librariesAll ? librariesFlag = true : locationsFlag = true;
				}
			});
			$(this).bind('unCheckedState',
					function() {
						if (checkall.attr("data-state") == "checked") {
							unCheckCb(checkall);
							if ($('#extended_retention_only').hasClass('checked') &&
									$('#basic_retention_only').hasClass('checked')) {
								unCheckCb($('#basic_retention_only'));
							}
						}
						var all = true;
						boxes.each(function() {
							if ($(this).attr("data-state") == "checked") {
								all = false;
								return false;
							}
						});
						if (libraryFlag) {
							checkall == librariesAll ? librariesFlag = false : locationsFlag = false;
							if (librariesFlag || locationsFlag) {
								return;
							}
						}
						if (all) {
							error_note.children("*").css("display", "none");
							$("#removeMe").remove();
							error_note.prepend("<span class='paintred " + errorClass + "'>" + note + "</span>");
							error_note.css("visibility", "visible");
							generateBtn.attr("disabled", true);
							generateBtn.addClass("btnDisabled");
						}
					});
		});
	}

	report.initComponents = function() {

		error_note = $("#error_label");
		generateBtn = $('#btngenerate');

		minimumSelectionCheck($("#mediaStatusAll"),
				$(".mediaStatus"),
				localMsg.storageInformation.statusError,
				"errMsgStat");
		minimumSelectionCheck($("#mediaGroupsAll"),
				$(".mediaGroup"),
				localMsg.storageInformation.groupError,
				"errMsgGroup");

		$('#extended_retention_only').bind('checkedState', function() {
			if (!($('#mediaTypeAll').hasClass('checked'))) {
				unCheckCb($('#basic_retention_only'));
			}
		});
		$('#basic_retention_only').bind('checkedState', function() {
			if (!($('#mediaTypeAll').hasClass('checked'))) {
				unCheckCb($('#extended_retention_only'));
			}
		});

		$("#mediaDetail h1 div.arrowPlaceholder").click();
		$("#mediaDetail h1 div.arrowPlaceholder").click(function() {
			if (!detailsClicked) {
				$("#mediaDetail").mask();
				cvUtil.loadPageWithGET(getDetailsUrl(), "", function(data1) {
					updateDetailsTable(data1);
					$("#mediaDetail").unmask();
					detailsClicked = true;

				});
			}
		});

		var copyTypes = $(".copyType");
		var none = false;
		var errorCopyClass = "errMsgCopy";
		copyTypes.each(function() {
			$(this).bind('checkedState', function() {
				none = false;
				$("." + errorCopyClass).remove();
				error_note.children(":first").css("display", "block");

				if (error_note.is(":empty")) {
					error_note.css("visibility", "hidden");
					generateBtn.attr("disabled", false);
					generateBtn.removeClass("btnDisabled");
					error_note.append("<span id='removeMe'>&nbsp;</span>");
				}
			});
			$(this).bind('unCheckedState',
					function() {
						none = true;
						copyTypes.each(function() {
							if ($(this).attr("data-state") == "checked") {
								none = false;
								return false;
							}
						});
						if (none) {
							error_note.children("*").css("display", "none");
							$("#removeMe").remove();
							error_note.prepend("<span class='paintred " + errorCopyClass + "'>" +
									localMsg.storageInformation.copyTypeError + "</span>");
							error_note.css("visibility", "visible");
							generateBtn.attr("disabled", true);
							generateBtn.addClass("btnDisabled");
						}
					});
		});

		librariesFilterPanel = new EntitySelection({
			id : "libraryList"
		});
		locationsFilterPanel = new EntitySelection({
			id : "locationList"
		});
		policiesFilterPanel = new EntitySelection({
			id : "policyList"
		});

		var treeReqURL = cvUtil.getContextPath() + '/server/report?type=storageinformation&json=' +
				window.encodeURIComponent('{"invisible":{"initial":true},"component":"StoragePoliciesTree"}');

		cvUtil.loadPageWithGET(treeReqURL,
				"",
				function(data) {

					var tapeLibAvailable = data["tapeLibAvailable"];

					if (data["libraries"] != null) {
						librariesFilter = data["libraries"];
						librariesFilterPanel.updateList(getLibraries(data));
					}
					if (data["locations"] != null)
						locationsFilterPanel.updateList(data["locations"]);
					if (data["spTree"] != null)
						policiesFilterPanel.updateList(data["spTree"]);

					if (report.requestString.libraryFilter != null)
						librariesFilterPanel.setSelectedList(report.requestString.libraryFilter);

					if (report.requestString.locationFilter != null)
						locationsFilterPanel.setSelectedList(report.requestString.locationFilter);

					if (report.requestString.copyFilter != null)
						policiesFilterPanel.setSelectedList(report.requestString.copyFilter);

					if (tapeLibAvailable == false) {
						$("#libTabs").hide();
						$('#tabs').tabs('destroy');
						$('#mediaLibraries').removeClass('ui-tabs-panel');
						$('#mediaLibraries').removeClass('ui-widget-content');
						$('#mediaLibraries').removeClass('ui-corner-bottom');

						$("#locOption").hide();
						$("#librariesSubTitle").hide();
						$("#libraryAndLocTitle a").html(localMsg.storageInformation.libraries);
					}

					var policyAll = $("#policyList_AllCheckBox"), policyBoxes = $(".policyList");

					minimumSelectionCheck(policyAll,
							policyBoxes,
							localMsg.storageInformation.ErrorNotePolicies,
							"errMsgPolicy");

					/*
					 * By default we wont have a case where both of these are unchecked
					 */
					librariesFlag = true, locationsFlag = true;

					librariesAll = $("#libraryList_AllCheckBox"), librariesBoxes = $(".libraryList");
					if (tapeLibAvailable == true) {
						locationsAll = $("#locationList_AllCheckBox"), locationsBoxes = $(".locationList");

						minimumSelectionCheck(librariesAll,
								librariesBoxes,
								localMsg.storageInformation.ErrorNoteLibOrLoc,
								"errMsg",
								true);
						minimumSelectionCheck(locationsAll,
								locationsBoxes,
								localMsg.storageInformation.ErrorNoteLibOrLoc,
								"errMsg",
								true);
					} else {
						minimumSelectionCheck(librariesAll,
								librariesBoxes,
								localMsg.storageInformation.ErrorNoteLib,
								"errMsg");
					}

				});
		$('.tabs').tabs();
		$("#filterPanel").accordion({
			fillSpace : true
		});

		chartOtherDrillIndex = 0;

		$(".exportatserver").click(function() {
			report.mask(report.idtomask, true, localMsg.exporting);
			report.requestString.invisible.exportType = this.id.substring("export".length);
			report.requestString.invisible.exportAll = true;
			report.isExport = true;
			if (report.getChartSVGs)
				report.getChartSVGs(report.chartIds);
			cvUtil.loadPageWithPOST(cvUtil.getContextPath() + "/server/report", getRequestUrlForExport(), function(
					data1) {
				$("#fileNameToDownload").val(data1);
				$("#exportForm").submit();
				report.unmask(report.idtomask);
			});
			report.requestString.invisible.exportType = undefined;
			requestString.invisible.exportAll = false;
			requestString.invisible.chartsSVGs = null;
		});
		$("#groupByPanel").customSelect(0);
	};

	function resetEveything() {
		$("#MediaType").attr("selected", "selected");
		report.requestString.invisible.drilldown = false;
		resetDetailsTable();
	}

	function resetDetailsTable() {
		detailsClicked = false;

		$('#mediaDetail_table > tbody').empty();
		$('#mediaDetail_table_info').empty();
		$('#mediaDetail > h1').children(".arrowPlaceholder").removeClass("expand").addClass("collapse");
		$('#mediaDetail_table_wrapper').hide('fast');
		$('#mediaDetail_holder').css("display", "none");
		$('#mediaDetail_Filter').css("display", "none");

		if (report.requestString.mediaDetail) {
			delete report.requestString.mediaDetail;
		}
	}

	report.initComponentState = function() {

		$.each(report.requestString.mediaStatus, function(index, value) {
			if (value == 'all') {
				checkCb($('#mediaStatusAll'));
				$('#mediaStatusAll').trigger('checkedState');
			} else {
				checkCb($('#' + value));
			}
		});

		$.each(report.requestString.mediaGroup, function(index, value) {
			if (value == 'all') {
				checkCb($('#mediaGroupsAll'));
				$('#mediaGroupsAll').trigger('checkedState');
			} else {
				checkCb($('#' + value));
			}
		});

		$.each(report.requestString.mediaType, function(index, value) {
			if (value != 'all') {
				checkCb($('#' + value));
			}
		});

		if (report.requestString.retention != undefined) {
			checkCb($('#' + report.requestString.retention));
		}

		if (report.requestString.mediaState == 'good') {
			$('#mediaStateGood').prop("checked", true);
		} else if (report.requestString.mediaState == 'bad') {
			$('#mediaStateBad').prop("checked", true);
		} else {
			$('#mediaStateAll').prop("checked", true);
		}

		$('#andExistsIn').prop("checked", true);

		if (report.requestString.exportedOption) {
			if (report.requestString.exportedOption == 'ALL') {
				checkCb($('#locIn'));
				checkCb($('#locOut'));
			} else if (report.requestString.exportedOption == 'IN_LIBRARY_ONLY') {
				checkCb($('#locIn'));
			} else if (report.requestString.exportedOption == 'EXPORTED_ONLY') {
				checkCb($('#locOut'));
			}
		}
		var dist = report.requestString.distribution;
		if (dist == "StorageCapacity") {
			$('#groupByPanel :nth-child(2)').click();
		} else if (dist == "Library") {
			$('#groupByPanel :nth-child(3)').click();
		} else if (dist == "StoragePolicy") {
			$('#groupByPanel :nth-child(4)').click();
		} else if (dist == "MediaType") {
			$('#groupByPanel :nth-child(1)').click();
		} else {
			$('#groupByPanel :nth-child(1)').click();
		}

		$.each(report.requestString.copyType, function(index, value) {
			if (value == 'all') {
				$('.copyType').each(function() {
					checkCb($(this));
				});
			} else {
				checkCb($('#' + value));
			}
		});

	};

	report.beforeLoadReport = function() {
		$("#chartPanel > .showifnodata").hide();
		$("#chartPanel > .hideifnodata").show();
		$("#groupByPanel").removeAttr("disabled");
		$("#piechartcontainer").mask("");
	};

	report.onLoadReport = function(data) {
		chartData = data["chart"]["points"];

		var rows = 0;
		var rowsTotal = 0;
		while (rows < chartData.length) {
			rowsTotal += chartData[rows].y;
			rows++;
		}

		if (rowsTotal <= 0) {
			$("#chartPanel > .showifnodata").show();
			$("#chartPanel > .hideifnodata").hide();
			$("#groupByPanel").attr("disabled", "disabled");
		} else {
			$("#chartPanel > .showifnodata").hide();
			$("#chartPanel > .hideifnodata").show();
			$("#groupByPanel").removeAttr("disabled");

			chartData.sort(sorter);
			var currentData = [];
			var i = 0;
			var maxSlice = MAX_NUM_PIE_SLICE - 1;
			if (chartData.length > MAX_NUM_PIE_SLICE) {
				for (i = 0; i < chartData.length && i < maxSlice; i++)
					currentData.push(chartData[i]);

				var total = 0;
				while (i++ < chartData.length - 1)
					total += chartData[i].y;

				currentData.push({
					"name" : "Others",
					"y" : total,
					"id" : -1
				});
			} else {
				currentData = chartData;
			}

			createRaphaelchart(currentData);
		}
		$("#chartTitle").html(data.chart.chartTitle);

		Reports.configureClientSideTable('mediaSummary', localMsg.summary, data.mediaSummary);

		updateBreadCrumb();
		$("#piechartcontainer").unmask();
	};

	function sorter(p1, p2) {
		return p2.y - p1.y;
	}

	report.updateRequestString = function() {
		updateReportOptions();
	};

	report.init();

	report.customizeTable = function(dataTable) {

		var oSettings = dataTable.fnSettings();
		var freeSpaceCol;
		var freeSpaceIndex = 0;
		for (var iColumn = 0; iColumn < oSettings.aoColumns.length; iColumn++) {
			if (oSettings.aoColumns[iColumn].mDataProp == "availableCapacity") {
				freeSpaceCol = oSettings.aoColumns[iColumn];
				freeSpaceIndex = iColumn;
				break;
			}
		}
		if (freeSpaceCol == null || freeSpaceCol.bVisible == false) {
			return;
		}

		var data = oSettings.aoData;
		var htmlRows = $("tr", dataTable);
		for (var i = 0; i < data.length; i++) {
			var rawData = data[i]._aData;
			var library = rawData.libraryName;
			var Type = getLibraryType(library);
			if (Type != "CLOUD") {
				var freeSpace = parseInt(rawData.availableCapacity);
				var usedSpace = parseInt(rawData.usedCapacity);
				if (freeSpace < 0) {
					freeSpace = 0;
				}
				var percent = roundNumber((freeSpace / (freeSpace + usedSpace)) * 100, 2);
				if (percent < 10) {
					for (var j = 1; j < htmlRows.length; j++) {
						var htmlCols = $("td", htmlRows[j]);
						var entityVal = $(htmlCols[0]).text();//Getting entity name
						if (entityVal == library) {
							var colFirst = htmlCols[0];
							var colFree = htmlCols[freeSpaceIndex];
							//col.style.background = criticalColor;
							colFirst.style.borderLeft = '4px solid red';
							//colFree.style.textShadow = '3px 3px 6px red';
							colFree.style.boxShadow = '0px 0px 5px red inset';
							colFree.title = localMsg.storageInformation.LibraryFreeSpaceTooltip;
							break;
						}
					}
				}
			}
		}
	};
	report.saveFilters = function() {
		savedFilters = {
			"distribution" : report.requestString["distribution"],
			"groupBy" : report.requestString["groupBy"],
			"reportFilters" : report.requestString.reportFilters
		};
	};
	report.updateFilters = function() {
		if (savedFilters != null) {
			report.requestString["distribution"] = savedFilters.distribution;
			report.requestString["groupBy"] = savedFilters.groupBy;
			report.requestString.reportFilters = savedFilters.reportFilters;
			savedFilters = null; // resetting
		}
	};

	this.breadCrumbClick = function(index) {
		report.requestString.invisible.drilldown = true;
		resetDetailsTable();
		if (index == 0) {
			if (this.requestString.reportFilters == null || report.requestString.reportFilters.length == 0) {
				return;
			}
			report.requestString.reportFilters = [];
			report.requestString.groupBy = report.requestString.distribution;
		} else if (report.requestString.reportFilters != null) {
			if (index != report.requestString.reportFilters.length) {
				report.requestString.groupBy = report.requestString.reportFilters[index].entityType;
				var i = 0;
				for (i = report.requestString.reportFilters.length - 1; i >= index; i--)
					report.requestString.reportFilters.pop();
			}
		}
		chartOtherDrillIndex = 0;
		report.requestString.othersLevel = 0;
		report.processReport();

	};

	$("#groupByPanel").on("customSelectionChanged", function(e, el) {
		handleGroupByChange($(el).attr("id"));
	});

	function handleGroupByChange(distribution) {
		chartDrillDown(0, "", "", distribution);
	}

	function chartDrillDown(entityId, entityName, type, distribution) {
		report.requestString.invisible.drilldown = true;
		resetDetailsTable();
		if (distribution != null) {
			report.requestString.reportFilters = [];
			report.requestString.groupBy = distribution;
			report.requestString.distribution = distribution;
			report.requestString.othersLevel = 0;
			chartData = null;
			chartOtherDrillIndex = 0;
			report.processReport();
		} else if (entityId == -1) {
			chartOtherDrillIndex++;
			var currentData = [];
			var seriesCount = 0;
			var startIndex = chartOtherDrillIndex * MAX_NUM_PIE_SLICE;

			for (; startIndex < chartData.length && seriesCount < MAX_NUM_PIE_SLICE - 1; startIndex++, seriesCount++) {
				currentData.push(chartData[startIndex]);
			}

			if (startIndex < chartData.length - 1) {
				var total = 0;
				while (startIndex++ < chartData.length - 1)
					total += chartData[startIndex].y;

				currentData.push({
					"name" : localMsg.others,
					"y" : total,
					"id" : -1
				});
			}
			createRaphaelchart(currentData);
		} else {
			chartOtherDrillIndex = 0;
			report.requestString.othersLevel = 0;

			var entity = {};
			//entity.entityId = entityId;
			entity.entityName = entityName;
			entity.entityType = type;
			entity.order = report.requestString.reportFilters.length;
			report.requestString.reportFilters.push(entity);

			//set the new groupBy
			if (report.requestString.distribution == "StorageCapacity") {
				if (report.requestString.groupBy == "StorageCapacity")
					report.requestString.groupBy = "MediaType";
				else if (report.requestString.groupBy == "MediaType")
					report.requestString.groupBy = "Library";
				else if (report.requestString.groupBy == "Library")
					report.requestString.groupBy = "StoragePolicy";
				else if (report.requestString.groupBy == "StoragePolicy")
					report.requestString.groupBy = "StoragePolicyCopy";
			} else if (report.requestString.distribution == "Library") {
				if (report.requestString.groupBy == "Library")
					report.requestString.groupBy = "StorageCapacity";
				else if (report.requestString.groupBy == "StorageCapacity")
					report.requestString.groupBy = "StoragePolicy";
				else if (report.requestString.groupBy == "StoragePolicy") {
					report.requestString.groupBy = "StoragePolicyCopy";
				}
			} else if (report.requestString.distribution == "StoragePolicy") {
				if (report.requestString.groupBy == "StoragePolicy")
					report.requestString.groupBy = "StoragePolicyCopy";
				else if (report.requestString.groupBy == "StoragePolicyCopy")
					report.requestString.groupBy = "Library";
			} else if (report.requestString.distribution == "MediaType") {
				if (report.requestString.groupBy == "MediaType")
					report.requestString.groupBy = "StorageCapacity";
				else if (report.requestString.groupBy == "StorageCapacity")
					report.requestString.groupBy = "Library";
				else if (report.requestString.groupBy == "Library")
					report.requestString.groupBy = "StoragePolicy";
				else if (report.requestString.groupBy == "StoragePolicy")
					report.requestString.groupBy = "StoragePolicyCopy";
			}
			report.processReport();
		}

	}

	function updateBreadCrumb() {
		//clear the breadcrumb
		$("#breadCrumbPanel").empty();

		var counter = 0;
		var id = 'path' + counter;

		$("#breadCrumbPanel").append('<li class="breadcrumb" id=' + id + '>' + localMsg.storageInformation.capacity +
				'</li>');
		$("#" + id).click(function() {
			breadCrumbClick(this.id.substring('path'.length));
		});
		counter++;

		if (requestString.reportFilters != null) {
			$.each(requestString.reportFilters, function(index, value) {
				id = 'path' + counter;
				$("#breadCrumbPanel").append('<li class="breadcrumb" id=' + id + '>' + decodeURI(value.entityName) +
						'</li>');
				$("#" + id).click(function() {
					breadCrumbClick(this.id.substring('path'.length));
				});
				counter++;
			});

		}

		$('#breadCrumbPanel li').last().css('background-image', 'none');
		$("#breadCrumbPanel").append('<li class="endBreadcrumb"></li>');
	}

	function createRaphaelchart(currentData) {
		$("#piechartcontainer").empty();
		var values = $.map(currentData, function(node) {
			return node["y"] < 0 ? 0 : node["y"];
		});
		currentData.values = values;
		labels = $.map(currentData, function(node) {
			return node["name"] + " - " + autoConvert(node["y"] * 1024 * 1024);
		});

		toolTipLabels = $.map(currentData, function(node) {
			if (node["toolTip"]) {
				return node["toolTip"];
			}
			return node["name"] + " - " + autoConvert(node["y"] * 1024 * 1024);
		});

		var events = {};
		events.clickFtn = [];
		var enableDrill = true;
		if (report.requestString.distribution == "StorageCapacity") {
			if (report.requestString.groupBy == "Library")
				enableDrill = report.requestString.reportFilters[0].entityName == 'Used' ? true : false;
			else if (report.requestString.groupBy == "StoragePolicy")
				enableDrill = report.requestString.reportFilters[0].entityName == 'Used' ? true : false;
			else if (report.requestString.groupBy == "StoragePolicyCopy")
				enableDrill = false;
		} else if (report.requestString.distribution == "Library") {
			if (report.requestString.groupBy == "StoragePolicy") {
				enableDrill = report.requestString.reportFilters[1].entityName == 'Used' ? true : false;
			} else if (report.requestString.groupBy == "StoragePolicyCopy") {
				enableDrill = false;
			}
		} else if (report.requestString.distribution == "StoragePolicy") {
			if (report.requestString.groupBy == "Library") {
				enableDrill = false;
			}
		} else if (report.requestString.distribution == "MediaType") {
			if (report.requestString.groupBy == "Library")
				enableDrill = report.requestString.reportFilters[1].entityName == 'Used' ? true : false;
			else if (report.requestString.groupBy == "StoragePolicy")
				enableDrill = report.requestString.reportFilters[1].entityName == 'Used' ? true : false;
			else if (report.requestString.groupBy == "StoragePolicyCopy")
				enableDrill = false;
		}

		for (var i = 0; i < currentData.length; i++) {
			if (enableDrill || currentData[i].name == 'Others') {
				events.clickFtn[i] = function() {
					chartDrillDown(this.params.id, this.params.name, this.params.type, null);
				};
			}
		}

		hideFilterPanels(report.requestString.distribution, report.requestString.reportFilters);
		Raphael("piechartcontainer", 700, 300).pieChart(250,
				160,
				180,
				currentData,
				labels,
				500,
				40,
				"#fff",
				events,
				null,
				null,
				null,
				toolTipLabels);
		initialRequest = "FALSE";
	}

	function updateReportOptions() {
		resetEveything();
		updateGeneral();

		librariesFilterPanel.populateRequestString("libraryFilter");
		locationsFilterPanel.populateRequestString("locationFilter");
		policiesFilterPanel.populateRequestString("copyFilter");

		report.requestString.copyType = new Array();
		var all = true;
		$(".copyType").each(function() {
			if ($(this).attr("data-state") == "unchecked") {
				all = false;
			}
		});

		if (all) {
			report.requestString.copyType.push("all");
		} else {
			$(".copyType").each(function() {
				if ($(this).attr('data-state') == 'checked')
					report.requestString.copyType.push($(this).attr('id'));
			});
		}
	}

	function updateGeneral() {

		requestString.mediaStatus = [];
		if ($('#mediaStatusAll').hasClass('checked')) {
			report.requestString.mediaStatus.push('all');
		} else {
			$.each($(".mediaStatus"), function() {
				if ($(this).hasClass('checked'))
					report.requestString.mediaStatus.push($(this).attr('id'));
			});
		}

		var andOrexists = $('#andExistsIn');
		if (andOrexists.attr("checked") != "undefined" && andOrexists.attr("checked") == "checked") {
			report.requestString.existsIn = 'and';
		} else {
			report.requestString.existsIn = 'or';
		}

		requestString.mediaGroup = [];

		if ($('#mediaGroupsAll').hasClass('checked')) {
			requestString.mediaGroup.push('all');
		} else {
			$.each($(".mediaGroup"), function() {
				if ($(this).hasClass('checked'))
					report.requestString.mediaGroup.push($(this).attr('id'));
			});
		}
		requestString.mediaType = [];
		$.each($(".mediaType"), function() {
			if ($(this).hasClass('checked'))
				report.requestString.mediaType.push($(this).attr('id'));
		});
		if (requestString.mediaType.length == 0) {
			report.requestString.mediaType.push("all");
		}

		if ($('#extended_retention_only').hasClass('checked')) {
			requestString.retention = 'extended_retention_only';
		} else if ($('#basic_retention_only').hasClass('checked')) {
			requestString.retention = 'basic_retention_only';
		} else if (requestString.retention) {
			delete requestString.retention;
		}

		var mediaStateAll = $('#mediaStateAll');
		if (mediaStateAll.attr("checked") != "undefined" && mediaStateAll.attr("checked") == "checked") {
			report.requestString.mediaState = 'all';
		} else {
			var mediaStateGood = $('#mediaStateGood');
			if (mediaStateGood.attr("checked") != "undefined" && mediaStateGood.attr("checked") == "checked") {
				report.requestString.mediaState = 'good';
			} else {
				report.requestString.mediaState = 'bad';
			}
		}

	}

	function updateDetailsTable(data) {
		Reports.configureTable('mediaDetail',
				localMsg.storageInformation.mediaDetails,
				data.mediaDetail,
				report.requestString,
				getDetailsUrl);
	}

	function getDetailsUrl() {
		this.requestString.mode = 4; // WR_REquestModeFlag.MEDIA_LIST
		var url = cvUtil.getContextPath() + '/server/report?type=' + report.type + '&json=' +
				window.encodeURIComponent(JSON.stringify(this.requestString));
		this.requestString.mode = 1; // resetting to SUMMARY
		return url;
	}

	function getRequestUrlForExport() {
		this.requestString.mode = 1 + 4; // WR_REquestModeFlag.SUMMARY + WR_REquestModeFlag.MEDIA_LIST
		var url = {
			type : report.type,
			json : JSON.stringify(this.requestString)
		};
		this.requestString.mode = 1; // resetting to SUMMARY
		return url;
	}

	report.defaultSortColumn = function() {
		return 'barcode';
	};
}

function hideFilterPanels(distribution, reportFilters) {
	if (distribution == "MediaType") {
		if (reportFilters != "") {
			updateLib(reportFilters[0].entityName.toUpperCase());
			updateFilterPanel(reportFilters[0].entityName.toUpperCase());
		} else {
			updateLib("ALL");
			updateFilterPanel("ALL");
		}
	} else if (distribution == "StorageCapacity") {
		if (reportFilters.length > 1) {
			updateLib(reportFilters[1].entityName.toUpperCase());
			updateFilterPanel(reportFilters[1].entityName.toUpperCase());
		} else {
			updateLib("ALL");
			updateFilterPanel("ALL");
		}
	} else if (distribution == "Library") {
		if (reportFilters != "") {
			var type = getLibraryType(reportFilters[0].entityName);
			updateFilterPanel(type);
		} else {
			updateLib("ALL");
			updateFilterPanel("ALL");
		}
	} else if (distribution == "StoragePolicy") {
		showStoragePolicy();
	} else {
		showAllFilters();
	}
}

function showStoragePolicy() {

	$('#filter-panel-cont').show();
	$("#filterPanel").accordion('destroy');
	$("#storagePolicyTitle").show();
	$("#storagePoliciesDiv").show();
	$("#mediaStatusTitle").hide();
	$("#mediaStatusDiv").hide();
	$("#mediaGroupsTitle").hide();
	$("#mediaGroupsDiv").hide();
	$("#advancedTitle").hide();
	$("#mediaTypesDiv").hide();
	$("#libraryAndLocTitle").hide();
	$("#libAndLocDiv").hide();
	$("#filterPanel").accordion();
	$("#filterPanel").accordion("activate", $("#storagePolicyTitle"));
	$("#storagePoliciesDiv").css('height', '');
}
function showAllFilters() {

	$('#filter-panel-cont').show();

}
function getLibraries(libraries) {
	var libs = new Array();
	for (var i = 0; i < libraries["libraries"].length; i++) {
		libs.push(libraries["libraries"][i].Name);
	}
	return libs;
}
function getLibraryType(library) {
	for (var i = 0; i < librariesFilter.length; i++) {
		if (library == librariesFilter[i].Name)
			return librariesFilter[i].Type;
	}

}

function updateLib(type) {
	if (type == "ALL") {
		for (var i = 0; i < $("#libraryList li").length; i++) {
			$("#libraryList li:eq(" + (i) + ")").show();
		}
	} else {
		for (var i = 0; i < $("#libraryList li").length - 1; i++) {
			if (librariesFilter[i].Type != type) {
				$("#libraryList li:eq(" + (parseInt(i) + 1) + ")").hide();
			}
		}
	}
}

function updateFilterPanel(Type) {
	if (Type != "TAPE" && Type != "ALL") {
		$("#filterPanel").accordion('destroy');
		$("#mediaStatusTitle").hide();
		$("#mediaLocation").hide();
		$("#mediaStatusDiv").hide();
		$("#mediaGroupsTitle").hide();
		$("#mediaGroupsDiv").hide();
		$("#advancedTitle").hide();
		$("#mediaTypesDiv").hide();
		$("#mediaLocation").hide();
		$("#libraryAndLocTitle").show();
		$("#libAndLocDiv").show();
		$("#libTabs li:eq(" + (1) + ")").hide();

		$("#filterPanel").accordion();
	} else {
		$("#filterPanel").accordion('destroy');
		$("#mediaStatusTitle").show();
		$("#mediaLocation").show();
		$("#mediaStatusDiv").show();
		$("#mediaGroupsTitle").show();
		$("#mediaGroupsDiv").show();
		$("#advancedTitle").show();
		$("#mediaTypesDiv").show();
		$("#mediaLocation").show();
		$("#libraryAndLocTitle").show();
		$("#libAndLocDiv").show();
		$("#libTabs li:eq(" + (1) + ")").show();
		$("#filterPanel").accordion();
	}
}

$(document).ready(function() {
	StorageInformationReport();
});
